'use strict';
if (typeof console == "undefined" || typeof console.log == "undefined")
    var console = {
        log : function() {
        }
    };

// /* Error Handling */
Ext.ns('nsWISA');
nsWISA.showError = function(title, message) {
    Ext.Msg.show({
        title : title,
        msg : message,
        icon : Ext.Msg.ERROR,
        buttons : Ext.Msg.OK
    });
};
nsWISA.showAjaxError = function(message) {
    nsWISA.showError('Ajax Error', message);
};
nsWISA.getResponseError = function(provider, response, message) {
    var result = (!Ext.isEmpty(response)) && (Ext.isEmpty(response.error)) && (Ext.isDefined(response.result));
    if (!result) {
        if ((!Ext.isEmpty(response)) && (!Ext.isEmpty(response.error))) {
            message = message + response.error.message;
        } else {
            message = message + 'Interne server fout.';
        }
        nsWISA.showAjaxError(message);
    }
    return result;
};

// /* Server Configuration */
Ext.ns('nsWISA');
nsWISA.getServerConfiguration = function(options) {
    var urlProvider = Application.getUrlApplication() + '/Provider/' + options.ProviderName + '/';
    var objHttpProxy = new Ext.data.HttpProxy({
        api : {
            read : urlProvider + 'Read/',
            update : urlProvider + 'Update/',
            create : urlProvider + 'Insert/',
            destroy : urlProvider + 'Delete/'
        }
    });
    // We expect metadata from the server.
    var objJsonReader = new Ext.data.JsonReader({
        root : 'rows',
        messageProperty : 'message',
        successProperty : 'success'
    });
    var objJsonWriter = new Ext.data.JsonWriter({
        encode : true,
        writeAllFields : true
    });
    return {
        reader : objJsonReader,
        writer : objJsonWriter,
        proxy : objHttpProxy
    };
};

// /* Basic Components */
Ext.ns('nsWISA.ux');
/**
 * @class nsWISA.ux.BaseClientStore
 * @extends Ext.Datastore
 * @information The basic data store for all WISA applications. <br />
 *              Config is an object that must contain
 *              <ul>
 *              <li>ProviderName : String with name of provider.</li>
 *              <li>Fields: List of fieldnames to expect.</li>
 *              <li>IDField : String with name of ID field of data.</li>
 *              </ul>
 * @author Michael Van Canneyt
 * @version 0.9
 */
nsWISA.ux.BaseClientStore = Ext.extend(Ext.data.Store, {
    // Constructor
    constructor : function(options) {
        var optionsServer = nsWISA.getServerConfiguration(options);
        // Merge data
        Ext.apply(options, optionsServer);
        nsWISA.ux.BaseClientStore.superclass.constructor.call(this, options);
    }
});

/**
 * @class nsWISA.ux.BaseClientGroupingStore
 * @extends Ext.Data.GroupingStore
 * @information The basic data store for all WISA applications. <br />
 *              Config is an object that must contain
 *              <ul>
 *              <li>ProviderName : String with name of provider.</li>
 *              <li>Fields: List of fieldnames to expect.</li>
 *              <li>IDField : String with name of ID field of data.</li>
 *              </ul>
 * 
 * @author Michael Van Canneyt
 * @version 0.9
 */
nsWISA.ux.BaseClientGroupingStore = Ext.extend(Ext.data.GroupingStore, {
    // Constructor
    constructor : function(options) {
        var optionsServer = nsWISA.getServerConfiguration(options);
        // Merge data
        Ext.apply(options, optionsServer);
        nsWISA.ux.BaseClientGroupingStore.superclass.constructor.call(this, options);
    }
});

/**
 * @class nsWISA.ux.BaseDialog
 * @information The basic dialog for all WISA applications.
 * @extends Ext.Window
 * @author Michael Van Canneyt
 * @version 0.9
 */
nsWISA.ux.BaseDialog = Ext.extend(Ext.Window, {
    buildForm : function() {
        return null;
    },
    applyParams : function(params) {
        // Apply any params you may see fit
    },
    initComponent : function() {
        var options = this.BuildForm(); // TODO Compatibility Rename
        if (!Ext.isEmpty(options)) {
            Ext.apply(this, options);
        }
        nsWISA.ux.BaseDialog.superclass.initComponent.call(this);
    },
    // TODO Compatibility Artifact Below
    BuildForm : function() {
        return this.buildForm();
    },
    ApplyParams : function(params) {
        return this.applyParams(params);
    }
});

/**
 * @class nsWISA.ux.BaseGrid
 * @information The basic grid for all WISA applications.
 * @extends Ext.grid.GridPanel
 * @author Michael Van Canneyt
 * @version 0.9
 */
nsWISA.ux.BaseGrid = Ext.extend(Ext.grid.GridPanel, {});

/**
 * @class nsWISA.ux.BaseForm
 * @information The basic form for all WISA applications.
 * @extends Ext.Panel
 * @author Michael Van Canneyt
 * @version 0.9
 */
nsWISA.ux.BaseForm = Ext.extend(Ext.Panel, {
    /**
     * @method BuildForm
     * @information The BuildForm method creates the actual GUI. This method must be overridden and must build the FORM
     *              gui. It cannot perform any server calls.
     * @author Michael Van Canneyt
     */
    buildForm : function() {
        return null;
    },
    applyParams : function(params) {
        // Apply any params you may see fit
    },
    getFormTitle : function() {
        return 'Onbekend scherm';
    },
    initComponent : function() {
        var options = this.BuildForm(); // TODO Compatibility Rename
        if (!Ext.isEmpty(options)) {
            Ext.apply(this, options);
        }
        nsWISA.ux.BaseForm.superclass.initComponent.call(this);
    },
    // TODO Compatibility Artifact Below
    BuildForm : function() {
        return this.buildForm();
    },
    ApplyParams : function(params) {
        return this.applyParams(params);
    }
});

// /* Application */
Ext.ns('nsWISA');
Ext.ns('nsWISA.nsForms');
/**
 * @class nsWISA.ux.BaseApplication
 * @information De WISA Application class: er wordt slechts 1 instance van deze class gemaakt.
 * @extends Ext.util.Observable
 * @author Michael Van Canneyt
 * @version 0.9
 */
nsWISA.ux.BaseApplication = Ext.extend(Ext.util.Observable, (function() {
    // /*** Private Static Properties [ var = ] ***/
    // /*** Private Static Methods [ var = ] ***/
    return {
        constructor : function(options) {
            // /*** Below has Access to All Properties/Methods ***/
            // /*** Public Properties [ this.prop = ] ***/
            // none

            // /*** Private Properties [ var prop = ] ***/
            var isAPILoaded = false;
            var isCGI = false;
            var isThemeComboVisible = false;
            var isMainmenuVisible = true;
            var isSubmenuVisible = true;
            var urlBase = '.';
            var urlApplication = '';
            var userId = -1;
            var userName = '';
            var userLastlogin = '(onbekend)';
            var menu = null;
            var submenuPanel = null;
            var submenuConfig = {
                region : 'west',
		cls: 'wisa-menu',
                collapsible : true,
                collapsed : true,
                layout : 'fit',
                title : 'Menu',
                margins : '0 0 3 3',
		cmargins: '0 0 3 3',
                width : 200
            };
            var FMainMenuConfig = null;
            var bottomPanelConfig = null;
            var formMain = 'motd';
            var formParent = null;
            var dialogActive = null;

            // /*** Public Methods [ this.fct = ] ***/ // & //
            // /*** Private Methods [ var fct = ] ***/
            // // Setters and Getters
            var setIsAPILoaded = function(value) {
                isAPILoaded = value;
            };
            var getIsAPILoaded = function() {
                return isAPILoaded;
            };
            var setIsCGI = function(value) {
                isCGI = value;
            };
            var getIsCGI = function() {
                return isCGI;
            };
            var setIsThemeComboVisible = function(value) {
                isThemeComboVisible = value;
            };
            var getIsThemeComboVisible = function() {
                return isThemeComboVisible;
            };
            var setIsMainmenuVisible = function(value) {
                isMainmenuVisible = value;
            };
            var getIsMainmenuVisible = function() {
                return isMainmenuVisible;
            };
            var setMainMenuConfig = function (value) { 
                FMainMenuConfig = Ext.apply(value,{
                    region: 'north', 
                    collapsible: true
                }); 
            };
            this.getMainMenuConfig = function () { 
                return FMainMenuConfig;
            };
            var setIsSubmenuVisible = function(value) {
                isSubmenuVisible = value;
            };
            var getIsSubmenuVisible = function() {
                return isSubmenuVisible;
            };
            var setUrlBase = function(value) {
                urlBase = value;
            };
            var getUrlBase = function() {
                return urlBase;
            };
            var setUrlApplication = function(value) {
                urlApplication = value;
            };
            this.getUrlApplication = function() {
                if (urlApplication !== '') {
                    return urlApplication;
                } else {
                    if (getIsCGI()) {
                        urlApplication = getUrlBase() + '/server.cgi';
                    } else {
                        urlApplication = getUrlBase() + '/server';
                    }
                    return urlApplication;
                }
            };
            this.getUrlAPI = function() {
                return (this.getUrlApplication() + '\/RPC\/API\/');
            };
            this.getUrlDialogs = function() {
                return (this.getUrlApplication() + '\/' + 'Forms\/');
            };
            this.getUrlForms = function() {
                return (this.getUrlApplication() + '\/' + 'Forms\/');
            };
            var setUserData = function(data) {
                userId = data.userid;
                userName = data.name;
                userLastlogin = data.lastlogin;
            };
            this.getUserId = function() {
                return userId;
            };
            this.getUserName = function() {
                return userName;
            };
            this.getUserLastLogin = function() {
                return userLastlogin;
            };
            var setMenu = function(value) {
                menu = value;
            };
            this.getMenu = function() {
                return menu;
            };
            var setSubmenuPanel = function(value) {
                submenuPanel = value;
            };
            this.getSubmenuPanel = function() {
                return submenuPanel;
            };
            var setSubmenuConfig = function(options) {
                submenuConfig = Ext.apply(options, {
                    region : 'west',
                    collapsible : true
                });
            };
            var getSubmenuConfig = function() {
                return submenuConfig;
            };
            var setBottomPanelConfig = function (value) { 
                 bottomPanelConfig = Ext.apply(AValue,{
                     region: 'south', 
                     collapsible: false
                 });
            };
            this.getBottomPanelConfig = function () { 
                return bottomPanelConfig;
            };
            var setFormMain = function(value) {
                formMain = value;
            };
            this.getFormMain = function() {
                return formMain;
            };
            var setFormParent = function(value) {
                formParent = value;
            };
            var getFormParent = function() {
                return formParent;
            };
            var setDialogActive = function(object) {
                dialogActive = object;
            };
            var getDialogActive = function() {
                return dialogActive;
            };
            // // API
            var checkIsAPILoaded = function() {
                if (true !== getIsAPILoaded()) {
                    return false;
                } else {
                    return true;
                }
            };
            var setLoadAPISuccess = function(response, options) {
                try {
                    eval(response.responseText);
                } catch (err) {
                    nsWISA.showAjaxError('AJAX error during API Load: ' + err.message);
                }
                Ext.Direct.addProvider(WISA.API);
                setIsAPILoaded(true);
                this.fireEvent('APILoaded');
            };
            var setLoadAPIFailure = function() {
                window.alert('WISA API could not be loaded');
            };
            this.loadAPI = function() {
                Ext.Ajax.request({
                    url : this.getUrlAPI(),
                    success : setLoadAPISuccess,
                    failure : setLoadAPIFailure,
                    scope : this
                });
            };
            this.buildMenu = function() {
                var o = [];
                var mnu = this.getMenu();
                if (mnu) {
                    for ( var m = 0; m < mnu.Items.length; m += 1) {
                        var mi = mnu.Items[m];
                        o[m] = {
                            xtype : 'button',
                            text : mi.Caption,
                            iconCls : mi.Icon,
                            tooltip : mi.Hint,
                            handler : (function(mi) {
                                var it = mi;
                                return function(e) {
                                    if (!Ext.isEmpty(it.Items)) {
                                        this.showSubmenu(it);
                                    }
                                    this.showForm({
                                        form : it.Form
                                    });
                                };
                            }(mi)),
                            scope : this
                        };
                    }
                }
                o[o.length] = '->';
                if (getIsThemeComboVisible()) {
                    o[o.length] = {
                        xtype : 'themecombo'
                    };
                }
                o[o.length] = {
                    xtype : 'button',
                    text : this.getUserName(),
                    cls : 'wisa-profile',
                    iconCls : 'profile',
                    tooltip : 'Mijn gebruikersgegevens beheren',
                    handler : function() {
                        this.showForm({
                            form : 'myprofile'
                        });
                    },
                    scope : this
                };
                o[o.length] = {
                    xtype : 'button',
                    text : 'Afmelden',
                    cls : 'wisa-logout',
                    iconCls : 'logout',
                    tooltip : 'Afmelden',
                    handler : function() {
                        this.logout();
                    },
                    scope : this
                };
                return o;
            };
            this.createMainForm = function() {
                var it = [];
                Ext.QuickTips.init();
                setFormParent(new Ext.Panel({
                    title : 'Gegevens opvragen',
                    border : true,
                    region : 'center',
		    cls: 'wisa-center',
		    frame: false,
                    margins : '5 5 2 0',
		    cmargins: '5 5 2 0',
                    layout : 'fit'
                }));
                if (getIsSubmenuVisible()) {
                    setSubmenuPanel(new Ext.Panel(getSubmenuConfig()));
                    it[it.length] = this.getSubmenuPanel();
                }
                if (getIsMainmenuVisible()) {
                    var t = this.getMainMenuConfig();
                    if (Ext.isEmpty(t)) {
                        t = {
                            xtype : 'toolbar',
                            region : 'north',
		     	    cls: 'wisa-top-toolbar',
			    margins: '0 0 0 0',
                            height : 30,
                            items : this.buildMenu()
                        };
                    };
                    it[it.length] = t;
                };
                var c = this.getBottomPanelConfig();
                if (! Ext.isEmpty(c)) {
                  it[it.length]=c;
                }
                it[it.length] = getFormParent();
                this.ViewPort = new Ext.Viewport({
                    layout : 'border',
                    items : it
                });
            };
            this.showSubmenu = function(menu) {
                var p = this.getSubmenuPanel();
                p.setTitle(menu.Caption);
                p.removeAll(true);
                var r = new Ext.tree.TreeNode({
                    text : menu.Caption,
                    expanded : false
                });
                for ( var m = 0; m < menu.Items.length; m += 1) {
                    var mi = menu.Items[m];
                    var o = {
                        text : mi.Caption,
                        iconCls : mi.Icon,
                        leaf : true,
                        qtip : mi.Hint,
                        listeners : {
                            'click' : {
                                fn : (function(mi) {
                                    var it = mi;
                                    return function(e) {
                                        this.showForm({
                                            form : it.Form
                                        });
                                    };
                                }(mi)),
                                scope : this
                            }
                        }
                    };
                    r.appendChild(o);
                }
                var t = new Ext.tree.TreePanel({
                    root : r
                });
                p.add(t);
                if (p.collapsed) {
                    p.toggleCollapse();
                }
                p.doLayout();
            };
            // // Dialogs and Forms
            this.showDialog = function(options) {
                if ((Ext.isEmpty(nsWISA.nsForms[options.form])) && ('noloop' !== arguments[arguments.length - 1])) {
                    /* */console.log('Loading dialog ' + options.form + '.js');
                    options = Ext.apply(options, {
                        url : this.getUrlDialogs() + options.form + '.js',
                        success : setLoadDialogSuccess,
                        failure : setLoadDialogFailure,
                        scope : this
                    });
                    Ext.Ajax.request(options);
                } else {
                    var objForm = new nsWISA.nsForms[options.form];
                    setDialogActive(objForm);
                    objForm.show();
                    objForm.ApplyParams(options.Params);
                }
            };
            var setLoadDialogSuccess = function(response, options) {
                try {
                    eval(response.responseText);
                } catch (err) {
                    nsWISA.showAjaxError('AJAX error : ' + err.message);
                }
                this.fireEvent('DialogLoaded', options.form); // TODO Check Change: dialog dependant
                if (Ext.isEmpty(nsWISA.nsForms[options.form])) {
                    nsWISA.showAjaxError('"' + options.form + '" did not define a valid dialog');
                } else {
                    if (!Ext.isFunction(nsWISA.nsForms[options.form])) {
                        nsWISA.showAjaxError('"' + options.form + '" is not a valid form constructor');
                    } else {
                        this.showDialog(options, 'noloop');
                    }
                }
            };
            var setLoadDialogFailure = function(response, options) {
                window.alert('Kan scherm \"' + options.form + '\" niet openen: ' + response.statusText);
            };
            var closeDialogActive = function() {
                if (getDialogActive()) {
                    getDialogActive().close();
                }
            };
            this.showForm = function(options) {
                if (Ext.isEmpty(getFormParent())) {
                    this.createMainForm();
                }
                if ((Ext.isEmpty(nsWISA.nsForms[options.form])) && ('noloop' !== arguments[arguments.length - 1])) {
                    /* */console.log('Loading form ' + options.form + '.js');
                    options = Ext.apply(options, {
                        url : this.getUrlForms() + options.form + '.js',
                        success : setLoadFormSuccess,
                        failure : setLoadFormFailure,
                        scope : this
                    });
                    Ext.Ajax.request(options);
                } else {
                    if ('noloop' == arguments[arguments.length - 1]) {
                        var clear = true;
                        var objForm = new nsWISA.nsForms[options.form];
                        var objFormParent = getFormParent();
                        if (clear) {
                            objFormParent.removeAll(true);
                        }
                        objFormParent.add(objForm);
                        objFormParent.doLayout();
                        objForm.ApplyParams(options.Params);
                        objFormParent.setTitle(objForm.getFormTitle());
                        if (objForm.iconCls) {
                          objFormParent.setIconClass(objForm.iconCls);
                        }
                        return false; // stop task.
                    } else { // MVC: Workaround voor IE resource/memory leak. Niet dadelijk nieuwe form tonen, maar in timed event.
                      Ext.TaskMgr.start({
                        run: this.showForm,
                        args: [options,'noloop'],
                        interval: 100,
                        scope: this,
                        repeat: 1
                      });
                    }
                }
            };
            var setLoadFormSuccess = function(response, options) {
                try {
                    eval(response.responseText);
                } catch (err) {
                    nsWISA.showAjaxError('AJAX error : ' + err.message);
                }
                this.fireEvent('FormLoaded', options.form); // TODO Check Change: form dependant
                if (Ext.isEmpty(nsWISA.nsForms[options.form])) {
                    nsWISA.showAjaxError('"' + options.form + '" did not define a valid form');
                } else {
                    if (!Ext.isFunction(nsWISA.nsForms[options.form])) {
                        nsWISA.showAjaxError('"' + options.form + '" is not a valid form constructor');
                    } else {
                        this.showForm(options, 'noloop');
                    }
                }
            };
            var setLoadFormFailure = function(response, options) {
                window.alert('Kan scherm \"' + options.form + '\" niet openen: ' + response.statusText);
            };
            // // User Management
            this.login = function(options) {
                WisaUserAPI.Authenticate(options.user, options.password, handleLogin.createDelegate(this));
            };

            this.showLogin = function(options) {
                if (checkIsAPILoaded()) {
                    this.showDialog(Ext.apply({
                        form : 'login'
                    }, options));
                } else {
                    /* */console.log('Waiting for API...');
                    this.addListener('APILoaded', function() {
                        this.showLogin(options);
                    }, this, {
                        single : false
                    });
                }
            };
            var handleLogin = function(provider, response) {
                if (!Ext.isEmpty(response.error)) {
                    nsWISA.showAjaxError('Fout bij aanmelden: ' + response.error.message);
                } else {
                    /*
                     * Ext.Msg.show({title:'Welkom', msg: 'Welkom, '+response.result.Name+'
                     * !',icon:Ext.Msg.INFO,buttons:Ext.Msg.OK});
                     */
                    setUserData(response.result.UserData);
                    setMenu(response.result.menu);
                    closeDialogActive();
                    if (this.hasListener('Login')) {
                        this.fireEvent('Login');
                    } else {
                        this.showForm({
                            form : this.getFormMain()
                        });
                    }
                }
            };
            this.logout = function() {
               Ext.MessageBox.confirm('Afmelden','Bent u zeker dat u de sessie wil afbreken ?',
                 function (bid) {  
                   if (bid=='yes') {
                     var strLogoutURL = getUrlBase() + '/start.html';
                     WisaUserAPI.Logout(function() {
                        window.location = strLogoutURL;
                     });
                   }
                }, 
                this
               );
            };
            // Compatibility
            this.getShowThemeCombo = this.getIsThemeComboVisible;
            this.setShowMainMenu = this.setIsMainmenuVisible;
            this.getShowMainMenu = this.getIsMainmenuVisible;
            this.setShowSubMenu = this.setIsSubmenuVisible;
            this.getShowSubMenu = this.getIsSubmenuVisible;
            this.getBaseURL = this.getUrlBase;
            this.getAppURL = this.getUrlApplication;
            this.saveUserData = this.setUserData;
            this.getUserID = this.getUserId;
            this.getLastLogin = this.getUserLastLogin;
            this.setSubMenuConfig = this.setSubmenuConfig;
            this.getSubMenuConfig = this.getSubmenuConfig;
            this.setMainForm = this.setFormMain;
            this.getMainForm = this.getFormMain;
            this.setActiveDialog = this.setDialogActive;
            this.getActiveDialog = this.getDialogActive;
            this.closeActiveDialog = this.closeDialogActive;
            this.ShowDialog = this.showDialog;
            this.ShowForm = this.showForm;
            this.ShowLogin = this.showLogin;
            this.HandleLogin = this.handleLogin;
            this.DoLogin = this.login;
            this.DoLogout = this.logout;
            this.CreateMainForm = this.createMainForm;
            this.BuildMenu = this.buildMenu;

            // Constructor Code
            // // Configuration
            if (Ext.isDefined(options.url)) {
              if (Ext.isDefined(options.url.base)) {
                  setUrlBase(options.url.base);
              }
              if (Ext.isDefined(options.url.base)) {
                  setUrlBase(options.url.base);
              }
              if (Ext.isDefined(options.url.app)) {
                  setUrlApplication(options.url.app);
              }
            }
            if (Ext.isDefined(options.hasThemes)) {
                setIsThemeComboVisible(options.hasThemes);
            }
            if (Ext.isDefined(options.mainmenu)) {
              if (Ext.isDefined(options.mainmenu.visible)) {
                setIsMainmenuVisible(options.mainmenu.visible);
              }
              if (Ext.isDefined(options.mainmenu.config)) {
                setMainMenuConfig(options.mainmenu.config);
              }
            }
            if (Ext.isDefined(options.submenu)) {
              if (Ext.isDefined(options.submenu.visible)) {
                  setIsSubmenuVisible(options.submenu.visible);
              }
              if (Ext.isDefined(options.submenu.config)) {
                  setSubmenuConfig(options.submenu.config);
              }
            }
            if (Ext.isDefined(options.mainform)) {
              if (Ext.isDefined(options.mainform.name)) {
                  setFormMain(options.mainform.name);
              }
            } 

            // // Events
            this.addEvents({
                'APILoaded' : true,
                'FormLoaded' : true,
                'DialogLoaded' : true,
                'Login' : true,
                'Logout' : true
            });
            // // Superclass Constructor
            nsWISA.ux.BaseApplication.superclass.constructor.call(this, options);
            // // Initialisation
            this.loadAPI();
        }
    // /*** Below has Access to All except Constructor ***/
    // /*** Public Properties [ prop : ] ***/
    // /*** Public Methods [ fct : ] ***/
    };
}()));

nsWISA.ux.SingletonApplication = Ext.extend(Ext.util.Observable, (function() {
    // Private Hidden Property - stays alive as long as reference exists
    var objSingletonInstance = null;
    // Return Object
    return {
        constructor : function(options) {
            if (!objSingletonInstance) {
                objSingletonInstance = new nsWISA.ux.BaseApplication(options);
            }
            return objSingletonInstance;
        }
    };
}()));

// /* Backwards Compatibility & Future Changes */ // TODO Compatibility Artifact Below
Ext.ns('WISA');
Ext.ns('WISA.Forms');
WISA.Forms = nsWISA.nsForms;
WISA.ShowError = nsWISA.showError;
WISA.AjaxError = nsWISA.showAjaxError;
WISA.CheckError = nsWISA.getResponseError;
WISA.CreateServerConfig = nsWISA.getServerConfiguration;
WISA.ClientDataset = nsWISA.ux.BaseClientStore;
WISA.GroupingClientDataset = nsWISA.ux.BaseClientGroupingStore;
WISA.Dialog = nsWISA.ux.BaseDialog;
WISA.Grid = nsWISA.ux.BaseGrid;
WISA.Form = nsWISA.ux.BaseForm;
WISA.FormLoader = nsWISA.ux.BaseFormLoader;
WISA.Application = nsWISA.ux.BaseApplication;

/**
 * <ul>
 * <li>Rename 'WisaUserAPI' : nsWISA.API.User.Logout, nsWISA.API.User.Authenticate</li>
 * <li>Rename 'WISA.API' also to 'nsWISA.nsUser.API' ?</li>
 * <li>Rename 'ProviderName' to 'providerName'</li>
 * </ul>
 */

// /* Application */
// /* JSLint.com */
// Predefined: window, console, Ext, Application, WISA, WisaUserAPI, nsWISA
