unit wmouderdata;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, HTTPDefs, websession, fpHTTP, fpWeb, 
  fpwebdata, webwisa, sqldbwebdata, db, idlist, fpjson, extjsjson, ouderutils,
  mdlwebwisad;

type

  { TOuderDataWebModule }

  TOuderDataWebModule = class(TWisaWebProviderModule)
    Interventies: TWisaWebProvider;
    DBWisa: TWisaWebDatabase;
    LoopbaanVerleden: TWisaWebProvider;
    PersoonsGegevens: TWisaWebProvider;
    TRWisa: TWisaWebTransaction;
    AdresGegevens: TWisaWebProvider;
    BroerzusGegevens: TWisaWebProvider;
    MedischeGegevens: TWisaWebProvider;
    Loopbaangegevens: TWisaWebProvider;
    Klasgenoten: TWisaWebProvider;
    QGetCurrentKlas: TWisaWebQuery;
    VakkenPakket: TWisaWebProvider;
    PedagogischTeam: TWisaWebProvider;
    Afwezigheden: TWisaWebProvider;
    KlasKalender: TWisaWebProvider;
    Facturen: TWisaWebProvider;
    Artikels: TWisaWebProvider;
    Begeleidingen: TWisaWebProvider;
    BegeleidingInterventies: TWisaWebProvider;
    LoopbaanAndereScholen: TWisaWebProvider;
    Rapporten: TWisaWebProvider;
    QGetRapportBestanden: TWisaWebQuery;
    ZorgGegevens: TWisaWebProvider;
    procedure AfwezighedenAfterRowToJSON(Sender: TObject; AObject: TJSONObject);
    procedure AfwezighedenMetaDataToJSON(Sender: TObject; AObject: TJSONObject);
    procedure DataModuleCreate(Sender: TObject);
    procedure InterventiesAfterRowToJSON(Sender: TObject; AObject: TJSONObject);
    procedure InterventiesMetaDataToJSON(Sender: TObject; AObject: TJSONObject);
    procedure KlasKalenderAfterRowToJSON(Sender: TObject; AObject: TJSONObject);
    procedure KlasKalenderMetaDataToJSON(Sender: TObject; AObject: TJSONObject);
    procedure PedagogischTeamAfterRowToJSON(Sender: TObject; AObject: TJSONObject);
    procedure DoCheckLLID(Sender: TObject; const ParamName, AValue: String; var AType: TFieldtype);
    procedure PedagogischTeamMetaDataToJSON(Sender: TObject; AObject: TJSONObject);
    procedure GetStandardCode(Sender: TObject; P: TParam;  var Handled: Boolean);
    procedure RapportenAfterDataToJSON(Sender: TObject; AObject: TJSONObject);
    procedure RapportenAfterRowToJSON(Sender: TObject; AObject: TJSONObject);
    procedure RapportenBeforeDataToJSON(Sender: TObject; AObject: TJSONObject);
    procedure RapportenMetaDataToJSON(Sender: TObject; AObject: TJSONObject);
  private
    FrapportBestanden: TRapportBestanden;
    FDCIDS : TIDList;
    procedure CheckleerlingID(ID: Integer);
    function GetCurrentID(LLID: Integer; IDType : TCurrentID): Integer;
    procedure HandleStdCode(Adaptor: TCustomWebdataInputAdaptor; P: TParam; var Handled: Boolean);

    { private declarations }
  public
    { public declarations }
  end; 

var
  OuderDataWebModule: TOuderDataWebModule;

implementation

uses dateutils, cfgWebouders;

{ TOuderDataWebModule }

procedure TOuderDataWebModule.CheckleerlingID(ID : Integer);

begin
  ouderutils.CheckLeerlingID(Session,ID);
end;

procedure TOuderDataWebModule.DoCheckLLID(
  Sender: TObject; const ParamName, AValue: String; var AType: TFieldtype);
begin
  If (ParamName='ID') then
    begin
    Log('Check Leerling ID %s',[AValue]);
    CheckLeerlingID(StrToIntDef(AVAlue,-1));
    AType:=ftInteger;
    end;
end;

procedure TOuderDataWebModule.PedagogischTeamMetaDataToJSON(Sender: TObject;
  AObject: TJSONObject);
begin
  AObject.Arrays['fields'].Add(TJSONObject.Create(['name','ROL','type','string']));
end;

procedure TOuderDataWebModule.GetStandardCode(Sender: TObject;
  P: TParam; var Handled: Boolean);
begin
  HandleStdCode((Sender as TWisaWebProvider).Adaptor,P,Handled);
end;

procedure TOuderDataWebModule.RapportenAfterDataToJSON(Sender: TObject;
  AObject: TJSONObject);
begin
  Session.Variables['DocumentIDS']:=FDCIDS.AsString;
  FreeAndNil(FRapportBestanden);
end;

procedure TOuderDataWebModule.RapportenAfterRowToJSON(Sender: TObject;
  AObject: TJSONObject);

Var
  B : TRapportBestand;

begin
  B:=FrapportBestanden.FindBestandForPeriode(Rapporten.Dataset.FieldByName('PE_ID').AsInteger);
  if Assigned(B) then
    begin
    AObject.Add('D_ID',TJSONIntegerNumber.Create(B.DocumentID));
    FDCIDS.Add(B.DocumentID);
    end
  else
    AObject.Add('D_ID',TJSONNull.Create);
end;

procedure TOuderDataWebModule.RapportenBeforeDataToJSON(Sender: TObject;
  AObject: TJSONObject);

Var
  H : Boolean;

begin
  With QGetRapportBestanden do
    begin
    ParamByName('K_ID').asinteger:=Settings.RapKastID;
    HandleStdCode((Sender as TWisaWebProvider).Adaptor,ParamByName('IU_ID'),H);
    Open;
    try
      if Assigned(FRapportBestanden) then
        FRapportBestanden.Clear
      else
        FRapportBestanden:=TRapportBestanden.Create(TRapportBestand);
      FRapportBestanden.LoadFromDataset(QGetRapportBestanden);
      If not Assigned(FDCIDS) then
        FDCIDS:=TIDList.Create;
      FDCIDS.AsString:=Session.variables['DocumentIDS'];
    finally
      Close;
    end;
    end;
end;

procedure TOuderDataWebModule.RapportenMetaDataToJSON(Sender: TObject;
  AObject: TJSONObject);

begin
  AObject.Arrays['fields'].Add(TJSONObject.Create(['name','D_ID','type','integer']));
end;


procedure TOuderDataWebModule.PedagogischTeamAfterRowToJSON(Sender: TObject;
  AObject: TJSONObject);
begin
  AObject.Add('ROL',GetRolNamen(PedagogischTeam.Dataset.FieldByName('ROLMASK').AsInteger))
end;

procedure TOuderDataWebModule.HandleStdCode(Adaptor: TCustomWebdataInputAdaptor; P: TParam; var Handled: Boolean);

Var
  N,S : String;
  ID : Integer;

begin
  N:=lowercase(P.Name);
  if (N='datum') then
    begin
    P.AsDateTime:=Settings.WerkDatum;
    Log('Applying param Datum %s',[DateToStr(P.AsDateTime)]);
    Handled:=True;
    end
  else If (N='id') or (N='kl_id') or (N='ll_id') or (N='lb_id') or (N='iu_id') then
    begin
    if not (Adaptor.TryParamValue('LL_ID',S) or Adaptor.TryParamValue('ID',S)) then
      Raise Exception.Create('Geen leerling parameter beschikbaar');
    ID:=StrToIntDef(S,-1);
    CheckLeerlingID(ID);
    If (N='id') or (N='ll_id') then
      P.AsInteger:=ID
    else if (N='kl_id') then
      P.AsInteger:=GetCurrentID(ID,ciKL)
    else if (N='lb_id') then
      P.AsInteger:=GetCurrentID(ID,ciLB)
    else if (N='iu_id') then
      P.AsInteger:=GetCurrentID(ID,ciIU);
    Log('Set param %s to %d',[p.name,p.AsInteger]);
    Handled:=True;
    end
  else if (n='niveau') then
    begin
    P.AsInteger:=99; // Settings.OuderRolNiveau;
//    P.AsInteger:=Settings.OuderRolNiveau;
    Log('Setting param niveau to %d',[P.AsInteger]);
    Handled:=True;
    end
  else
    begin
    Log('Unknown standard parameter name : %s',[n]);
    Handled:=False;
    end;
end;

function TOuderDataWebModule.GetCurrentID(LLID : Integer; IDType : TCurrentID) : Integer;

begin
  ouderutils.GetCurrentID(Session,Database,LLID,IDType);
end;

procedure TOuderDataWebModule.KlasKalenderAfterRowToJSON(Sender: TObject;
  AObject: TJSONObject);

Var
  DV,DT : TDateTime;
  TV,TT : Integer;
  S : String;

begin
  With Afwezigheden.Dataset do
    begin
    DV:=FieldByName('KD_VAN').AsDateTime;
    DT:=FieldByName('KD_TOT').AsDateTime;
    TV:=FieldByName('KD_VANUUR').AsInteger;
    TT:=FieldByName('KD_TOTUUR').AsInteger;
    end;
  If (Round(DV)=Round(DT)) then
    begin
    if (TV=TT) then
      S:=DatumUur(DV,TV)
    else
      S:=Format('%s - %s',[DatumUur(DV,TV),Uurnaam(TV,True)]);
    end
  else
    S:=Format('%s - %s',[DatumUur(DV,TV),DatumUur(DT,TT)]);
  AObject.Add('PERIODE',S);
  AObject.Add('MAAND',FormatDateTime('mmmm',DV));
end;

procedure TOuderDataWebModule.KlasKalenderMetaDataToJSON(Sender: TObject;
  AObject: TJSONObject);
begin
  AObject.Arrays['fields'].Add(TJSONObject.Create(['name','PERIODE','type','string']));
  AObject.Arrays['fields'].Add(TJSONObject.Create(['name','MAAND','type','string']));
end;

procedure TOuderDataWebModule.DataModuleCreate(Sender: TObject);

Var
  S : String;

begin
  S:=Format('AND (IU_SCHOOL_FK IN (%s))',[Settings.SCIDS.AsString]);
  LoopbaanGegevens.SelectSQL.Add(S);
  S:=S+Format(' AND (LB_TOT<''6/30/%d'')',[Settings.GetSchoolJaar]);
  LoopbaanVerleden.SelectSQL.Add(S);
  LoopbaanVerleden.SelectSQL.Add('ORDER BY LB_VAN DESC');
  S:=Format('AND NOT (IU_SCHOOL_FK IN (%s))',[Settings.SCIDS.AsString]);
  LoopbaanAndereScholen.SelectSQL.Add(S);
  LoopbaanAndereScholen.SelectSQL.Add('ORDER BY LB_VAN DESC');
  QGetCurrentKlas.SQL.Add(S);
  S:=Format('and (DB_MainGroup_FK IN (%s))',[Settings.MGIDS.AsString]);
  Facturen.SelectSQL.Add(S);
  With Settings do
    begin
    Begeleidingen.SelectSQL.Text:=Format(Begeleidingen.SelectSQL.Text,[SCIDS.ASstring,BGSubTypes.AsString,BGTypes.AsString]);
    BegeleidingInterventies.SelectSQL.Text:=Format(BegeleidingInterventies.SelectSQL.Text,[LITypes.AsString,SCIDS.ASstring]);
    Interventies.SelectSQL.Text:=Format(Interventies.SelectSQL.Text,[LITypes.AsString,SCIDS.ASstring]);
    end;

end;

procedure TOuderDataWebModule.InterventiesAfterRowToJSON(Sender: TObject;
  AObject: TJSONObject);
begin
  With Interventies.Dataset do
    AObject.Add('LI_MAAND',FormatDateTime('mmmm',FieldByName('LI_DATUM').AsDateTime));
end;

procedure TOuderDataWebModule.InterventiesMetaDataToJSON(Sender: TObject;
  AObject: TJSONObject);

begin
  AObject.Arrays['fields'].Add(TJSONObject.Create(['name','LI_MAAND','type','string']));
end;


procedure TOuderDataWebModule.AfwezighedenAfterRowToJSON(Sender: TObject; AObject: TJSONObject);

Var
  DV,DT : TDateTime;
  TV,TT : Integer;
  S : String;

begin
  With Afwezigheden.Dataset do
    begin
    DV:=FieldByName('AF_VAN').AsDateTime;
    DT:=FieldByName('AF_TOT').AsDateTime;
    TV:=FieldByName('AF_VANUUR').AsInteger;
    TT:=FieldByName('AF_TOTUUR').AsInteger;
    end;
  If (Round(DV)=Round(DT)) then
    begin
    if (TV=TT) then
      S:=DatumUur(DV,TV)
    else
      S:=Format('%s - %s',[DatumUur(DV,TV),Uurnaam(TV,True)]);
    end
  else
    S:=Format('%s - %s',[DatumUur(DV,TV),DatumUur(DT,TT)]);
  AObject.Add('PERIODE',S);
  AObject.Add('MAAND',FormatDateTime('mmmm',DV));
end;

procedure TOuderDataWebModule.AfwezighedenMetaDataToJSON(Sender: TObject; AObject: TJSONObject);
begin
  AObject.Arrays['fields'].Add(TJSONObject.Create(['name','PERIODE','type','string']));
  AObject.Arrays['fields'].Add(TJSONObject.Create(['name','MAAND','type','string']));
end;

initialization
  {$I wmouderdata.lrs}

  RegisterWISAProviderModule(TOuderDataWebModule);
end.

