unit wmoudersusers;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, websession, fpjson, fpextdirect,
  db, sqldb, webwisa, fpjsonrpc, mdlwebwisad;

type
  
  { TRPCOudersUsers }

  TRPCOudersUsers = class(TWisaWebRPCModule)
    Authenticate: TWisaRPCHandler;
    DBWisa: TWisaWebDatabase;
    QAuthenticateUser: TWisaWebQuery;
    QUpdateLastLogout: TWisaWebQuery;
    TRWIsa: TWisaWebTransaction;
    QUpdateLastLogin: TWisaWebQuery;
    Logout: TWisaRPCHandler;
    QScreenRights: TWisaWebQuery;
    QProfileRights: TWisaWebQuery;
    GetUserMenu: TWisaRPCHandler;
    ChangePassword: TWisaRPCHandler;
    QChangePassword: TWisaWebQuery;
    QCheckPasswordChange: TWisaWebQuery;
    QMijnLeerlingen: TWisaWebQuery;
    QFindUserByName: TWisaWebQuery;
    QInsertUser: TWisaWebQuery;
    QUpdateRolID: TWisaWebQuery;
    procedure AuthenticateExecute(Sender: TObject; const Params: TJSONData;
      out Res: TJSONData);
    procedure ChangePasswordExecute(Sender: TObject; const Params: TJSONData;
      out Res: TJSONData);
    procedure GetUserMenuExecute(Sender: TObject; const Params: TJSONData; out
      Res: TJSONData);
    procedure LogoutExecute(Sender: TObject; const Params: TJSONData; out
      Res: TJSONData);
  private
    Function CheckExistingUser(UID: INteger; P, N: String) : string;
    function CheckNewUser(ARol: TLeerlingRol; LDS: TloginDataList;
      const AUserName, APassword: String): TUserData;
    function CreateLeerlingMenu(LLID: Integer; Const ANaam : String): TJSONObject;
    function CreateNewUser(AUserName, APassword: String): Integer;
    function FindUser(const AUserName: String): Integer;
    function CreateUserMenu(LD: TLoginDataList): TJSONObject;
    procedure LoadLoginData(LD: TLoginDataList; AUserName: String);
    procedure ReadSecurity;
    procedure UpdateLastLogin(AUserID: Integer);
    procedure UpdateRolUID(ALeerling: Integer; ARol: TleerlingRol;
      const AUID: Integer);
    { private declarations }
  protected
    Class Function ActionName : String; override;
  public
    { public declarations }
  end; 

var
  OudersRPCUsers: TRPCOudersUsers;

Procedure RegisterOudersUserManagement;

implementation

uses slcrypt,webwisamenu, idlist, cfgwebouders;

resourcestring
  SErrUnknownUser = 'Onbekende gebruiker: "%s"';
  SErrInvalidPassword = 'Ongeldig wachtwoord ingegeven.';
  SErrAccountExpired = 'Gebruiker "%s" niet langer geldig.';

Procedure RegisterOudersUserManagement;

begin
  RegisterWISARPCModule(TRPCOudersUsers);
end;

{ TRPCOudersUsers }

procedure TRPCOudersUsers.UpdateLastLogin(AUserID : Integer);

begin
  With QUpdateLastLogin do
    begin
    Params.ParamByName('U_ID').AsInteger:=AUserID;
    ExecSQL;
    (Transaction as TWisaWebTransaction).CommitRetaining;
    end;
end;

class function TRPCOudersUsers.ActionName: String;
begin
  Result:='WisaUserAPI';
end;

procedure TRPCOudersUsers.ReadSecurity;

var
  profiles : array of record Id,rights : integer; end;
  profileCount, profileLoop, rechten : integer;
  FUserRights : Integer;
  FScreenRights : TStringList;

  procedure UserStandaard;

  Var
    O,FSR : TField;

  begin
    FUserRights := 0; //DefaultRights;
    with QProfileRights do
      begin
      Close;
      params[0].asinteger := Session.UserID;
      open;
      try
        first;
        FSR:=FieldByName('O_STANDAARDRECHTEN');
        O:=FieldByName('O_ID');
        ProfileCount := QProfileRights.recordCount;
        SetLength (Profiles, ProfileCount);
        ProfileLoop := 0;
        while not eof do
          begin
          FUserRights := FUserRights OR FSR.asinteger;
          with profiles[ProfileLoop] do
            begin
            ID := O.asinteger;
            Rights := FSR.asinteger;
            end;
          inc (ProfileLoop);
          next;
          end;
      finally
        close;
      end;
      end
  end;

  var PrevScreen : string;
      mustcontinue : boolean;

  procedure AddRight (X : integer);
  begin
    rechten := rechten OR X;
    inc (profileLoop);
    mustcontinue := (rechten and AllRights) <> AllRights;
  end;

  procedure NewScreen;
  begin
    while (profileLoop < profileCount) and mustcontinue do
      AddRight (profiles[profileLoop].rights);
    if PrevScreen <> '' then
      FScreenRights.values[prevScreen] := inttostr(rechten);
    rechten := 0;
    profileloop := 0;
    mustcontinue := True;
    PrevScreen := QScreenRights.FieldByName('S_NAAM').Asstring;
  end;

  procedure ScreenRights;
  var
    ThisProfile : integer;
    nextscreen : boolean;
    R,P,N : TField;

  begin
    with QScreenRights do
      begin
      Close;
      params[0].asinteger := Session.UserID;
      FScreenRights.BeginUpdate;
      open;
      try
        N:=FieldByName('S_NAAM');
        P:=FieldByName('R_PROFILE_FK');
        R:=FieldByName('R_RECHTENVLAGGEN');
        first;
        FScreenrights.Sorted := false;
        prevScreen := '';
        while not eof do
          begin
          if PrevScreen <> N.asstring then
            NewScreen;
          ThisProfile := P.asinteger;
          while (Profiles[profileLoop].Id <> ThisProfile) and mustcontinue do
            AddRight (Profiles[profileLoop].rights);
          AddRight (R.asinteger);
          repeat
            next;
            NextScreen:=(N.Asstring <> prevScreen) or eof;
          until mustcontinue or nextscreen;
          end;
        if PrevScreen <> '' then
          NewScreen;
      finally
        close;
        FScreenRights.sorted := True;
        FScreenRights.EndUpdate;
      end;
      end;
  end;

begin
  FScreenRights:=TStringList.Create;
  try
    If (Session.UserID<>0) then
      begin
      UserStandaard;
      ScreenRights;
      end
    else
      FUserRights:=$FFFF;
    Session.ScreenRights:=FScreenRights;
    Session.DefaultRights:=FUserRights;
  finally
    FreeAndNil(FScreenRights);
  end;
end;

Function TRPCOudersUsers.FindUser(Const AUserName : String) : Integer;

Var
  Q : TWisaWebQuery;

begin
  Result:=-1;
  Q:=QFindUserByName;
  Q.Params.ParamByName('U_NAAM').AsString:=AUserName;
  Q.Open;
  try
    If not (Q.EOF and Q.BOF) then
      Result:=Q.FieldByname('U_ID').AsInteger;
  finally
    Q.Close;
  end;
end;

Function TRPCOudersUsers.CreateNewUser(AUserName,APassword : String) : Integer;

Var
  I : Integer;
  PWD,Un : shortString;
  Q : TWisaWebQuery;

begin
  I:=0;
  Repeat
    If (I>0) then
      UN:=AUserName+inttostr(i)
    else
      UN:=AUserName;
    Inc(I);
  until (FindUser(UN)=-1);
  Result:=GEN_ID('GEN_USER');
  Q:=QInsertUser;
  Q.Params.ParamByName('U_ID').AsInteger:=Result;
  Q.Params.ParamByName('U_NAAM').AsString:=UN;
  Q.Params.ParamByName('U_NAAMVOLLEDIG').AsString:=UN;
  Pwd:=lcrypt(APassword,pwdsalt);
  Q.Params.ParamByName('U_PASWOORD').AsString:=pwd;
  Q.ExecSQL;
 (Q.Transaction as TSQLTransaction).Commit;
end;

Function EncodeUserPassword(Pwd : String) : String;

begin
  Result:=lcrypt(Pwd,pwdsalt);
end;

Function TRPCOudersUsers.CheckExistingUser(UID : Integer; P,N : String) : String;

begin
  With QAuthenticateUser do
    begin
    Close;
    Params.ParamByName('UID').AsInteger:=UID;
    Open;
    try
      If (EOF and BOF) then
        Raise EWisaWeb.CreateFmt(SErrUnknownUser,[N]);
      If (FieldByname('U_PASWOORD').AsString<>EncodeUserPassword(P)) then
        Raise EWisaWeb.Create(SErrInvalidPassword);
      If (Not FieldByName('U_GELDIGTOT').IsNull) and (FieldByName('U_GELDIGTOT').AsDateTime<Now) then
        Raise EWisaWeb.CreateFmt(SErrAccountExpired,[N]);
      Session.Variables['flags']:=FieldByName('U_APPFLAGS').AsString;
      Session.Variables['changepwd']:=FieldByName('U_KANVERANDERPASW').AsString;
      Result:=FieldByName('U_NAAMVOLLEDIG').AsString;
      UpdateLastLogin(UID);
    finally
      close;
    end;
    end;
end;

Procedure TRPCOudersUsers.UpdateRolUID(ALeerling : Integer; ARol : TleerlingRol; Const AUID : Integer);

Const
    SUpdateLeerlingID = 'UPDATE LEERLING SET LL_USER_FK=:UID WHERE (LL_ID=:LL_ID)';
    SUpdateVaderID    = 'UPDATE LEERLING SET LL_VADERUSER_FK=:UID WHERE (LL_ID=:LL_ID)';
    SUpdateMoederID   = 'UPDATE LEERLING SET LL_MOEDERUSER_FK=:UID WHERE (LL_ID=:LL_ID)';
    SUpdateVoogdID    = 'UPDATE LEERLING SET LL_VOOGDUSER_FK=:UID WHERE (LL_ID=:LL_ID)';

Var
  SQL : String;
  Q : TWisaWebQuery;

begin
  Case ARol of
    lrLeerling : SQL:=SUpdateleerlingID;
    lrVader    : SQL:=SUpdateVaderID;
    lrMoeder   : SQL:=SUpdateMoederID;
    lrVoogd    : SQL:=SUpdateVoogdID;
  else
    Raise Exception.CreateFmt('UpdateRolID: Onbekende rol %d',[Ord(ARol)]);
  end;
  Q:=QUpdateRolID;
  try
    Q.SQL.Text:=SQL;
    Log('Update rol : %s (UID: %d, LLID: %d)',[SQL,AUID,ALeerling]);
    Q.Params.ParamByName('LL_ID').AsInteger:=ALeerling;
    Q.Params.ParamByName('UID').AsInteger:=AUID;
    Q.ExecSQL;
    (Q.Transaction as TSQLTransaction).Commit;
  finally
    Q.Free;
  end;
end;

Function TRPCOudersUsers.CheckNewUser(ARol : TLeerlingRol; LDS : TloginDataList; Const AUserName,APassword : String) : TUserData;

Var
  L : TLoginData;
  I,RID,ID : Integer;
begin
  Result:=Nil;
  Log('Searching RR match');
  L:=LDS.GetRRMatch(APassword);
  Log('Searching RR match %d',[Ord(L<>Nil)]);
  If (L<>Nil) then
    begin
    Log('Creating new user match');
    ID:=CreateNewUser(AUserName,APassword);
    Log('Created new user: %d ',[ID]);
    For I:=0 to LDS.Count-1 do
      begin
      L:=LDS[i];
      RID:=L.GetRolUID(ARol);
      Log('Checking leerling userid: %d  (%d)',[L.leerlingID,RID]);
      if (CompareText(L.GetRolEmail(ARol),AUserName)=0) and (RID<=0) and (RID<>ID) then
        UpdateRolUID(L.LeerlingID,ARol,ID);
      end;
    Log('Creating result with ID',[ID]);
    Result:=TUserData.Create(ID);
    Result.password:=EncodeUserPassword(APassword);
    end;
end;

procedure TRPCOudersUsers.LoadLoginData(LD : TLoginDataList; AUserName : String);

Var
  Q : TWisaWebQuery;
begin
  Q:=QMijnLeerlingen;
  Q.Params.ParamByName('MoederEmail').AsString:=AUserName;
  Q.Params.ParamByName('VaderEmail').AsString:=AUserName;
  Q.Params.ParamByName('VoogdEmail').AsString:=AUserName;
  Q.Open;
  try
    If Not (Q.EOF AND Q.BOF) then
      LD.LoadFromDataset(Q);
  finally
    Q.Close;
  end;
end;

procedure TRPCOudersUsers.AuthenticateExecute(Sender: TObject;
  const Params: TJSONData; out Res: TJSONData);

Var
  O : TJSONArray;
  N,P,FN : String;
  I,UID : Integer;
  M : Tjsonobject;
  LL : TDateTime;
  LD : TLoginDataList;
  LI : TLoginData;
  R : TLeerlingRol;
  IDS : TIDList;

begin
  O:=Params as TJSONArray;
  N:=O.Strings[0];
  P:=O.Strings[1];
  LD:=TLoginDataList.Create(TLoginData);
  try
    LoadLoginData(LD,N);
    Li:=LD.GetMatch(N,R);
    If (LI<>nil) then
      begin
      Case R of
        lrVader  : UID:=Li.VaderUID;
        lrMoeder : UID:=Li.MoederUID;
        lrVoogd  : UID:=Li.VoogdUID;
      else
         Raise Exception.CreateFmt('Onbekende rol %d',[Ord(R)]);
      end;
      end
    else
      Raise Exception.Create('Interne fout: geen logindata');
    If (UID>0) then
      FN:=CheckExistingUser(UID,P,N)
    else
      CheckNewUser(R,LD,N,P);
    IDS:=TIDList.Create(LD.Count);
    try
      For I:=0 to LD.Count-1 do
        begin
        Li:=LD[i];
        IDS.Add(LI.LeerlingID);
        end;
      Session.Variables['Leerlingen']:=IDS.AsString;
      Log('Leerlingen : '+IDS.AsString);
    finally
      IDS.Free;
    end;
    M:=CreateUserMenu(LD);
  finally
    LD.Free;
  end;
  Session.UserID:=UID;
  Session.Variables['FullName']:=FN;
  Session.Variables['LastLogin']:=DateTimeToStr(LL);
  Res:=TJSONObject.Create(['UserData',TJSonObject.Create(['userid',UID,'name',FN,'lastlogin',DateTimeToStr(LL)]),'menu',M]);
end;

Function TRPCOudersUsers.CreateLeerlingMenu(LLID : Integer; Const ANaam : String) : TJSONObject;

  Procedure AddLeerlingParam(O : TJSONObject);

  Var
    I : integer;
    Items : TJSONArray;

  begin
    O.Add('Params',TJSonObject.Create(['ID',LLID]));
    i:=O.IndexOfName('Items');
    If (I<>-1) then
      begin
      Items:=O.Items[I] as TJSONArray;
      For I:=0 to Items.Count-1 do
        AddleerlingParam(Items.Objects[i]);
      end;
  end;

begin
  Result:=FullMenu.ToJSON(true);
  Result.Strings['Caption']:=ANaam;
  AddLeerlingParam(Result);
end;


Function TRPCOudersUsers.CreateUserMenu(LD : TLoginDataList) : TJSONObject;


Var
  I: Integer;
  Items : TJSONArray;

begin
  Result:=TJSONObject.Create;
  Items:=TJSONArray.Create();
  Result.Add('Items',Items);
  For I:=0 to LD.Count-1 do
    begin
    Items.Add(CreateLeerlingMenu(LD[i].LeerlingID,LD[i].VoorNaam+' '+LD[i].Naam));
    end;
end;

procedure TRPCOudersUsers.ChangePasswordExecute(Sender: TObject;
  const Params: TJSONData; out Res: TJSONData);

Var
  A : TJSONArray;
  Old,New : String;

begin
  A:=Params as TJSONArray;
  Old:=lcrypt(A.Strings[0],pwdsalt);
  New:=lcrypt(A.Strings[1],pwdsalt);
  If (UserID<=0) then
    Raise EWisaWeb.Create('Het wachtwoord van deze gebruiker mag niet veranderd worden.');
  With QCheckPasswordChange do
    begin
    Close;
    Params.ParamByName('U_ID').AsInteger:=UserID;
    Open;
    try
      if (EOF and BOF) then
        Raise EWisaWeb.CreateFmt('Geen gegevens voor gebruiker %d',[UserID]) ;
      If FieldByName('U_KANVERANDERPASW').AsString<>'+' then
        Raise EWisaWeb.Create('Gebruiker mag zijn wachtwoord zelf niet wijzigen') ;
      If FieldByName('U_PASWOORD').AsString<>Old then
        Raise EWisaWeb.Create('Opgegeven huidig wachtwoord is niet correct') ;
    finally
      Close;
    end;
    end;
  With QChangePassword do
    begin
    ParamByName('U_ID').AsInteger:=UserID;
    ParamByName('U_PASWOORD').AsString:=New;
    ParamByName('U_VERANDERDDOOR').AsInteger:=UserID;
    ExecSQl;
    (Transaction as TSQLTransaction).CommitRetaining;
    end;
  res:=TJSONBoolean.Create(True);
end;

procedure TRPCOudersUsers.GetUserMenuExecute(Sender: TObject;
  const Params: TJSONData; out Res: TJSONData);

Var
  ID : Integer;

begin
  ID:=Session.UserID;
  If ID=-1 then
    Res:=TJSONString.Create('not logged in')
  else
    Res:=FullMenu.ToJSON(True);
end;

procedure TRPCOudersUsers.LogoutExecute(Sender: TObject; const Params: TJSONData; out
  Res: TJSONData);

Var
  ID : Integer;

begin
  ID:=Session.UserID;
  If ID=-1 then
    Res:=TJSONString.Create('not logged in')
  else
    begin
    Log('Logging out');
    Session.UserID:=-1;
    Session.FullName:='';
    Session.DatabaseName:='';
    Res:=TJSONString.Create('bye, bye!');
    With QUpdateLastLogOut do
      begin
      Params.ParamByName('U_ID').AsInteger:=ID;
      ExecSQL;
      (Transaction as TSQLTransaction).Commit;
      end;
    end;
end;

initialization
  {$I wmoudersusers.lrs}

end.

