unit wrouders;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, websession,  fpweb, fpjson, fpextdirect,
  webwisa, fpjsonrpc, fpfcgi;

Const
  MaxUur = 2;

Type
  TAfwezigCell = Record
    Code : String[2];
    kleur : Integer;
  end;
  TMonthAfwezig = Array[1..31,1..MaxUur] of TAfwezigCell;
  TJaarAfwezig = Array[1..12] of TMonthAfwezig;

  { TWisaWebRPCModule1 }

  TWisaWebRPCModule1 = class(TWisaWebRPCModule)
    GetLessenroosterURL: TWisaRPCHandler;
    GetAfwezighedenOverzicht: TWisaRPCHandler;
    DBWisa: TWisaWebDatabase;
    QAfwezigheden: TWisaWebQuery;
    TRWisa: TWisaWebTransaction;
    procedure GetAfwezighedenOverzichtExecute(Sender: TObject;
      const Params: TJSONData; out Res: TJSONData);
    procedure GetLessenroosterURLExecute(Sender: TObject;
      const Params: TJSONData; out Res: TJSONData);
  private
    { private declarations }
    Class function ActionName : string; override;
    procedure FillJaar(var AJaar: TJaarAfwezig; Y: Word; Q: TWisaWebQuery);
    function GetAfwezighedenOverzichtHTML(JaarOverzicht: TJaarAfwezig): string;
  public
    { public declarations }
  end; 

var
  WisaWebRPCModule1: TWisaWebRPCModule1; 

implementation

uses cfgwebouders, mdlwebwisad, dateutils,ouderutils;

{$R *.lfm}


{ TWisaWebRPCModule1 }

procedure TWisaWebRPCModule1.GetLessenroosterURLExecute(Sender: TObject;
  const Params: TJSONData; out Res: TJSONData);

Var
  KLID,LLID : Integer;

begin
  Res:=Nil;
  LLID:=(Params as TJSONArray).Integers[0];
  KLID:=GetCurrentID(Session,Database,LLID,ciKL);
  Res:=TJSONString.Create(Application.ApplicationURL+'/Files/rooster?KLID='+IntToStr(KLID));
end;

procedure TWisaWebRPCModule1.GetAfwezighedenOverzichtExecute(Sender: TObject;
  const Params: TJSONData; out Res: TJSONData);

Var
  IUID,LLID : Integer;
  J : TJaarAfwezig;

begin
  Res:=Nil;
  LLID:=(Params as TJSONArray).Integers[0];
  IUID:=GetCurrentID(Session,Database,LLID,ciIU);
  With QAfwezigheden do
    begin
    ParamByName('IU_ID').AsInteger:=IUID;
    Open;
    try
      FillJaar(J,Settings.GetSchooljaar,QAfwezigheden);
      Res:=TJSONString.Create(GetAfwezighedenOverzichtHTML(J));
    finally
      Close;
    end;
    end;
end;

Function TWisaWebRPCModule1.GetAfwezighedenOverzichtHTML(JaarOverzicht: TJaarAfwezig) : string;

  Function MaandNaam (M : Integer) : String;

  begin
    Case M of
      1 : Result  := 'januari';
      2 : Result  := 'februari';
      3 : Result  := 'maart';
      4 : Result  := 'april';
      5 : Result  := 'mei';
      6 : Result  := 'juni';
      7 : Result  := 'juli';
      8 : Result  := 'augustus';
      9 : Result  := 'september';
      10 : Result := 'oktober';
      11 : Result := 'november';
      12 : Result := 'december';
    end;
  end;

Var
  I,j,K,WC,M : Integer;
  S,H, L,KL : String;

begin
  S:='<TABLE BORDER=1 class="wisa-afwezigheden-tabel">';
  S:=S+'<TR><TH ROWSPAN="2">Dag</TH>';
  For I:=1 to 31 do
    S:=S+Format('<TH COLSPAN="%d" align="center" class="wisa-afwezigheden-tabel-header">%d</TH>',[Maxuur,I]);
  S:=S+'</TR><TR>';
  For I:=1 to 31 do
    For M:=1 to MaxUur do
      S:=S+Format('<TH class="wisa-afwezigheden-tabel-header">%s</TH>',[Uurnaam(M)]);
  S:=S+'</TR>';
  For I:=1 to 10 do
    begin
    M:=I+8;
    If M>12 then
      M:=M-12;
    S:=S+Format('<TR><TD>%s</TD>',[MaandNaam(M)]);
    For J:=1 to 31 do
      For K:=1 to MaxUur do
        begin
        KL:=GetKleur(JaarOverzicht[I,J,K].Kleur);
        S:=S+Format('<TD bgcolor="#%s" class="wisa-afwezigheden-tabel-cell">%s</TD>',[kl,JaarOverzicht[I,J,K].Code]);
        end;
    S:=S+'</TR>';
    end;
  S:=S+'</TABLE>';
  L:=''; // '<div class="legende">Legende</div>';
  Wc:=(192 shl 16) or (192 shl 8) or 192;
  KL:=GetKleur(Wc);
  L:=L+'<table class="legende-detail">';
  L:=L+Format('<TR><TD BGCOLOR="#%s" class="wisa-afwezigheden-tabel-cell wisa-afwezigheden-legende-cell">&nbsp;</TD>',[KL]);
  L:=L+'<TD class="wisa-afwezigheden-legende-cell">Weekend</TD></TR></TABLE>';
  Result:=S+L;
end;

procedure TWisaWebRPCModule1.FillJaar(Var AJaar: TJaarAfwezig; Y: Word; Q : TWisaWebQuery);

Var
  I,J,TV,TT,KL : Integer;
  DV,Dt,B,E : TDateTime;
  DY,M,D : Word;
  W : Boolean;
  Code : String;

begin
  For I:=1 to 12 do
    for J:=1 to 31 do
      For M:=1 to MaxUUr do
        begin
        AJaar[I,J,M].Code:='';
        AJaar[I,J,M].Kleur:=-1;
        end;
  B:=EncodeDate(Y,9,1);
  E:=EncodeDate(Y+1,6,30);
  While not Q.EOF do
    begin
    DV:=Q.FieldByName('AF_VAN').AsDateTime;
    DT:=Q.FieldByName('AF_TOT').AsDateTime;
    TV:=Q.FieldByName('AF_VANUUR').AsInteger;
    TT:=Q.FieldByName('AF_TOTUUR').AsInteger;
    Code:=Q.FieldByName('AC_CODE').AsString;
    KL:=Q.FieldByName('AC_KLEUR').AsInteger;
    If (DV>=B) and (DT<=E) then
      begin
      Repeat
        DecodeDate(DV,DY,M,D);
        If M<9 then M:=M+12;
        Dec(M,8);
        While (TV<=MaxUur) do
          begin
          If (Round(DV)<Round(DT)) or (TV<=TT) then
            begin
            AJaar[M,D,TV].Code:=Copy(Code,1,2);
            AJaar[M,D,TV].Kleur:=KL;
            end;
          Inc(TV);
          end;
        DV:=DV+1;
        TV:=1;
      Until Round(DV)>Round(DT);
      end;
    Q.Next;
    end;
  // Weekends
  For I:=1 to 12 do
    for J:=1 to 31 do
      For M:=1 to MaxUUr do
        begin
        If I>4 then
          W:=TryEncodeDate(Y+1,I-4,J,Dt)
        else
          W:=TryEncodeDate(Y,I+8,J,Dt);
        If W then
          W:=DayOfTheWeek(Dt) in [7,6];
        If W then
          For TV:=1 to MaxUUr do
            begin
            AJaar[I,J,TV].Code:=' ';
            AJaar[I,J,TV].Kleur:=(192 shl 16) or (192 shl 8) or 192;
            end;
        end;
end;

class function TWisaWebRPCModule1.ActionName: string;
begin
  Result:='WebOudersAPI';
end;

initialization
  RegisterWISARPCModule(TWisaWebRPCModule1); 
end.

