'use strict';

if ((typeof Range !== "undefined") && !Range.prototype.createContextualFragment)
{
  Range.prototype.createContextualFragment = function(html)
  {
    var frag = document.createDocumentFragment(),
    div = document.createElement("div");
    frag.appendChild(div);
    div.outerHTML = html;
    return frag;
  };
};

if (typeof console == "undefined" || typeof console.log == "undefined")
    var console = {
        log : function() {
        }
    };

// namespace definitions
Ext.ns('WISA');
Ext.ns('WISA.Forms');
Ext.ns('WISA.controls');
Ext.ns('WISA.data');
Ext.ns('WISA.baseforms');
Ext.ns('WISA.configpanels');
Ext.ns('doc.namespaces');

Ext.apply(WISA,{
  /**
   * @method WISA.control
   * @member WISA
   * @information Create a WISA control by it's name
   * @param {name} name of control to create.
   * @return control instance
   * @author Michael Van Canneyt
   * @version 0.9
   */
  control : function(name) {
      var args = Array.prototype.slice.call(arguments);
      args[0] = 'WISA.controls.' + name;
      return Ext.create.apply(Ext,args);
  },
  /**
   * @method WISA.dataset
   * @member WISA
   * @information Create a WISA ClientDataset by it's name
   * @param {config} configuration for dataset
   * @return control instance
   * @author Michael Van Canneyt
   * @version 0.9
   */
  dataset : function(config) {
    return Ext.create('WISA.data.ClientDataset',config);
  },
  showError : function(title, message) {
    Ext.Msg.show({
        title : title,
        msg : message,
        icon : Ext.Msg.ERROR,
        buttons : Ext.Msg.OK
    });
  },

  showAjaxError : function(message) {
    WISA.showError('Ajax Error', message);
  },

  isValidResponse : function(response) {
   return (!Ext.isEmpty(response)) && (Ext.isEmpty(response.error)) && (Ext.isDefined(response.result));
  },

  getResponseError : function(provider, response, message) {
    var result = WISA.isValidResponse(response);
    if (!result) {
        if ((!Ext.isEmpty(response)) && (!Ext.isEmpty(response.error))) {
            message = message + response.error.message;
        } else {
            message = message + 'Interne server fout.';
        }
        WISA.showAjaxError(message);
    }
    return result;
  },

// /* Server Configuration */
  getServerConfiguration : function(options) {
    var urlProvider = Application.getUrlApplication() + '/Provider/' + options.ProviderName + '/';
    var objHttpProxy = new WISA.data.HttpProxy({
         // we gebruiken WISA. ipv Ext. voor de fix om het model in de store aan te passen met metadata van server
         writer: Ext.create('Ext.data.writer.Json',{
           allowSingle: true,
           encode: true,
           writeAllFields: true,
           root: 'rows'}),
         api : {
           read : urlProvider + 'Read/',
           update : urlProvider + 'Update/',
           create : urlProvider + 'Insert/',
           destroy : urlProvider + 'Delete/'
        }
    });
    // We expect metadata from the server.
    var objJsonReader = new Ext.data.JsonReader({
        root : 'rows',
        messageProperty : 'message',
        successProperty : 'success'
    });
    var objJsonWriter = new Ext.data.JsonWriter({
        encode : true,
        writeAllFields : true
    });
    return {
        reader : objJsonReader,
        writer : objJsonWriter,
        proxy : objHttpProxy
    };
  }
});

// /* Basic Components */

/**
 * @class WISA.controls.Label
 * @information The basic control to show non editable text on the form.
 * @extends Ext.form.Label
 */
Ext.define('WISA.controls.Label',{
  extend: 'Ext.form.Label'
});

/**
 * @class WISA.controls.TextField
 * @information The basic edit control for all WISA applications.
 * @extends Ext.form.TextField
 */
Ext.define('WISA.controls.TextField',{
  extend: 'Ext.form.field.Text',
  config : {
    minLengthText: 'De waarde moet minstens {minLength} tekens bevatten',
    maxLengthText: 'De waarde mag niet meer dan {maxLength} tekens bevatten',
    blankText: 'Dit is een verplicht veld'
  }
});

/**
 * @class WISA.controls.NumberField
 * @information The basic number control for all WISA applications.
 * @extends Ext.form.NumberField
 */
Ext.define('WISA.controls.NumberField',{
  extend: 'Ext.form.field.Number',
  config : {
    minLengthText: 'De waarde moet minstens {minLength} tekens bevatten',
    maxLengthText: 'De waarde mag niet meer dan {maxLength} tekens bevatten',
    blankText: 'Dit is een verplicht veld'
  }
});

/**
 * @class WISA.controls.DateField
 * @information The basic datum control for all WISA applications.
 * @extends Ext.form.DateField
 */
Ext.define('WISA.controls.DateField',{
  extend: 'Ext.form.DateField',
  constructor : function (options) {
    Ext.applyIf(options, {
      startDay : 1,
      value: new Date()
    });
    this.callParent(arguments);
  }
});

/**
 * @class WISA.controls.TimeField
 * @information The basic time control for all WISA applications.
 * @extends Ext.form.TimeField
 */
Ext.define('WISA.controls.TimeField',{
  extend: 'Ext.form.field.Time'
});

/**
 * @class WISA.controls.ComboBox
 * @information The basic combo box for all WISA applications.
 * @extends Ext.form.ComboBox
 */
Ext.define('WISA.controls.ComboBox',{
  extend: 'Ext.form.field.ComboBox',
  config: {
    triggerAction:'all',
    lazyInit: false,
    forceSelection: true,
    mode: 'local',
    valueNotFoundText : 'Waarde niet gevonden'
  }
});

/**
 * @class WISA.controls.CheckBox
 * @information The basic check box for all WISA applications.
 * @extends Ext.form.Checkbox
 */
Ext.define('WISA.controls.CheckBox',{
 extend: 'Ext.form.field.Checkbox'
});

/**
 * @class WISA.controls.DateField
 * @information The basic datum control for all WISA applications.
 * @extends Ext.form.Radio
 */
Ext.define('WISA.controls.Radio',{
  extend: 'Ext.form.field.Radio'
});

/**
 * @class WISA.controls.RadioGroup
 * @information The basic radio buttons for all WISA applications.
 * @extends Ext.form.RadioGroup
 */
Ext.define('WISA.controls.RadioGroup',{
  extend: 'Ext.form.RadioGroup',
  getSelectedValue : function () {
    var r=this.getChecked();
    if (r.length=1) {
      return r[0].getGroupValue();
    } else {
      return null
    }
  }
});

/**
 * @class WISA.controls.HtmlEditor
 * @information The basic HTML editor for all WISA applications.
 * @extends Ext.form.HtmlEditor
 */
Ext.define('WISA.controls.HtmlEditor',{
  extend: 'Ext.form.field.HtmlEditor',
});

/**
 * @class WISA.controls.TextArea
 * @information The basic memo control for all WISA applications.
 * @extends Ext.form.TextArea
 */
Ext.define('WISA.controls.TextArea',{
  extend:'Ext.form.field.TextArea'
});

/**
 * @class WISA.controls.Panel
 * @information The basic Panel for all WISA applications.
 * @extends Ext.Panel
 */
Ext.define('WISA.controls.Panel',{
  extend: 'Ext.form.Panel',
});

/**
 * @class WISA.controls.Button
 * @information The basic Button for all WISA applications.
 * @extends Ext.Panel
 */
Ext.define('WISA.controls.Button',{
  extend: 'Ext.button.Button'
});
/**
 * @class WISA.controls.FormButton
 * @information Button to show a form. Parameters include
 * FormName : name of the form.
 * GetParams : function to get form parameters.
 * GetConfig : function to return form build config.
 * @extends Ext.Panel
 * gebruikt getCurrentDescription om uitleg te geven in te tonen scherm
 * geeft de huidige formclass mee om terug te kunnen keren
 * geeft de huidige waarde van het IDField mee om naar het huidige record terug te kunnen keren met setCurrentID
 */
Ext.define('WISA.controls.FormButton',{
  extend: 'WISA.controls.Button',
  doShowForm: function (f) {
    Application.ShowForm(f);
  },
  openForm: function () {
    var f = { form: this.getFormName() };
    var o = this.getFormParams();
    var of = this.getParamsFn();
    var hf = this.getHeaderForm();
    if (!(Ext.isEmpty(o) && Ext.isEmpty(of))) {
      o = o || {};
      Ext.apply(o, Ext.isFunction(of) ? of() : {});
      Ext.apply(f, { Params : o });
    };
    o  = this.getFormConfig();
    of = this.getConfigFn();
    if (!(Ext.isEmpty(o) && Ext.isEmpty(of))) {
      o = o || {};
      Ext.apply(o, Ext.isFunction(of) ? of() : {});
      if (Ext.isEmpty(o.headerConfig) && ! Ext.isEmpty(hf)) {
        Ext.apply(o , { headerConfig : hf.getHeaderValue(this) });
      };
      Ext.apply(f, { BuildConfig : o });
    };
    this.doShowForm(f);
  },
  constructor : function(options) {
    var formName = '';
    var getparamsfn = null;
    var getconfigfn = null;
    var formparams = null;
    var formconfig = null;
    var headerform = null;
    Ext.apply(options, {
      handler: this.openForm,
      scope: this
    });
    if (!Ext.isEmpty(options.FormName)) {
      formName=options.FormName;
    };
    if (!Ext.isEmpty(options.FormParams)) {
      formparams=formparams || {};
      Ext.apply(formparams,options.FormParams);
    };
    if (!Ext.isEmpty(options.FormConfig)) {
      formconfig=formconfig || {};
      Ext.apply(formconfig,options.FormConfig);
    };
    if (Ext.isFunction(options.getParams)) {
      getparamsfn=options.getParams;
    };
    if (Ext.isFunction(options.getConfig)) {
      getconfigfn=options.getConfig;
    };
    if (Ext.isObject(options.headerForm)) {
      headerform = options.headerForm;
    };
    this.getFormName = function () {
      return formName;
    };
    this.getFormConfig = function () {
      return formconfig;
    };
    this.getHeaderForm = function () {
      return headerform;
    };
    this.getFormParams = function () {
      return formparams;
    };
    this.getFormName = function () {
      return formName;
    };
    this.setFormName = function (aValue) {
      formName = aValue;
    };
    this.getParamsFn = function () {
      return getparamsfn;
    };
    this.getConfigFn = function () {
      return getconfigfn;
    };
    // Superclass Constructor
    this.callParent(arguments);
  }
});
/**
 * @class WISA.baseforms.DialogButton
 * @information Button to show a Dialog. Parameters include
 * FormName : name of the form.
 * GetParams : function to get form parameters.
 * GetConfig : function to return form build config.
 * @extends Ext.Panel
 */
Ext.define('WISA.controls.DialogButton',{
  extend: 'WISA.controls.FormButton',
  doShowForm: function (f) {
    Application.ShowDialog(f);
  }
});

/**
 * @class WISA.controls.Grid
 * @information The basic grid for all WISA applications.
 * @extends Ext.grid.GridPanel
 * @author Michael Van Canneyt
 * @version 0.9
 */
Ext.define('WISA.controls.Grid',{
  extend: 'Ext.grid.Panel',
  alternateClassName: 'WISA.Grid',
  initComponent : function() {
    if (Ext.isBoolean(this.initialConfig.dynamicColumns) && (this.originalConfig.dynamicColumns)) {
      this.store.on("metachange", this.onMetaChange, this);
    };
    this.callParent(arguments);
  },
  onMetaChange : function(store, meta) {
    var columns = [];
    for(var i=0; i<meta.fields.length; i++){
      var c = Ext.Apply({},meta.fields[i]);
      if (Ext.isEmpty(c.dataIndex)) {
        c.dataIndex = c.name;
      }
      if(!c.id) {
        c.id = 'c' + i;
      }
      delete c.name;
      columns.push(c);
    }
    this.grid.reconfigure(this.store, new Ext.grid.ColumnModel(columns));
  }
});
/**
 * @class WISA.controls.EditorGridPanel
 * @information The basic editing grid for all WISA applications.
 * @extends Ext.grid.EditorGridPanel
 * @author Michael Van Canneyt
 * @version 0.9
 */
Ext.define('WISA.controls.EditorGridPanel',{
  extend: 'Ext.grid.Panel',
  config: {
    charThatStartedEdit : null,
    killValue: false,
    killedValue: null,
    selType: 'cellmodel',
    plugins: [
       Ext.create('Ext.grid.plugin.CellEditing', {
         clicksToEdit: 1
       })
    ],
  },
  initEvents : function(){
       this.callParent(arguments);
       this.on("keypress", this.handleKeyPressEvent, this);
       this.on("validateedit", this.handleValidateEdit, this);
  },
  handleValidateEdit : function (e) {
    if (!Ext.isEmpty(this.killedValue)) {
      e.value = this.killedValue;
      this.killedValue=null;
    }
   return true;
  },
  handleKeyPressEvent : function (e) {
      var code = e.getKey();
      if (code!=undefined)  {
        switch (code) {
          case e.LEFT :
          case e.RIGHT :
          case e.UP :
          case e.DOWN :
          case e.ESC :
          case e.ENTER :
          case e.TAB :
          case e.SHIFT :
          case e.PAGEUP :
          case e.PAGEDOWN :
          case e.HOME:
          case e.F5 :
          case e.END :
          case e.DELETE :
          case e.CONTROL :
          case e.BACKSPACE :
            break;
          default :
            this.charThatStartedEdit = code;
            var sc = this.selModel.getSelectedCell();
            var col = sc[1];
            var row = sc[0];
            this.startEditing(row, col);
      }
    }
    return true;
  },
  constructor : function (options) {
    if (!Ext.isDefined(options.autoEdit)) {
      Ext.apply(options,{autoEdit : true});
    }
    this.callParent(arguments);
  },
  preEditValue : function(r, field) {
    if (!Ext.isEmpty(this.charThatStartedEdit)) {
      var r = String.fromCharCode(this.charThatStartedEdit);
      this.charThatStartedEdit=null;
      this.killValue=true;
      return r;
    } else {
      return this.callParent(arguments);
    }
  },
  postEditValue : function(value,originalValue,r, field) {
    if (this.killValue) {
      this.killedValue=value;
      this.killValue=false
      return '';
    } else {
      this.killedValue=null;
      return this.callParent(arguments);
    }
  }
});

Ext.override(Ext.selection.CellModel, {
    onEditorKey : function(field, e){
        var k = e.getKey(), newCell, g = this.grid, ed = g.activeEditor;
        if(k == e.TAB){
            if(e.shiftKey){
                newCell = g.walkCells(ed.row, ed.col-1, -1, this.acceptsNav, this);
            }else{
                newCell = g.walkCells(ed.row, ed.col+1, 1, this.acceptsNav, this);
            }
            e.stopEvent();
        }else if(k == e.ENTER){
            var sc = g.selModel.getSelectedCell();
            var c = sc[1];
            var r = sc[0]+1;
            if (!Ext.isEmpty(ed)) {
              ed.completeEdit();
            }
            e.stopEvent();
            newCell = g.walkCells(r, c, 1, this.acceptsNav, this);
        }else if(k == e.ESC){
            e.stopEvent();
            ed.cancelEdit();
            g.getView().focusCell(ed.row, ed.col);
        }else if(k == e.DOWN){
            ed.completeEdit();
            e.stopEvent();
            newCell = g.walkCells(ed.row+1, ed.col, 1, this.acceptsNav, this);
        }else if(k == e.UP){
            ed.completeEdit();
            e.stopEvent();
            newCell = g.walkCells(ed.row-1, ed.col, -1, this.acceptsNav, this);
        }
        if(newCell){
            g.startEditing(newCell[0], newCell[1]);
        }
    }
});

/**
 * @class WISA.controls.Form
 * @information The basic form panel for all WISA applications.
 * @extends Ext.form.Panel
 * @author Michael Van Canneyt
 * @version 0.9
 */
Ext.define('WISA.controls.Form',{
  extend: 'Ext.form.Panel',
  alternateClassName: 'WISA.controls.FormPanel'
});

Ext.define('WISA.controls.FileUploadPanel',{
  extend: 'WISA.controls.Form',
  constructor : function (options) {
    var fileID = null;
    this.addEvents({
      'fileUploaded' : true,
      'fileUploadFailed' : true
    });
    Ext.applyIf(options, {
      title: 'Bestand verzenden',
      fileUpload: true,
      autoHeight: true,
      frame: true,
      bodyStyle: 'padding: 10px 10px 0 10px;',
      labelWidth: 50,
      width: 500
    });
    this.getFileID = function () {
      return fileID;
    };
    this.fileUploaded = function (f,o) {
      fileID = o.result.fileID;
      this.fireEvent('fileUploaded',this,o.result.fileID);
    };
    this.fileUploadFailed = function (f,o) {
      fileID = null;
      WISA.showError('Fout bij verzenden bestand', o.result.errors.message);
      this.fireEvent('fileUploadFailed',this, o.result.errors);
    };
    var o = options.items || [];
    o.push({
      xtype: 'fileuploadfield',
      emptyText: Ext.isEmpty(options.fileEmptyText) ? 'Selecteer een bestand' : options.fileEmptyText,
      fieldLabel: Ext.isEmpty(options.fileFieldLabel) ? 'Selecteer een bestand' : options.fileFieldLabel,
      name: Ext.isEmpty(options.fileFieldName) ? 'uploadfile' : options.fileFieldName,
      buttonText: '',
      buttonCfg: {
       iconCls: 'upload-icon'
        }
    });
    Ext.apply(options,{
      items: o,
      buttons: [{
            id : 'btn-zenden',
            iconCls : 'btn-zenden',
            text: 'Zenden',
            scope : this,
            handler: function(){
              var fp = this.getForm();
              if(fp.isValid()){
	        fp.submit({
	          url: Application.getFileUploadURL(),
	          waitMsg: Ext.isEmpty(options.fileUploadMessage) ? 'Bestand wordt verzonden...' : options.fileUploadMessage,
	          success: Ext.bind(this.fileUploaded,this),
                  failure: Ext.bind(this.fileUploadFailed,this)
                });
              }
            }
        },{
          id : 'btn-leegmaken',
          iconCls : 'btn-leegmaken',
          text: 'Leegmaken',
          scope : this,
          handler: function(){
            this.getForm().reset();
          }
        }]
    });
    // Superclass Constructor
    this.callParent(arguments);
  }
});

Ext.define('WISA.controls.Cards',{
  extend : 'WISA.controls.Panel',
  config : {
    layout: 'card'
  }
});


/**
 * @class WISA.controls.LookupComboBox
 * @information Lookup combo box with built-in store.
 * @extends Ext.form.LookupComboBox
 * @author Michael Van Canneyt
 * @version 0.9
 */
Ext.define('WISA.controls.LookupComboBox',{
  extend: 'WISA.controls.ComboBox',
  constructor : function(options) {
     var mystore = WISA.dataset({
        ProviderName : options.ProviderName,
        autoLoad: Ext.isEmpty(options.autoLoad) ? true : options.autoLoad,
        baseParams: Ext.isEmpty(options.baseParams) ? null : options.baseParams,
        listeners : {
          'load': { fn: this.storeLoaded, scope : this }
        }
      });
    var defaultID = null;
    if (Ext.isDefined(options.defaultID)) {
      defaultID=options.defaultID;
    };
    Ext.apply(options,{
      store : mystore
      //valueNotFoundText : '(Waarde niet gevonden)'  // zit nu in ext-lang-wisa.js
    });
    // Getter for DefaultID
    this.getDefaultID = function () {
      return defaultID;
    };
    // Setter for DefaultID
    this.setDefaultID = function (value) {
      defaultID=value;
      checkDefaultID();
    };
    // Superclass Constructor
    this.callParent(arguments);
    // Triggered when schools have been loaded.
  },
  // Triggered when store has been loaded.
  storeLoaded : function (store,records, options) {
    if (records.length == 1) {
      this.setValue(records[0].get(this.initialConfig.valueField));
      this.fireEvent('select', this, records[0], 0);
    } else {
      this.checkDefaultID();
    }
  },
  // Used to check if a default ID must/can be loaded
  checkDefaultID : function  () {
    var s = this.getStore();
    if (( !Ext.isEmpty(this.getDefaultID())) && (s.records.length>0)) {
      var i = s.findExact(this.initialConfig.valueField,this.getDefaultID());
      if (i != -1) {
        this.setValue(this.getDefaultID());
        this.fireEvent('select', this, records[i], i);
      }
    }
  }
});

/**
* Fix voor het niet overnemen van de metadata van de server in het model van de datastore
* gevonden op http://www.sencha.com/forum/showthread.php?132015-Stores-using-implicit-models-initialized-via-metaData-broken-for-anything-but-reads&highlight=metadata
*
Ext.define('Fixed.data.proxy.Rest', {
    alias: 'proxy.rest-fixed',
    extend: 'Ext.data.proxy.Rest',

    setModel: function(model, setOnStore) {
        this.callParent([model, setOnStore]);

        // This is a hack for Ext JS 4.0 to make sure that an implicit model
        // is available for the store to use (otherwise store.add won't work)
        if (setOnStore) {
            this.metaStore.model = model;
        }
    }
});

Ext.define('Fixed.data.Store', {
    extend: 'Ext.data.Store',

    constructor: function(config) {
        config.proxy = Ext.apply({
            type: 'rest-fixed',
            metaStore: this
        }, config.proxy);

        this.callParent([config]);
    }
});
*
*
*
*/

Ext.define('WISA.data.HttpProxy', {
    alias: 'proxy.wisa-proxy',
    extend: 'Ext.data.HttpProxy',

    setModel: function(model, setOnStore) {
        this.callParent([model, setOnStore]);

        // This is a hack for Ext JS 4.0 to make sure that an implicit model
        // is available for the store to use (otherwise store.add won't work)
        if (setOnStore) {
            this.metaStore.model = model;
        }
    }
});

/**
 * @class WISA.data.ClientDataset
 * @extends Ext.data.Store
 * @information The basic data store for all WISA applications. <br />
 *              Config is an object that must contain
 *              <ul>
 *              <li>ProviderName : String with name of provider.</li>
 *              <li>Fields: List of fieldnames to expect.</li>
 *              <li>IDField : String with name of ID field of data.</li>
 *              </ul>
 * @author Michael Van Canneyt
 * @version 0.9
 */
Ext.define('WISA.data.ClientDataset',{
  extend: 'Ext.data.Store',
  alternateClassName: 'WISA.ClientDataset',
  doOnException: function (proxy, res, action, arg) {
    var resp;
    var ishtml = false;
    if ((Ext.isEmpty(res.responseText)?0:res.responseText.length) > 0) {
      ishtml = !Ext.isEmpty(res.responseText.match(/^\<html\>/i)) &&
               !Ext.isEmpty(res.responseText.match(/\<\/html\>$/i));
    }
    if (ishtml) {
      var msgwindow=window.open('','msgwindow','height=150,width=450');
	msgwindow.document.write(res.responseText);
	msgwindow.document.close();
    } else {
      var m = 'Ongekende fout bij ' + action.action;
      if (!Ext.isEmpty(res.message)) {
        m = res.message;
      } else if (!Ext.isEmpty(res.responseText)) {
        eval('resp='+res.responseText);
        if (!Ext.isEmpty(resp.message)) {
          m = resp.message;
        } else {
          m = 'Geen fout bericht gekregen';
        }
      }
      WISA.showError('Fout bij verwerken data', m);
    }
  },
  load: function (options) {
    var me = this;
    me.callParent(arguments);
    me.lastOptions = options;
  },
  // Constructor
  constructor : function(options) {
    var optionsServer = WISA.getServerConfiguration(options);
    // Merge data
    Ext.apply(options, optionsServer);
    options.proxy.metaStore = this;  // voor de fix om het model in de store aan te passen met metadata van server
    options.proxy.on('exception', this.doOnException);

    this.callParent(arguments);

  }
});

// No longer needed, store now contains all it needs.
Ext.define('WISA.data.GroupingClientDataset',{
  extend: 'WISA.data.ClientDataset'
});


/**
 * @class WISA.baseforms.Dialog
 * @information The basic dialog for all WISA applications.
 * @extends Ext.Window
 * @author Michael Van Canneyt
 * @version 0.9
 */
Ext.define('WISA.baseforms.Dialog',{
   extend: 'Ext.Window',
   alternateClassName: 'WISA.Dialog',
   initComponent : function() {
        var bc = this.initialConfig.BuildConfig || {};
        var options = this.BuildForm(bc);
        if (!Ext.isEmpty(options)) {
            Ext.apply(this, options);
        }
        this.callParent(arguments);
    },
    BuildForm : function(config) {
        return null;
    },
    ApplyParams : function(params) {
        // Apply any parameters you see fit
    }
});

/**
 * @class WISA.baseforms.Form
 * @information The basic form for all WISA applications.
 * @extends Ext.Panel
 * @author Michael Van Canneyt
 * @version 0.9
 */
Ext.define('WISA.baseforms.Form',{
    extend: 'Ext.panel.Panel',
    alternateClassName: 'WISA.Form',
    /**
     * @method BuildForm
     * @information The BuildForm method creates the actual GUI. This method must be overridden and must build the FORM
     *              gui. It cannot perform any server calls.
     * @author Michael Van Canneyt
     */
    getFormTitle : function() {
        return 'Onbekend scherm';
    },
    initComponent : function() {
        var bc = this.initialConfig.BuildConfig || {};
        var options = this.BuildForm(bc);
        if (!Ext.isEmpty(options)) {
            Ext.apply(this, options);
        }
        this.callParent(arguments);
    },
    BuildForm : function(config) {
        return null;
    },
    ApplyParams : function(params) {
        // Apply any parameters you see fit
    }
});

Ext.define('WISA.BaseConfigPanel',{
  extend: 'WISA.controls.Form',
  setConfig: function (cfg) {
    this.getForm().setValues(cfg);
  },
  getConfig: function (cfg) {
    Ext.apply(cfg,this.getForm().getFieldValues(true));
  }
});

// /* Application */
/**
 * @class WISA.Application
 * @information De WISA Application class: er wordt slechts 1 instance van deze class gemaakt.
 * @extends Ext.util.Observable
 * @author Michael Van Canneyt
 * @version 0.9
 */
Ext.define('WISA.Application',function() {
    // /*** Private Static Properties [ var = ] ***/
    // /*** Private Static Methods [ var = ] ***/
    return {
        extend: 'Ext.util.Observable',
        constructor : function(options) {
            // /*** Below has Access to All Properties/Methods ***/
            // /*** Public Properties [ this.prop = ] ***/
            // none

            // /*** Private Properties [ var prop = ] ***/
            var isAPILoaded = false;
            var isCGI = false;
            var isThemeComboVisible = false;
            var isBorderLayout = true;
            var isMainmenuVisible = true;
            var isSubmenuVisible = true;
            var urlBase = '.';
            var urlApplication = '';
            var userId = -1;
            var userName = '';
            var userLastlogin = '(onbekend)';
            var menu = null;
            var submenuPanel = null;
            var submenuConfig = {
                region : 'west',
                id : 'wisa-submenu',
		cls: 'wisa-submenu',
                collapsible : true,
                collapsed : true,
                layout : 'fit',
                title : 'Menu',
                margins : '0 3 3 3',
		cmargins: '0 3 3 3',
                width : 249,
                height: 500
            };
            var FMainMenuConfig = null;
            var bottomPanelConfig = null;
            var formMain = 'motd';
            var formParent = null;
            var dialogActive = null;
            var sessionConfig = null;
            var buildsubmenucallback = null;
            var fileUploadURL = 'bin/filemanager.cgi/upload/put';
            var configPanels = new Ext.util.MixedCollection(true);

            // /*** Public Methods [ this.fct = ] ***/ // & //
            // /*** Private Methods [ var fct = ] ***/
            // // Setters and Getters
            var setIsAPILoaded = function(value) {
                isAPILoaded = value;
            };
            var getIsAPILoaded = function() {
                return isAPILoaded;
            };
            var setIsCGI = function(value) {
                isCGI = value;
            };
            var getIsCGI = function() {
                return isCGI;
            };
            var setIsThemeComboVisible = function(value) {
                isThemeComboVisible = value;
            };
            var getIsThemeComboVisible = function() {
                return isThemeComboVisible;
            };
            var setIsBorderLayout = function(value) {
                isBorderLayout = value;
            };
            var getIsBorderLayout = function() {
                return isBorderLayout;
            };
            var setIsMainmenuVisible = function(value) {
                isMainmenuVisible = value;
            };
            var getIsMainmenuVisible = function() {
                return isMainmenuVisible;
            };
            var setMainMenuConfig = function (value) { 
                FMainMenuConfig = Ext.apply(value,{
                    region: 'north', 
                    collapsible: true
                }); 
            };
            this.getMainMenuConfig = function () { 
                return FMainMenuConfig;
            };
            var setIsSubmenuVisible = function(value) {
                isSubmenuVisible = value;
            };
            var getIsSubmenuVisible = function() {
                return isSubmenuVisible;
            };
            var setUrlBase = function(value) {
                urlBase = value;
            };
            var getUrlBase = function() {
                return urlBase;
            };
            var setUrlApplication = function(value) {
                urlApplication = value;
            };
            this.getFileUploadURL = function () {
              return fileUploadURL;
            }
            this.getUrlApplication = function() {
                if (urlApplication !== '') {
                    return urlApplication;
                } else {
                    if (getIsCGI()) {
                        urlApplication = getUrlBase() + '/server.cgi';
                    } else {
                        urlApplication = getUrlBase() + '/server';
                    }
                    return urlApplication;
                }
            };
            this.getUrlAPI = function() {
                return (this.getUrlApplication() + '\/RPC\/API\/');
            };
            this.getUrlDialogs = function() {
                return (this.getUrlApplication() + '\/' + 'Forms\/');
            };
            this.getUrlForms = function() {
                return (this.getUrlApplication() + '\/' + 'Forms\/');
            };
            var setUserData = function(data) {
                userId = data.userid;
                userName = data.name;
                userLastlogin = data.lastlogin;
            };
            var setSessionConfig = function(data) {
                sessionConfig = data;
            };
            this.getUserId = function() {
                return userId;
            };
            this.getSessionConfig = function() {
                return sessionConfig;
            };
            this.setBuildSubMenu = function(value) {
                buildsubmenucallback = value;
            };
            this.getBuildSubMenu = function() {
                return buildsubmenucallback;
            };
            this.getUserName = function() {
                return userName;
            };
            this.getUserLastLogin = function() {
                return userLastlogin;
            };
            var setMenu = function(value) {
                menu = value;
            };
            this.getMenu = function() {
                return menu;
            };
            var setSubmenuPanel = function(value) {
                submenuPanel = value;
            };
            this.getSubmenuPanel = function() {
                return submenuPanel;
            };
            this.setSubmenuConfig = function(options) {
                submenuConfig = Ext.apply(options, {
                    region : 'west',
                    collapsible : false
                });
            };
            this.getSubmenuConfig = function() {
                return submenuConfig;
            };
            this.setBottomPanelConfig = function (value) {
                 bottomPanelConfig = Ext.apply(value,{
                     region: 'south', 
                     collapsible: false
                 });
            };
            this.getBottomPanelConfig = function () { 
                return bottomPanelConfig;
            };
            var setFormMain = function(value) {
                formMain = value;
            };
            this.getFormMain = function() {
                return formMain;
            };
            var setFormParent = function(value) {
                formParent = value;
            };
            var getFormParent = function() {
                return formParent;
            };
            var setDialogActive = function(object) {
                dialogActive = object;
            };
            var getDialogActive = function() {
                return dialogActive;
            };
            // // API
            var checkIsAPILoaded = function() {
                if (true !== getIsAPILoaded()) {
                    return false;
                } else {
                    return true;
                }
            };
            var setLoadAPISuccess = function(response, options) {
                try {
                    eval(response.responseText);
                } catch (err) {
                    WISA.showAjaxError('AJAX error during API Load: ' + err.message);
                }
                Ext.Direct.addProvider(WISA.API);
                setIsAPILoaded(true);
                this.fireEvent('APILoaded');
            };
            var setLoadAPIFailure = function() {
                window.alert('WISA API could not be loaded');
            };
            this.loadAPI = function() {
                Ext.Ajax.request({
                    url : this.getUrlAPI(),
                    success : setLoadAPISuccess,
                    failure : setLoadAPIFailure,
                    scope : this
                });
            };
            this.buildMenu = function() {
                var o = [];
                var mnu = this.getMenu();
                if (mnu) {
                    for ( var m = 0; m < mnu.Items.length; m += 1) {
                        var mi = mnu.Items[m];
                        o[m] = {
                            xtype : 'button',
                            id : 'wisa-mainmenu-' + mi.Form,
                            text : mi.Caption,
                            iconCls : mi.Icon,
                            tooltip : mi.Hint,
                            overCls : 'wisa-mainmenu-' + mi.Form + '-hover',
                            height  : 71,  
                            handler : (function(mi) {
                                var it = mi;
                                var cb = Application.getBuildSubMenu();
                                var d = false;
                                return function(e) {
                                    if (Ext.isFunction(cb)) {
                                        d=cb(it);
                                    } else {
                                        d=false;
                                    };
                                    if (!(d || Ext.isEmpty(it.Items))) {
                                        this.showSubmenu(it);
                                    } else {
                                        var p = this.getSubmenuPanel();
                                        p.setTitle("");
                                        p.setIconCls('no-icon');
                                        p.removeAll(true);
                                    }
                                    this.showForm({
                                        form : it.Form
                                    });
                                };
                            }(mi)),
                            scope : this
                        };
                    }
                }
                o[o.length] = {
                    xtype : 'button',
                    text : 'Help',
                    cls : 'wisa-help-button',
                    tooltip : 'Help',
                    handler : function() {
                        alert('help');
                    },
                    scope : this
                };
                o[o.length] = '->';
                if (getIsThemeComboVisible()) {
                    o[o.length] = {
                        xtype : 'themecombo'
                    };
                }
                o[o.length] = {
                    xtype : 'button',
                    text : this.getUserName(),
                    cls : 'wisa-profile',
                    iconCls : 'profile',
                    tooltip : 'Mijn gebruikersgegevens beheren',
                    handler : function() {
                        this.showForm({
                            form : 'myprofile',
                            hideSubmenu : true
                        });
                    },
                    scope : this
                };
                o[o.length] = {
                    xtype : 'button',
                    text : 'Afmelden',
                    cls : 'wisa-logout',
                    iconCls : 'logout',
                    tooltip : 'Afmelden en terug naar het aanmeldscherm',
                    handler : function() {
                        this.logout();
                    },
                    scope : this
                };
                return o;
            };
            this.createMainForm = function() {
                var it = [];
                Ext.QuickTips.init();
                setFormParent(Ext.widget('panel',{
                    id    : "wisa-center",  // DEVart : added id
                    title : 'A parent',
                    height : 500,
                    width : 500,
                    border : false,
                    region : 'center',
		    cls: 'wisa-center',
                    margins : '0 3 3 0',
		    cmargins: '0 3 3 0',
                    layout : 'fit'
                }));
                if (getIsMainmenuVisible()) {
                    var t = this.getMainMenuConfig();
                    if (Ext.isEmpty(t)) {
                        t = {
                            xtype : 'toolbar',
                            region : 'north',
			    cls: 'wisa-top-toolbar',
			    margins: '0 0 0 0',
                            height : 89,
                            items : this.buildMenu()
                        };
                    };
                    it[it.length] = t;
                };
                if (getIsSubmenuVisible()) {
                    var s = this.getSubmenuConfig();
                    //Ext.apply(s, {height:50, width:50});
                    setSubmenuPanel(Ext.widget('panel',s));
                    it[it.length] = this.getSubmenuPanel();
                }
                var c = this.getBottomPanelConfig();
                if (! Ext.isEmpty(c)) {
                  it[it.length]=c;
                }
                it[it.length] = getFormParent();
                if (getIsBorderLayout()) {
                    this.ViewPort = Ext.widget('viewport',{
                        layout : 'border',
                        items : it
                    });
                } else {
                    if (getIsMainmenuVisible()) {
                      t.width = 1019;
                    }
                    var sm = this.getSubmenuConfig();
                    sm.collapsed = false;
                    sm.collapsible = false;
                    sm.flex  = 1;
                    sm.autoScroll = true;
                    sm.width = 50;
                    sm.heigth = 50;
                    setSubmenuPanel(Ext.widget('panel',sm));
                    this.ViewPort = Ext.widget('viewport', {
                        layout : 'vbox',
                        layoutConfig : {
                            align : 'center'
                        },
                        width : 1050,
                        heigth : 1000,
                        items : [
                            t
                            ,{
                                id    : 'content-wrap',
                                layout: 'hbox',
                                border : false,
                                layoutConfig:{
                                    align:'stretch'
                                },
                                flex  : 1,
                                width : 1019,
                                height : 1000,
                                items : [
                                    this.getSubmenuPanel(), 
                                    {
                                        border : false,        
                                        width  : 764,         
                                        autoScroll : true,
                                        items: [{
                                            xtype : 'panel',
                                            style: 'margin-left: 26px; margin-top:32px;',
                                            width : 720,
                                            height : 480,
                                            border: false,
                                            items: [                                            
                                                getFormParent()
                                            ]
                                        }]
                                    }                                    
                                ]
                            },
                            this.getBottomPanelConfig()
                        ]
                    });
                }
            };
            // DEVart add function for hiding submenu
            this.hideSubmenu = function() {
                var p = this.getSubmenuPanel();
                p.setTitle(false);
                p.setIconCls('no-icon');
                p.header.hide();
                p.removeAll(true);
            };
            // DEVart end
            
            this.showSubmenu = function(menu) {
                var p = this.getSubmenuPanel();
                var ts = Ext.create('Ext.data.TreeStore',{
                   root: {
                     text : menu.Caption,
                     expanded : true,
                     children : []
                   }
                });
                p.setTitle(menu.Caption);
                p.setIconCls('icon-' + menu.Form);
                p.removeAll(true);
                var r = ts.getRootNode();
                for ( var m = 0; m < menu.Items.length; m += 1) {
                    var mi = menu.Items[m];
                    var o = {
                        cls : 'wisa-submenu-' + mi.Form,
                        text : mi.Caption,
                        iconCls : mi.Icon,
                        leaf : true,
                        qtip : mi.Hint,
                        Form : mi.Form
                    };
                    r.appendChild(o);
                }
                var t = Ext.create('Ext.tree.Panel',{
                    store: ts,
                    width: 120,
                    height: 100,
                    rootVisible : false,
                    listeners : {
                       'itemclick' : {
                         fn : function(view, record, item, index, e, eOpts ){
                                this.showForm({form : record.data.Form});
                              },
                         scope : this
                       },
                     }
                });
                p.add(t);
                if (p.collapsed) {
                    p.toggleCollapse();
                }
                p.doLayout();
            };
            // // Dialogs and Forms
            this.showDialog = function(options) {
                if ((Ext.isEmpty(Ext.ClassManager.get('WISA.Forms.'+options.form))) && ('noloop' !== arguments[arguments.length - 1])) {
                    /* */console.log('Loading dialog ' + options.form + '.js');
                    options = Ext.apply(options, {
                        url : this.getUrlDialogs() + options.form + '.js',
                        success : setLoadDialogSuccess,
                        failure : setLoadDialogFailure,
                        scope : this
                    });
                    Ext.Ajax.request(options);
                } else {
                    var c = {};
                    if (Ext.isObject(options.BuildConfig)) {
                      c['BuildConfig'] = options.BuildConfig;
                    }
                    var objForm = Ext.create('WISA.Forms.'+options.form,c);
                    setDialogActive(objForm);
                    objForm.show();
                    objForm.ApplyParams(options.Params);
                    //objForm.focus();
                }
            };
            var setLoadDialogSuccess = function(response, options) {
                try {
                    eval(response.responseText);
                } catch (err) {
                    WISA.showAjaxError('AJAX error : ' + err.message);
                }
                this.fireEvent('DialogLoaded', options.form); // TODO Check Change: dialog dependant
                if (Ext.isEmpty(WISA.Forms[options.form])) {
                    WISA.showAjaxError('"' + options.form + '" did not define a valid dialog');
                } else {
                    if (!Ext.isFunction(WISA.Forms[options.form])) {
                        WISA.showAjaxError('"' + options.form + '" is not a valid form constructor');
                    } else {
                        this.showDialog(options, 'noloop');
                    }
                }
            };
            var setLoadDialogFailure = function(response, options) {
                window.alert('Kan scherm \"' + options.form + '\" niet openen: ' + response.statusText);
            };
            var closeDialogActive = function() {
                if (getDialogActive()) {
                    getDialogActive().close();
                }
            };
            this.showForm = function(options) {
                if (Ext.isDefined(options.hideSubmenu) && options.hideSubmenu === true) {
                    this.hideSubmenu();
                }
                if (Ext.isEmpty(getFormParent())) {
                    this.createMainForm();
                }
                if ((Ext.isEmpty(Ext.ClassManager.get('WISA.Forms.'+options.form))) && ('noloop' !== arguments[arguments.length - 1])) {
                    /* */console.log('Loading form ' + options.form + '.js');
                    options = Ext.apply(options, {
                        url : this.getUrlForms() + options.form + '.js',
                        success : setLoadFormSuccess,
                        failure : setLoadFormFailure,
                        scope : this
                    });
                    Ext.Ajax.request(options);
                } else {
                    if ('noloop' == arguments[arguments.length - 1]) {
                        var clear = true;
                        var c = {};
                        if (Ext.isObject(options.BuildConfig)) {
                            c['BuildConfig'] = options.BuildConfig;
                        }
                        closeDialogActive();
                        var objForm = Ext.create(WISA.Forms[options.form], c);
                        var objFormParent = getFormParent();
                        if (clear) {
                            objFormParent.removeAll(true);
                        }
                        objFormParent.add(objForm);
                        objFormParent.doLayout();
                        objForm.ApplyParams(options.Params);
                        var title = objForm.getFormTitle();
                        if (title === false || title.length <= 0) {
                            objFormParent.header.hide();
                        } else {
                           objFormParent.header.show();
                           objFormParent.setTitle(title); 
                        }
                        objFormParent.setTitle(objForm.getFormTitle(options.BuildConfig));
                        if (Ext.isDefined(objForm.iconCls)) {
                          objFormParent.setIconCls(objForm.iconCls || 'no-icon');
                        }
                        return false; // stop task.
                    } else { // MVC: Workaround voor IE resource/memory leak. Niet dadelijk nieuwe form tonen, maar in timed event.
                      var t = new Ext.util.DelayedTask(this.showForm,this,[options,'noloop'])
                      t.delay(100);
/*
                      Ext.TaskManager.start({
                        run: this.showForm,
                        args: [options,'noloop'],
                        interval: 100,
                        scope: this,
                        repeat: 1
                      });
*/
                    }
                }
            };
            var setLoadFormSuccess = function(response, options) {
                try {
                    eval(response.responseText);
                } catch (err) {
                    WISA.showAjaxError('AJAX error : ' + err.message);
                }
                this.fireEvent('FormLoaded', options.form); // TODO Check Change: form dependant
                if (Ext.isEmpty(Ext.ClassManager.get('WISA.Forms.'+options.form))) {
                    WISA.showAjaxError('"' + options.form + '" did not define a valid form');
                } else {
                    this.showForm(options, 'noloop');
                }
            };
            var setLoadFormFailure = function(response, options) {
                window.alert('Kan scherm \"' + options.form + '\" niet openen: ' + response.statusText);
            };
            // // User Management
            this.login = function(options) {
                WisaUserAPI.Authenticate(options.user, options.password, Ext.bind(handleLogin,this));
            };

            this.showLogin = function(options) {
                if (checkIsAPILoaded()) {
                    this.showDialog(Ext.apply({
                        form : 'login'
                    }, options));
                } else {
                    /* */console.log('Waiting for API...');
                    this.addListener('APILoaded', function() {
                        this.showLogin(options);
                    }, this, {
                        single : false
                    });
                }
            };
            var realLogout = function () {
                var strLogoutURL = getUrlBase() + '/start.html';
                WisaUserAPI.Logout(function() {
                    window.location = strLogoutURL;
                });
            };
            /* Auto-logout functionality */
            var autoRefreshProcId = 0;
            var autoLogout = function() {
              realLogout();
            };
            var startLogoutTimer = function() {
                if (autoRefreshProcId) {
                    clearInterval(autoRefreshProcId);
                }
                autoRefreshProcId = setInterval(Ext.bind(autoLogout,this), (Application.getSessionConfig().SessionTimeOut - 1)*60000);
            };
            var stopLogoutTimer = function(){
               if(autoRefreshProcId){
                  clearInterval(autoRefreshProcId);
               }
            };
            var handleLogin = function(provider, response) {
                if (!Ext.isEmpty(response.error)) {
                    WISA.showAjaxError('Fout bij aanmelden: ' + response.error.message);
                } else {
                    if (!Ext.isEmpty(response.result.SessionConfig)) {
                      setSessionConfig(response.result.SessionConfig);
                      if (!Ext.isEmpty(this.getSessionConfig().SessionTimeOut)) {
                        startLogoutTimer();
                        Ext.Ajax.on('beforerequest', startLogoutTimer, this);
                      }
                    }
                    var l = Ext.getCmp('wisa-login-logo');
                    if (Ext.isObject(l)) l.destroy(); // DEVart hide logo
                    setUserData(response.result.UserData);
                    setMenu(response.result.menu);
                    closeDialogActive();
                    if (this.hasListener('Login')) {
                        this.fireEvent('Login');
                    } else {
                        this.showForm({
                            form : this.getFormMain()
                        });
                    }
                }
            };
            this.logout = function() {
               Ext.MessageBox.confirm('Afmelden','Bent u zeker dat u de sessie wil afbreken ?',
                 function (bid) {  
                   if (bid=='yes') {
                     realLogout();
                   }
                }, 
                this
               );
            };
            this.getConfigPanels = function () {
              return configPanels;
            }
            // Compatibility
            this.getShowThemeCombo = this.getIsThemeComboVisible;
            this.setShowMainMenu = this.setIsMainmenuVisible;
            this.getShowMainMenu = this.getIsMainmenuVisible;
            this.setShowSubMenu = this.setIsSubmenuVisible;
            this.getShowSubMenu = this.getIsSubmenuVisible;
            this.getBaseURL = this.getUrlBase;
            this.getAppURL = this.getUrlApplication;
            this.saveUserData = this.setUserData;
            this.getUserID = this.getUserId;
            this.getLastLogin = this.getUserLastLogin;
            this.setSubMenuConfig = this.setSubmenuConfig;
            this.getSubMenuConfig = this.getSubmenuConfig;
            this.setMainForm = this.setFormMain;
            this.getMainForm = this.getFormMain;
            this.setActiveDialog = this.setDialogActive;
            this.getActiveDialog = this.getDialogActive;
            this.closeActiveDialog = this.closeDialogActive;
            this.ShowDialog = this.showDialog;
            this.ShowForm = this.showForm;
            this.ShowLogin = this.showLogin;
            this.HandleLogin = this.handleLogin;
            this.DoLogin = this.login;
            this.DoLogout = this.logout;
            this.CreateMainForm = this.createMainForm;
            this.BuildMenu = this.buildMenu;

            // Constructor Code
            // // Configuration
            if (Ext.isDefined(options.url)) {
              if (Ext.isDefined(options.url.base)) {
                  setUrlBase(options.url.base);
              }
              if (Ext.isDefined(options.url.base)) {
                  setUrlBase(options.url.base);
              }
              if (Ext.isDefined(options.url.app)) {
                  setUrlApplication(options.url.app);
              }
            }
            if (Ext.isDefined(options.hasThemes)) {
                setIsThemeComboVisible(options.hasThemes);
            }
            if (Ext.isDefined(options.bottomPanelConfig)) {
                this.setBottomPanelConfig(options.bottomPanelConfig);
            }
            if (Ext.isDefined(options.isBorderLayout)) {
                setIsBorderLayout(options.isBorderLayout);
            }
            if (Ext.isDefined(options.mainmenu)) {
              if (Ext.isDefined(options.mainmenu.visible)) {
                setIsMainmenuVisible(options.mainmenu.visible);
              }
              if (Ext.isDefined(options.mainmenu.config)) {
                setMainMenuConfig(options.mainmenu.config);
              }
            }
            if (Ext.isDefined(options.submenu)) {
              if (Ext.isDefined(options.submenu.visible)) {
                  setIsSubmenuVisible(options.submenu.visible);
              }
              if (Ext.isDefined(options.submenu.config)) {
                  this.setSubmenuConfig(options.submenu.config);
              }
            }
            if (Ext.isDefined(options.mainform)) {
              if (Ext.isDefined(options.mainform.name)) {
                  setFormMain(options.mainform.name);
              }
            } 

            // // Events
            this.addEvents({
                'APILoaded' : true,
                'FormLoaded' : true,
                'DialogLoaded' : true,
                'Login' : true,
                'Logout' : true
            });
            // // Superclass Constructor
            WISA.Application.superclass.constructor.call(this, options);
            var cp = new Ext.state.CookieProvider({
               path: "/",
               domain: location.host
            });
            Ext.state.Manager.setProvider(cp);
            // // Initialisation
            this.loadAPI();
        }
    // /*** Below has Access to All except Constructor ***/
    // /*** Public Properties [ prop : ] ***/
    // /*** Public Methods [ fct : ] ***/
    }
}());

Ext.define('WISA.controls.SingletonApplication',
  function() {
    // Private Hidden Property - stays alive as long as reference exists
    var objSingletonInstance = null;
    // Return Object
    return {
        extend: 'Ext.util.Observable',
        constructor : function(options) {
            if (!objSingletonInstance) {
                objSingletonInstance = Ext.create('WISA.Application',options);
            }
            return objSingletonInstance;
        }
    };
  }()
);

/* functie bij Date om het juiste formaat voor een datum parameter te krijgen in een string */
Date.prototype.asWisaParameter = function () {
  return this.format('d/m/Y');
}

/* Backwards Compatibility & Future Changes */

WISA.ShowError = WISA.showError;
WISA.AjaxError = WISA.showAjaxError;
WISA.CheckError = WISA.getResponseError;
WISA.CreateServerConfig = WISA.getServerConfiguration;

/* baseforms */

/***************
 CustomGridForm: basisform die een grid toont met knoppen toevoegen, bewerken verwijderen
****************/
Ext.define('WISA.baseforms.CustomGridForm' ,{
  extend: 'WISA.baseforms.Form',
  /* velden met gegevens en componenten */
  buttonbar: null,
  butToevoegen: null,
  butBewerken: null,
  butVerwijderen: null,
  pFilter: null,
  pOverzicht: null,
  itemList: null,
  dsOverzicht: null,
  selOverzicht: null,
  positionID: -1,

  /* functions to override in descendants to configure the form */

  buildFilterConfig: function (config) {
    return null;
  },

  buildGridColumns: function (config) {
    return [
      {header: 'Titel', dataIndex: 'VELDNAAM'}
      ]
  },

  getProviderName: function () {
    return 'ProviderToUse';
  },

  getIDField: function () {
    return 'prefix_ID';
  },

  getGridIsPaging: function () {
    return false;
  },

  buildExtraButtons: function (config) {
    return [];
  },

  getFicheClass: function () {
    /* descendants moeten de naam van een descendant van CustomFicheDialog teruggeven */
    return 'CustomFicheDialog';
  },

  getCurrentDescription: function () {
    /* function die een omschrijving van het huidige record moet teruggeven,
       om bovenaan het volgende scherm te plaatsen als dat geopend wordt met een FormButton */
    return '';
  },

  /* functies voor interne werking */

  doeReload: function (data) {
    var opt = this.dsOverzicht.lastOptions;
    this.positionID = Ext.isNumber(data.ID) ? data.ID : -1;
    this.dsOverzicht.load(opt);
  },

  doePosition: function() {
    var rec;
    if (this.positionID>=0) {
      rec = this.dsOverzicht.findExact(this.getIDField(), this.positionID);
      if (!Ext.isEmpty(rec)) {
        this.selOverzicht.select(rec);
      }
    } else {
      if (this.dsOverzicht.getCount() > 0) {
        this.selOverzicht.select(0);
      }
    }
  },

  doeToevoegen: function () {
    this.editRecord (-1);
  },

  doeBewerken: function () {
    var recnr = this.getSelectedID();
    if (recnr >= 0) {
      this.editRecord (recnr);
    }
  },

  doeVerwijderen: function () {
    var rec = this.getSelected();
    if (!Ext.isEmpty(rec)) {
      this.dsOverzicht.remove(rec);
      this.dsOverzicht.save();
    }
  },

  editRecord: function (anID) {
    var fiche = this.getFicheClass();
    var idfield = this.getIDField();
    var params = {
      ID: anID,
      OKHandler: this.doeReload,
      OKScope: this
      };
    params[idfield] = anID;
    if (Ext.isFunction(this.createFicheParams)) {
      this.createFicheParams(fiche,params,this.getSelected());
    };
    Application.ShowDialog({
      form: fiche,
      Params: params
    });
  },

  getSelectedID: function () {
    var rec = this.getSelected();
    if (Ext.isEmpty(rec)) {
      return -1;
    } else {
      return rec.get(this.getIDField());
    }
  },

  getSelected: function () {
    if (Ext.isEmpty(this.selOverzicht)) {
      return null;
    } else {
      return this.selOverzicht.getLastSelected();
    }
  },

  getGridTitle : function (config) {
    if ((!Ext.isEmpty(config)) && (Ext.isString(config.headerConfig))) {
      return this.getFormTitle(config)+' : '+config.headerConfig;
    } else {
      return ''
    }
  },

  getHeaderField : function () {
    return null;
  },

  getHeaderValue : function (sender) {
    var s = this.getHeaderField();
    var r = this.getSelected();
    if (!(Ext.isEmpty(s) || Ext.isEmpty(r))) {
      return r.get(s);
    } else {
      return null;
    }
  },

  /* Build the actual form contents*/

  enableButtons :function (value) {
    var t = this.buttonbar;
    for (var i=0; i<t.items.length; i++) {
      var b = t.items.get(i);
      if (Ext.isBoolean(b.checkSelection) && b.checkSelection) {
        if (value) {
          b.enable();
        } else {
          b.disable();
        }
      }
    }
  },
  BuildForm: function (config) {
    var filterconfig = this.buildFilterConfig(config);
    var gridcolumns = this.buildGridColumns(config);
    var buttonconfig = this.buildExtraButtons(config);
    var gridconfig = null;
    var itemList = [];

    if (Ext.isArray(buttonconfig)) {
      for (var i=0; i<buttonconfig.length; i++) {
        if (Ext.isBoolean(buttonconfig[i].checkSelection) && buttonconfig[i].checkSelection) {
          buttonconfig[i].disabled=true;
        }
      }
    };
    if (Ext.isObject(filterconfig)) {
      this.pFilter = WISA.control('Panel', filterconfig);
      itemList.push(this.pFilter);
    };
    this.dsOverzicht = WISA.dataset({
      ProviderName: this.getProviderName(),
      autoSave: false,
      autoLoad: false,
      listeners: {
        "load": Ext.bind(this.doePosition, this),
        scope: this
      }
    });
    this.selOverzicht = Ext.create(Ext.selection.RowModel,{
      singleSelect: true,
      listeners: {
        scope: this,
        "selectionchange": function (sel) {
          this.enableButtons((sel.getCount() > 0));
        }
      }
    });
    buttonconfig.push('->');
    this.butToevoegen = WISA.control('Button',{
      id : 'btn-toevoegen',
      iconCls : 'btn-toevoegen', 
      text: 'Toevoegen',
      handler: this.doeToevoegen,
      scope: this
    });
    buttonconfig.push(this.butToevoegen);
    this.butBewerken = WISA.control('Button',{
      id : 'btn-bewerken',
      iconCls : 'btn-bewerken',
      text: 'Bewerken',
      disabled: true,
      handler: this.doeBewerken,
      scope: this,
      checkSelection : true
    });
    buttonconfig.push(this.butBewerken);
    this.butVerwijderen = WISA.control('Button',{
        id : 'btn-verwijderen',
        iconCls : 'btn-verwijderen',
        text: 'Verwijderen',
        disabled: true,
        handler: this.doeVerwijderen,
        scope: this,
        checkSelection : true
        });
    buttonconfig.push(this.butVerwijderen);
    this.buttonbar = Ext.create('Ext.toolbar.Toolbar', {
      items: buttonconfig,
      dock: 'top'
      });
    gridconfig = {
      cls: 'customGridForm', 
      border: true,
      title: this.getGridTitle(config),
//      title : this.getFormTitle(),  DEvart had this.
      height: 400,
      width: 600,
      store: this.dsOverzicht,
      selModel: this.selOverzicht,
      columns: gridcolumns,
      dockedItems: [this.buttonbar]
    };
    if (this.getGridIsPaging()) {
      gridconfig.start=0;
      gridconfig.limit=100;
    };
    this.pOverzicht = WISA.control('Grid',gridconfig);
    itemList.push(this.pOverzicht);
    return {
      layout: 'vbox',
      height : 500, // DEVart : added height;
      layoutConfig: {align: 'stretch', pack: 'start', defaultMargins: {top:3, right:10, bottom:5, left:5}},
      items: itemList
      };
  }
});

/******************
 CustomFicheDialog: Dialoog basis opgeroepen door overzichtsform
*******************/

Ext.define('WISA.baseforms.CustomFicheDialog',{
  extend: 'WISA.Dialog',
  /* interne velden */
  onSave: null,
  recordID: -1,
  dsFiche: null,
  fFicheForm : null,
  theRecord: null,

  /* function to override in descendant to configure the form */

  getProviderName: function () {
    return 'ProviderToUse';
  },

  getIDField: function () {
    return 'prefix_ID';
  },

  buildExtraButtons: function (config) {
    return []; // configuration array met extra buttons
  },

  loadRecord : function (rec) {  // Called when the data is loaded, rec is het te tonen datarecord
    this.fFicheForm.getForm().loadRecord(this.theRecord);
    if (Ext.isDefined(this.doLoadRecord)) {
      this.doLoadRecord(rec);
    }
  },

  updateRecord: function (rec) {  // Called when the Bewaren button is pressed, rec is het record van de dataset
    this.fFicheForm.getForm().updateRecord(this.theRecord);
    if (Ext.isDefined(this.doUpdateRecord)) {
      rec.beginEdit();
      this.doUpdateRecord(rec);
      rec.endEdit();
    }
  },

  buildFiche: function (config) {  // dsFiche zal reeds gecreeerd zijn als deze methode wordt opgeroepen
    return {}; // configuration object for a panel that shows the fiche
  },

  /* functies voor interne werking */

  afterSave: function () {
    this.fireEvent('saveOK', {ID: this.dsFiche.getAt(0).get(this.getIDField())});
    this.close();
  },

  afterLoad: function () {
    if (this.dsFiche.getCount() === 0) {
      this.theRecord = this.dsFiche.createModel({});
      if (Ext.isFunction(this.initNewRecord)) {
        this.initNewRecord(this.theRecord);
      }
    } else
      this.theRecord = this.dsFiche.getAt(0);
    this.loadRecord(this.theRecord);
  },

  doeBewaren: function () {
    var rec = this.theRecord;
    this.updateRecord(rec);
    if (this.dsFiche.getCount() === 0) {
      this.dsFiche.add(this.theRecord);
    }
    this.dsFiche.save();
  },

  doeAnnuleren: function () {
    this.close();
  },

  /* Build the actual form contents*/

  BuildForm: function (config) {
    var ficheCfg = this.buildFiche(config);
    var buttonconfig = this.buildExtraButtons(config);
    this.dsFiche = WISA.dataset({
        ProviderName: this.getProviderName(),
        autoLoad: false,
        autoSave: false,
        listeners: {
          "write": this.afterSave,
          "load": this.afterLoad,
          scope: this
        }
      });
    // verplichte buttons toevoegen
    buttonconfig.push({
      text: 'Bewaren',
      handler: this.doeBewaren,
      scope: this
    });
    buttonconfig.push({
      text: 'Annuleren',
      handler: this.doeAnnuleren,
      scope: this
    });
    Ext.apply(ficheCfg,{buttons: buttonconfig});
    // Form object maken.
    this.fFicheForm = WISA.control('Form',ficheCfg);
    return {
      //layout: 'form',
      title: this.getFormTitle(),
      items : [this.fFicheForm],
      width : ficheCfg.width+12,
      heigth : ficheCfg.height+12,
      defaultButton: this.fFicheForm.items[0]
    }
  },

  /* Process parameters which have been passed here */
  ApplyParams : function (Params) { // Zorg dat deze methode altijd uitgevoerd wordt !!
    if ( Params != undefined ) {
      /* Call inherited ApplyParams */
      this.callParent(arguments);
      if ( Params.OKHandler != undefined ) {
        this.addListener("saveOK",Params.OKHandler,Params.OKScope);
      this.recordID = Params.ID;
      }
    }
  }

});

/******************
 Wizard : Dialoog met verschillende paginas, elke pagina moet een WISA.baseforms.WizardPage zijn
*******************/

Ext.define('WISA.baseforms.Wizard',{
  extend: 'WISA.baseforms.Dialog',
  textNextButton: 'Volgende',
  textPreviousButton: 'Vorige',
  textCancelButton: 'Cancel',
  textTitle: 'Wizard stap {1}: {2}',  // 0 = aantal pages, 1 = pagina nummer, 2 is titel pagina
  getPages: function () {  // deze functie moet een array met pages teruggeven
    return [
/*      Ext.create('WISA.baseforms.WizardPage',{}),
        Ext.create('WISA.baseforms.WizardPage',{}),
        ... */
    ]
  },
  wizardData: {},  // params van ApplyParams worden hierin geplaatst voor de pages

  tTitle: null,
  pPages: null,
  bCancel: null,
  bNext: null,
  bPrevious: null,
  activePageIndex: -1,

  // initialization and creation

  BuildForm: function () {

    this.initButtons();
    this.initPages();

    return {
      layout: "hbox",
      title: 'Wizard',
      header: true,
      id: 'wisa-wizard',
      monitorValid: true,
      buttons: [
        this.bCancel,
        this.bPrevious,
        this.bNext
      ],
      items: [ this.pPages ]
    };
  },

  /* Process parameters which have been passed here */
  ApplyParams : function (Params) {
    if ( Params != undefined ) {
      /* Call inherited ApplyParams */
      this.callParent(Params);
    this.wizardData = {};
    Ext.apply(this.wizardData, Params);
    this.startWizard();
    }
  },

  initButtons: function () {
    this.bCancel = Ext.create('WISA.controls.Button',{
      height: 20,
      width: 110,
      text: this.textCancelButton,
      scope: this,
      handler: this.onCancelClick
    });
    this.bNext = Ext.create('WISA.controls.Button',{
      height: 20,
      width: 110,
      text: this.textNextButton,
      scope: this,
      handler: this.onNextClick
    });
    this.bPrevious = Ext.create('WISA.controls.Button',{
      height: 20,
      width: 110,
      text: this.textPreviousButton,
      scope: this,
      handler: this.onPreviousClick
    });
  },

  initPages: function () {
    var p;
    var h = 50;
    var w = 50;
    var pages = this.buildPages();
    for (r=0; r<pages.length; r++) {
      p = pages[r];
      p.pageIndex = r;
      p.on('pagefinished', this.onPageFinished, this);
      p.on('clientvalidation', this.onClientValidation, this);
      p.buildPage();
      if (p.heigth > h) { h = p.height }
      if (p.width > w) { w = p.width }
    };
    this.tTitle = Ext.create('Ext.Template',this.textTitle);
    this.tTitle.compile();
    this.pPages = Ext.create('WISA.controls.Cards',{
      layout: 'card',
      region: 'center',
      items: pages
    });
  },

  // Event handlers

  onCancelClick: function () {
    this.close();
  },

  onNextClick: function () {
    var current = this.getActivePage();
    if (current.canGotoNextPage(this.wizardData)) {
      this.startWaiting();
      current.executePage(this.wizardData);
    }
  },

  onPageFinished: function() {
    var newPageIndex = null;
    var newPage = null;
    var current = this.getActivePage();
    newPageIndex = current.getNextPageIndex(this.wizardData);
    this.stopWaiting();
    if ((newPageIndex >= this.pPages.items.length) || (newPageIndex == 777)) {
      // einde van de wizard
      this.close();
    } else {
      newPage = this.pPages.items.items[newPageIndex];
      newPage.initPage(this.wizardData);
      this.setActivePage(newPageIndex);
    }
  },

  onPreviousClick: function () {
    var current = this.getActivePage();
    var newPageIndex = current.getPreviousPageIndex(this.wizardData);
    var newPage = this.pPages.items[newPageIndex];
    this.setActivePage(newPageIndex);
  },

  onClientValidation: function (aPanel, valid) {
    this.checkButtonStates(valid);
  },

  // helper functions

  checkButtonStates: function (valid) {
    var ap = this.getActivePage();
    if (!Ext.isEmpty(ap)) {
      if (ap.canGotoPreviousPage(this.wizardData)) {
        this.bPrevious.enable();
      } else {
        this.bPrevious.disable();
      };
      if (valid && ap.canGotoNextPage(this.wizardData)) {
        this.bNext.enable();
      } else {
        this.bNext.disable();
      };
    };
  },

  checkTitle: function () {
    var ap = this.getActivePage();
    if (!Ext.isEmpty(ap)) {
      var t = ap.getPageTitle(this.wizardData);
      this.setTitle (this.tTitle.apply({
        0: this.pPages.items.length,
        1: this.activePageIndex+1,
        2: t
      }));
    }
  },

  startWizard: function () {
    var p = this.startPage?this.startPage:0;
    var page = this.pPages.items.items[p];
    page.initPage(this.wizardData);
    this.setActivePage (p);
  },

  setActivePage: function (index) {
    var ap = this.getActivePage();
    if (!Ext.isEmpty(ap)) {
      ap.stopMonitoring();
    }
    this.activePageIndex = index;
    this.pPages.layout.setActiveItem(index);
    this.checkButtonStates();
    this.checkTitle();
    ap = this.getActivePage();
    if (!Ext.isEmpty(ap)) {
      ap.startMonitoring();
    }
  },

  // public functions

  getActivePage: function () {
    return this.pPages.layout.activeItem;
  },

  startWaiting: function (aMessage) {
    if (Ext.isEmpty(this.processMask)) {
      this.processMask = new Ext.LoadMask('wisa-wizard',{ msg: aMessage?aMessage:'Bezig met verwerken gegevens. Even geduld...', removeMask: true});
    };
    this.processMask.show();
  },

  stopWaiting: function () {
    this.processMask.hide();
  }

});

/******************
 WizardPage : Een panel dat een pagina voor in de Wizard opbouwt
 De data parameter is een object dat door de wizard wordt doorgegeven aan alle pages om hun gegevens in te bewaren.
 De parameters die meegegeven worden bij het oproepen van de wizard, worden in dit object ook bewaard (door ApplyParams).
*******************/

Ext.define('WISA.baseforms.WizardPage',{
  extend:'WISA.controls.Form',

  initComponent: function () {
    this.callParent(arguments);
    this.addEvents('pagefinished');
  },

  // de nummer van de pagina. Zal gezet worden door de Wizard zelf
  pageIndex: -1,

  // eenmaal opgeroepen bij het maken van de wizard
  buildPage: function (data) {
  },

  // opgeroepen wanneer de pagina getoond word door de Next button.
  // opgelet: Wordt niet opgeroepen met de Previous om de laatste status van de
  //   pagina te behouden bij Previous
  // Kan gebruikt worden om default values in te vullen of gegevens op te halen
  initPage: function (data) {
  },

  // opgeroepen voor er naar de volgende pagina gegaan wordt
  // Hier kan je alles doen dat bij het eindigen van de page moet gebeuren, en
  //   om data op te laden om te beslissen welke pagina getoond moet worden.
  // Als alles afgewerkt en opgehaald is, moet de procedure pageFinished()
  //   opgeroepen worden om de wizard te laten verder gaan
  executePage: function (data) {
    this.pageFinished();
  },

  // moet opgeroepen worden nadat alles van executePage is geëindigd
  pageFinished: function () {
    this.fireEvent('pagefinished', this, this);
  },

  // opgeroepen om de volgende pagina index te krijgen
  // als de functie 777 teruggeeft, is de wizard gedaan en zal hij sluiten,
  //   zelfs als het niet de laatste pagina is
  getNextPageIndex: function (data) {
    return this.pageIndex + 1;
  },

  // opgeroepen om de vorige pagina index te krijgen
  getPreviousPageIndex: function (data) {
    return this.pageIndex - 1;
  },

  // de titel die de wizard moet tonen als deze pagina actief is
  getPageTitle: function (data) {
    return 'Wizard Pagina ' + pageIndex;
  },

  // functie die moet nagaan of de Next button actief mag zijn
  // Als de laatste page naar de volgende gaat, eindigt de wizard
  canGotoNextPage: function (data) {
    return true;
  },

  // functie die moet nagaan of de Previous button actief mag zijn
  canGotoPreviousPage: function (data) {
    return (this.pageIndex > 0);
  }

});

/* 2 methodes toevoegen aan grouping feature om collapse/expand te doen */
Ext.override(Ext.grid.feature.Grouping, {
  collapseAll: function() {
    var self = this, view = self.view;
    view.el.query(self.eventSelector).forEach(function (group) {
      var group_body = Ext.fly(group.nextSibling, '_grouping');
      self.collapse(group_body);
    });
  },

  expandAll: function() {
    var self = this, view = self.view;
    view.el.query(self.eventSelector).forEach(function (group) {
      var group_body = Ext.fly(group.nextSibling, '_grouping');
      self.expand(group_body);
    });
  }
});

