unit authwebouders;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpjson, webwisa, webwisaauth, mdlwebwisad;

type
  { TOudersAuthenticationProxy }

  TOudersAuthenticationProxy = Class(TAuthenticationProxy)
  Private
    FLD: TLoginDataList;
    function CheckNewUser(ARol: TLeerlingRol; LDS: TloginDataList;  const AUserName, APassword: String): Boolean;
    function CreateNewUser(AUserName, APassword, AFullName: String): Integer;
    function FindUser(const AUserName: String): Integer;
    procedure UpdateRolUID(ALeerling: Integer; ARol: TleerlingRol; const AUID: Integer);
    procedure UpdateSessionData;
  Protected
    function CreateLeerlingMenu(LLID: Integer; Const ANaam : String): TJSONObject;
    procedure LoadLoginData(LD: TLoginDataList; AUserName: String);
    function CheckExistingUser(UID: INteger; P: String) : boolean;
    function CreateQuery(ASQL : String) : TWisaWebQuery;
  Public
    destructor destroy; override;
    Function CreateUserMenu : TJSONObject; override;
    function AuthenticateUser(const AUserName, APassword: String): Boolean; override;
    Property LD: TLoginDataList Read FLD;
  end;

Procedure RegisterOudersUserManagement;

implementation

uses webwisamenu, slcrypt, idlist, cfgwebouders;

Procedure RegisterOudersUserManagement;

begin
  RegisterRootAuthProxy;
  RegisterAuthenticationProxy(TOudersAuthenticationProxy);
end;

{ TOudersAuthenticationProxy }

function TOudersAuthenticationProxy.CreateQuery(ASQL: String): TWisaWebQuery;
begin
  Result:=DatabaseManager.GetQuery(Self,ASQL,Database,Transaction);
end;

destructor TOudersAuthenticationProxy.destroy;
begin
  FreeAndNil(FLD);
  inherited destroy;
end;

function TOudersAuthenticationProxy.AuthenticateUser(const AUserName,
  APassword: String): Boolean;

Var
  UID : Integer;
  LI : TLoginData;
  R : TLeerlingRol;

begin
  Result:=False;
  FLD:=TLoginDataList.Create(TLoginData);
  LoadLoginData(LD,AUserName);
  Li:=LD.GetMatch(AUserName,R);
  If (LI<>nil) then
    begin
    Case R of
      lrVader  : UID:=Li.VaderUID;
      lrMoeder : UID:=Li.MoederUID;
      lrVoogd  : UID:=Li.VoogdUID;
    else
      Raise Exception.CreateFmt('Onbekende rol %d',[Ord(R)]);
    end;
    end
  else
    Exit;
  If (UID>0) then
    result:=CheckExistingUser(UID,APassword)
  else
    Result:=CheckNewUser(R,LD,AUserName,APassword);
  UpdateSessionData;
end;

Procedure TOudersAuthenticationProxy.UpdateSessionData;

Var
  IDS : TIDList;
  I : Integer;
  LI : TLoginData;

begin
  IDS:=TIDList.Create(LD.Count);
  try
    For I:=0 to LD.Count-1 do
      begin
      Li:=LD[i];
      IDS.Add(LI.LeerlingID);
      end;
    Session.Variables['Leerlingen']:=IDS.AsString;
    Logweb('Leerlingen : %s',[IDS.AsString]);
  finally
    IDS.Free;
  end;
end;


Function TOudersAuthenticationProxy.FindUser(Const AUserName : String) : Integer;

Const
  SQL = 'SELECT U_ID FROM USERS WHERE (U_NAAM=:U_NAAM)';

Var
  Q : TWisaWebQuery;

begin
  Result:=-1;
  Q:=CreateQuery(SQL);
  try
    Q.Params.ParamByName('U_NAAM').AsString:=AUserName;
    Q.Open;
    If not (Q.EOF and Q.BOF) then
      Result:=Q.FieldByname('U_ID').AsInteger;
  finally
    Q.Free;
  end;
end;

Function TOudersAuthenticationProxy.CreateNewUser(AUserName,APassword, AFullName: String) : Integer;

Const

  SQL = 'insert into USERS'+sLineBreak+
  ' (U_NAAM, U_NAAMVOLLEDIG, U_ID, U_PASWOORD,'+sLineBreak+
  '  U_VERANDERDDOOR, U_VERANDERDOP, U_KANVERANDERPASW)'+sLineBreak+
  'VALUES'+sLineBreak+
  ' (:U_NAAM, :U_NAAMVOLLEDIG, :U_ID, :U_PASWOORD,'+sLineBreak+
  '  0,''NOW'', ''+'' )';

Var
  I : Integer;
  PWD,Un : shortString;
  Q : TWisaWebQuery;

begin
  I:=0;
  Repeat
    If (I>0) then
      UN:=AUserName+inttostr(i)
    else
      UN:=AUserName;
    Inc(I);
  until (FindUser(UN)=-1);
  Result:=GEN_ID('GEN_USER');
  Q:=CreateQuery(SQL);
  try
    Q.Params.ParamByName('U_ID').AsInteger:=Result;
    Q.Params.ParamByName('U_NAAM').AsString:=UN;
    if (AFullName='') then
      AFullName:=UN;
    Q.Params.ParamByName('U_NAAMVOLLEDIG').AsString:=AFullName;
    Pwd:=lcrypt(APassword,pwdsalt);
    Q.Params.ParamByName('U_PASWOORD').AsString:=pwd;
    Q.ExecSQL;
    Transaction.Commit;
    Session.UserID:=Result;
    Session.Flags:=0;
    Session.AllowChangePassword:=True;
    Session.FullName:=AFullName;
  finally
    Q.Free;
  end;
end;

Function EncodeUserPassword(Pwd : String) : String;

begin
  Result:=lcrypt(Pwd,pwdsalt);
end;

Function TOudersAuthenticationProxy.CheckExistingUser(UID : Integer; P : String) : Boolean;

Const
  SQL = 'SELECT'+sLineBreak+
        '  U_ID,'+sLineBreak+
        '  U_NAAMVOLLEDIG,'+sLineBreak+
        '  U_LAATSTELOGIN,'+sLineBreak+
        '  U_KANVERANDERPASW,'+sLineBreak+
        '  U_GELDIGTOT,'+sLineBreak+
        '  U_APPFLAGS,'+sLineBreak+
        '  U_PASWOORD'+sLineBreak+
        'FROM'+sLineBreak+
        '  USERS'+sLineBreak+
        'WHERE'+sLineBreak+
        '  (U_ID=:UID)';

Var
  Q : TWisaWebQuery;

begin
  Result:=False;
  Q:=CreateQuery(SQL);
  With Q do
    begin
    try
      Params.ParamByName('UID').AsInteger:=UID;
      Open;
      If (EOF and BOF) then
        Exit;
      If (FieldByname('U_PASWOORD').AsString<>EncodeUserPassword(P)) then
        Exit;
      If (Not FieldByName('U_GELDIGTOT').IsNull) and (FieldByName('U_GELDIGTOT').AsDateTime<Now) then
        Exit;
      Session.UserID:=UID;
      Session.Flags:=FieldByName('U_APPFLAGS').AsInteger;
      Session.AllowChangePassword:=FieldByName('U_KANVERANDERPASW').AsString='+';
      Session.FullName:=FieldByName('U_NAAMVOLLEDIG').AsString;
      Result:=True;
    finally
      close;
    end;
    end;
end;

Procedure TOudersAuthenticationProxy.UpdateRolUID(ALeerling : Integer; ARol : TleerlingRol; Const AUID : Integer);

Const
    SUpdateLeerlingID = 'UPDATE LEERLING SET LL_USER_FK=:UID WHERE (LL_ID=:LL_ID)';
    SUpdateVaderID    = 'UPDATE LEERLING SET LL_VADERUSER_FK=:UID WHERE (LL_ID=:LL_ID)';
    SUpdateMoederID   = 'UPDATE LEERLING SET LL_MOEDERUSER_FK=:UID WHERE (LL_ID=:LL_ID)';
    SUpdateVoogdID    = 'UPDATE LEERLING SET LL_VOOGDUSER_FK=:UID WHERE (LL_ID=:LL_ID)';

Var
  SQL : String;
  Q : TWisaWebQuery;

begin
  Case ARol of
    lrLeerling : SQL:=SUpdateleerlingID;
    lrVader    : SQL:=SUpdateVaderID;
    lrMoeder   : SQL:=SUpdateMoederID;
    lrVoogd    : SQL:=SUpdateVoogdID;
  else
    Raise Exception.CreateFmt('UpdateRolID: Onbekende rol %d',[Ord(ARol)]);
  end;
  Q:=CreateQuery(SQL);
  try
    LogWeb('Update rol : %s (UID: %d, LLID: %d)',[SQL,AUID,ALeerling]);
    Q.Params.ParamByName('LL_ID').AsInteger:=ALeerling;
    Q.Params.ParamByName('UID').AsInteger:=AUID;
    Q.ExecSQL;
    Transaction.Commit;
  finally
    Q.Free;
  end;
end;

Function TOudersAuthenticationProxy.CheckNewUser(ARol : TLeerlingRol; LDS : TloginDataList; Const AUserName,APassword : String) : Boolean;

Var
  L : TLoginData;
  I,RID,ID : Integer;
begin
  Result:=False;
  LogWeb('Searching RR match',[]);
  L:=LDS.GetRRMatch(APassword);
  LogWeb('Searching RR match %d',[Ord(L<>Nil)]);
  If (L<>Nil) then
    begin
    LogWeb('Creating new user match',[]);
    ID:=CreateNewUser(AUserName,APassword,L.GetRolVoornaam(ARol)+' '+L.GetRolNaam(ARol));
    LogWeb('Created new user: %d ',[ID]);
    For I:=0 to LDS.Count-1 do
      begin
      L:=LDS[i];
      RID:=L.GetRolUID(ARol);
      LogWeb('Checking leerling userid: %d  (%d)',[L.leerlingID,RID]);
      if (CompareText(L.GetRolEmail(ARol),AUserName)=0) and (RID<=0) and (RID<>ID) then
        UpdateRolUID(L.LeerlingID,ARol,ID);
      end;
    LogWeb('Creating result with ID',[ID]);
    Result:=True;
    end;
end;

procedure TOudersAuthenticationProxy.LoadLoginData(LD : TLoginDataList; AUserName : String);

Const
  SQL = 'SELECT LL_ID, LL_NAAM, LL_VOORNAAM, LL_MOEDEREMAIL, LL_VADEREMAIL, LL_VOOGDEMAIL,'+sLineBreak+
  '  LL_VADERUSER_FK, LL_MOEDERUSER_FK, LL_VOOGDUSER_FK, LL_VADERNAAM, LL_VADERVOORNAAM,'+sLineBreak+
  '  LL_MOEDERNAAM, LL_MOEDERVOORNAAM, LL_VOOGDNAAM, LL_VOOGDVOORNAAM, LL_RIJKSREGISTERNR'+sLineBreak+
  'FROM'+sLineBreak+
  '  LEERLING'+sLineBreak+
  'WHERE'+sLineBreak+
  '  (LL_VADEREMAIL=:VaderEmail)'+sLineBreak+
  '  OR (LL_MOEDEREMAIL=:MoederEmail)'+sLineBreak+
  '  OR (LL_VOOGDEMAIL=:VoogdEmail)'+sLineBreak+
  'ORDER BY'+sLineBreak+
  '  LL_VOORNAAMSORT';

Var
  Q : TWisaWebQuery;
begin
  Q:=CreateQuery(SQL);
  try
    Q.Params.ParamByName('MoederEmail').AsString:=AUserName;
    Q.Params.ParamByName('VaderEmail').AsString:=AUserName;
    Q.Params.ParamByName('VoogdEmail').AsString:=AUserName;
    Q.Open;
    If Not (Q.EOF AND Q.BOF) then
      LD.LoadFromDataset(Q);
  finally
    Q.Free;
  end;
end;


Function TOudersAuthenticationProxy.CreateLeerlingMenu(LLID : Integer; Const ANaam : String) : TJSONObject;

  Procedure AddLeerlingParam(O : TJSONObject);

  Var
    I : integer;
    Items : TJSONArray;

  begin
    O.Add('Params',TJSonObject.Create(['ID',LLID]));
    i:=O.IndexOfName('Items');
    If (I<>-1) then
      begin
      Items:=O.Items[I] as TJSONArray;
      For I:=0 to Items.Count-1 do
        AddleerlingParam(Items.Objects[i]);
      end;
  end;

begin
  Result:=FullMenu.ToJSON(true);
  Result.Strings['Caption']:=ANaam;
  AddLeerlingParam(Result);
end;


Function TOudersAuthenticationProxy.CreateUserMenu: TJSONObject;


Var
  I: Integer;
  Items : TJSONArray;

begin
  Result:=TJSONObject.Create;
  Items:=TJSONArray.Create();
  Result.Add('Items',Items);
  For I:=0 to LD.Count-1 do
    begin
    Items.Add(CreateLeerlingMenu(LD[i].LeerlingID,LD[i].VoorNaam+' '+LD[i].Naam));
    end;
end;


end.

