unit cfgwebouders;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, strutils, idlist, fpjson, webwisa, cfgwebwisa;

Const
  MaxFeatureCount = 12;

Type
  { TWebOudersConfig }

  TWebOudersConfig = Class(TWebWisaConfig)
  private
    FBegeleidingHTML: String;
    FBGSubTypes: TIDList;
    FBGTypes: TIDList;
    FHoofdGroepen: String;
    FIDTypes: TIDList;
    FImageFile: String;
    FInstellingsCodes: String;
    FInterventieHTML: String;
    FLessenRooster: String;
    FLessenRoosterExt: String;
    FMotd: TStrings;
    FOuderRolID: Integer;
    FOuderRolNiveau: Integer;
    FPort: Word;
    FRapKast: Integer;
    FRapKastURI: String;
    FRol: String;
    FSchoolEmail: String;
    FSchoolFax: String;
    FSChoolGemeente: String;
    FSchoolImageHeight: Integer;
    FSchoolImageWidth: Integer;
    FSchoolNaam: String;
    FSchoolSite: String;
    FFeatures : Array[0..MaxFeatureCount] of Boolean;
    FISIDS,
    FMGIDS,
    FSCIDS: TIDList;
    FSchoolStraat: String;
    FSchoolTelefoon: String;
    FTheme: String;
    FThemes: TStrings;
    FAfwezigCodesCSS : String;
    procedure FillIDS(TypeRecord: Integer; RolFlag: Integer; IDS: TIDList; ADB : TWisaWebDatabase);
    function GetB(AIndex: integer): Boolean;
    function GetFN(AIndex : Integer): String;
    function GetWerkDatum: TDateTime;
    procedure LoadAfwColors(ADB: TWisaWebDatabase);
    function LoadText(FN: String; const ADefault: String): String;
    function SaveText(FN: String; const AValue: String): String;
    procedure SetB(AIndex: integer; const AValue: Boolean);
    procedure LoadISCodes(ADB: TWisaWebDatabase);
    procedure LoadMainGroups(ADB: TWisaWebDatabase);
    procedure LoadRollen(ADB: TWisaWebDatabase);
    procedure LoadRapportData(ADB: TWisaWebDatabase);
    procedure SetMotd(const AValue: TStrings);
  Protected
    Procedure ApplyConfig(Config : TJSONObject);
  Public
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
    Procedure InitDatabase(ADB : TWisaWebDatabase);
    Class Function AdminConfigOnly: Boolean; override;
    Function GetSchoolJaar : Integer;
    Procedure ConfigHiddenForms;
    procedure Changed; override;
    Property FeatureNames[AIndex : Integer] : String Read GetFN;
    Property Features[AIndex : Integer] : Boolean Read GetB Write SetB;
    Property AfwezigCodesCSS : String Read FAfwezigCodesCSS ;
    Property OuderRolID : Integer Read FOuderRolID Write FouderRolID;
    Property OuderRolNiveau : Integer Read FOuderRolNiveau Write FOuderRolNiveau;
    Property LITypes : TIDList Read FIDTypes;
    Property BGTypes : TIDList Read FBGTypes;
    Property BGSubTypes : TIDList Read FBGSubTypes;
    Property ISIDS : TIDList Read FISIDS;
    Property SCIDS : TIDList Read FSCIDS;
    Property MGIDS : TIDList Read FMGIDS;
    Property RapKastID : Integer Read FRapKast Write FRapKast;
    Property RapKastURI : String Read FRapKastURI Write FRapKastURI;
    property WerkDatum : TDateTime Read GetWerkDatum;
  Published
    Property Theme : String Read FTheme Write FTheme;
    Property InstellingsCodes : String Read FInstellingsCodes Write FInstellingsCodes;
    Property HoofdGroepen : String Read FHoofdGroepen Write FHoofdGroepen;
    Property SchoolImage : String Read FImageFile Write FImageFile;
    Property SchoolImageHeight : Integer Read FSchoolImageHeight Write FSchoolImageHeight;
    Property SchoolImageWidth : Integer Read FSchoolImageWidth Write FSchoolImageWidth;
    Property SchoolWWW : String Read FSchoolSite Write FSchoolSite;
    Property OuderRol : String Read FRol Write FRol;
    Property LessenRoosterDir : String Read FLessenRooster Write FLessenRooster;
    Property LessenRoosterExt : String Read FLessenRoosterExt Write FLessenRoosterExt;
    Property ToonPersonalData : Boolean Index 0 Read GetB Write SetB;
    Property ToonGezinData : Boolean Index 1 Read GetB Write SetB;
    Property ToonRapporten : Boolean Index 2 Read GetB Write SetB;
    Property ToonBegeleiding : Boolean Index 3 Read GetB Write SetB;
    Property ToonInterventie : Boolean Index 4 Read GetB Write SetB;
    Property ToonMedischeFiche : Boolean Index 5 Read GetB Write SetB;
    Property ToonAfwezig : Boolean Index 6 Read GetB Write SetB;
    Property ToonPedagogischTeam: Boolean Index 7 Read GetB Write SetB;
    Property ToonKalender : Boolean Index 8 Read GetB Write SetB;
    Property ToonExtraAdressen : Boolean Index 9 Read GetB Write SetB;
    Property ToonLoopbaan : Boolean Index 10 Read GetB Write SetB;
    Property ToonLoopbaanAndereScholen : Boolean Index 11 Read GetB Write SetB;
    Property ToonFacturen : Boolean Index 12 Read GetB Write SetB;
    Property Motd : TStrings Read FMotd Write SetMotd;
    Property SchoolNaam : String Read FSchoolNaam Write FSchoolNaam;
    Property SchoolStraat : String Read FSchoolStraat Write FSchoolStraat;
    Property SchoolGemeente: String Read FSChoolGemeente Write FSchoolGemeente;
    Property SchoolTelefoon: String Read FSchoolTelefoon Write FSchoolTelefoon;
    Property SchoolFax : String Read FSchoolFax Write FSchoolFax;
    Property SchoolEmail : String Read FSchoolEmail Write FSchoolEmail;
  end;

Function Settings : TWebOudersConfig;

Const
//  MasterPasswd = '8femmes!';
  DefBegeleidingHTML='<TABLE>'+
    '<TR><TD><B>Datum/tijd:</B></TD><TD>{datum} {tijd}</TD><TD><B>Status:</B></TD><TD>{status}</TD></TR>'+
    '<TR><TD><B>Type:</B></TD><TD>{type}</TD><TD><B>Detail:</B></TD><TD>{subtype}</TD></TR>'+
    '<TR><TD><B>Omschrijving:</B></TD><TD COLSPAN="3">{omschrijving}</TD></TR>'+
    '<TR><TD><B>Achterliggend probleem:</B></TD><TD COLSPAN="3">{achterliggendprobleem}</TD></TR>'+
    '<TR><TD><B>Opmerking:</B></TD><TD COLSPAN="3">{opmerking}</TD></TR>'+
    '<TR><TD><B>Interventiemotief:</B></TD><TD COLSPAN="3">{interventiemotief}</TD></TR>'+
    '<TR><TD><B>Referentie 1:</B></TD><TD>{externeidentificatie1}</TD><TD><B>Referentie 2:</B></TD><TD>{externeidentificatie2}</TD></TR>'+
    '<TR><TD><B>Betrokkene:</B></TD><TD COLSPAN="3">{betrokken1}</TD></TR>'+
    '<TR><TD><B>Betrokkene:</B></TD><TD COLSPAN="3">{betrokken2}</TD></TR>'+
    '<TR><TD><B>Betrokkene:</B></TD><TD COLSPAN="3">{betrokken3}</TD></TR>'+
    '</TABLE>';
  DefInterventieHTML='<TABLE>'+
    '<TR><TD><B>Datum/tijd:</B></TD><TD>{datum} {tijd}</TD></TR>'+
    '<TR><TD><B>Type:</B></TD><TD>{type}</TD></TR>'+
    '<TR><TD><B>Omschrijving:</B></TD><TD>{omschrijving}</TD></TR>'+
    '<TR><TD><B>Opmerking:</B></TD><TD>{opmerking}</TD></TR>'+
    '<TR><TD><B>Interventiemotief:</B></TD><TD>{motief}</TD></TR>'+
    '<TR><TD><B>Betrokkene:</B></TD><TD>{betrokkenpersoon}</TD></TR>'+
    '<TR><TD><B>Derden:</B></TD><TD>{derden}</TD></TR>'+
    '<TR><TD><B>Plaats:</B></TD><TD>{plaats}</TD></TR>'+
    '</TABLE>';

implementation


uses db,sqlwebouders,mdlwebwisad, webwisamenu;

Const
  FormNames : Array[0..12] of String = (
   'llpersoonfiche',
   '',
   'llrapport',
   'llbegeleiding',
   'llinterventie',
   'llzorgfiche',
   'llafwezigheden',
   'lbvakken',
   'klkalender',
   '',
   'lbklas',
   'llloopbaanandere',
   'llrekening' );

{ TWebOudersConfig }

Var
  TheSettings : TWebOudersConfig;

Function Settings : TWebOudersConfig;

begin
  If (TheSettings=Nil) then
    begin
    TheSettings:=TWebOudersConfig.Create(Nil);
    TheSettings.Name:='WebWisad';
    RegisterSessionConfigHandlers(@TheSettings.ApplyConfig);
    end;
  Result:=TheSettings;
end;

function TWebOudersConfig.GetB(AIndex: integer): Boolean;
begin
  Result:=FFeatures[AIndex];
end;

function TWebOudersConfig.GetFN(AIndex : Integer): String;
begin
  Case AIndex of
    0 : Result:='Toon Persoonlijke gegevens';
    1 : Result:='Toon gezinsgegevens';
    2 : Result:='Toon rapporten/toetsen';
    3 : Result:='Toon begeleidingen';
    4 : Result:='Toon interventies';
    5 : Result:='Toon medische fiche';
    6 : Result:='Toon afwezigheden';
    7 : Result:='Toon pedagogisch team';
    8 : Result:='Toon kalender';
    9 : Result:='Toon extra adressen';
    10 : Result:='Toon loopbaan';
    11 : Result:='Toon loopbaan in andere scholen';
    12 : Result:='Toon facturen';
    13 : Result:='Configuratie aanpassen via website toestaan';
  else
    Result:=Format('Onbekende feature %d',[AIndex]);
  end;
end;

function TWebOudersConfig.GetWerkDatum: TDateTime;

Var
  Y,M,D : Word;
  da : TDateTime;

begin
  da:=Date;
  DecodeDate(Da,Y,M,D);
//  Result:=EncodeDate(2006,04,01);
  if M in [7,8] then
    Da:=EncodeDate(Y,6,30);
{  else if M=8 then
    Da:=EncodeDate(Y,9,1);}
  Result:=Da;
end;

procedure TWebOudersConfig.SetB(AIndex: integer; const AValue: Boolean);
begin
  FFeatures[AIndex]:=Avalue;
end;



function TWebOudersConfig.GetSchoolJaar: Integer;

Var
  Y,M,D : Word;

begin
  DecodeDate(WerkDatum,Y,M,D);
  If M>8 then
    Result:=Y
  else
    Result:=Y-1;
end;

procedure TWebOudersConfig.ConfigHiddenForms;

  Procedure CheckForm(M : TWebMenuItem; L : TStrings);

  Var
    I: Integer;

  begin
    M.Hide:=L.IndexOf(M.Form)<>-1;
    if not M.Hide then
      For I:=0 to M.Count-1 do
        CheckForm(M.Items[i],L);
  end;

Var
  L : TStringList;
  I : Integer;

begin
  L:=TStringList.Create;
  try
    For I:=0 to MaxFeatureCount do
      If Not FFeatures[i] then
        if (FormNames[i]<>'') then
          begin
          L.Add(FormNames[i]);
          end;
    L.Sorted:=True;
    CheckForm(FullMenu,L);
  finally
    L.Free;
  end;

end;

procedure TWebOudersConfig.Changed;
begin
  inherited Changed;
  ConfigHiddenForms;
end;

Function TWebOudersConfig.LoadText(FN : String; Const ADefault : String) : String;

Var
  FFN : String;
  L : TStringList;

begin
  FFN:=Self.BaseDirectory+FN;
  If FileExists(FFN) then
    begin
    L:=TStringList.Create;
    try
      L.LoadFromFile(FFN);
      If L.Count>0 then
        Result:=L.Text;
    finally
      L.Free;
    end;
    end;
  If (Result='') then
    Result:=ADefault;
end;


Function TWebOudersConfig.SaveText(FN : String; Const AValue : String) : String;

Var
  FFN : String;
  L : TStringList;

begin
  FFN:=BaseDirectory+FN;
  L:=TStringList.Create;
  try
    L.Text:=AValue;
    L.SaveToFile(FFN);
    If L.Count>0 then
      Result:=L.Text;
  finally
    L.Free;
  end;
end;

constructor TWebOudersConfig.Create(AOwner: TComponent);

Var
  I : Integer;

begin
  inherited Create(AOwner);
  FPort:=2015;
  FISIDS:=TIDList.Create;
  FSCIDS:=TIDList.Create;
  FMGIDS:=TIDList.Create;
  FIDTypes:=TIDList.Create;
  FBGTypes:=TIDList.Create;
  FBGSubTypes:=TIDList.Create;
  FMotd:=TStringList.Create;
  FThemes:=TStringList.Create;
  For I:=0 to MaxFeatureCount do
    FFeatures[i]:=True;
  FBegeleidingHTML:=DefBegeleidingHTML;
  FInterventieHTML:=DefInterventieHTML;
end;

destructor TWebOudersConfig.Destroy;
begin
  FreeAndNil(FMotd);
  FreeAndNil(FIDTypes);
  FreeAndNil(FBGTypes);
  FreeAndNil(FBGSubTypes);
  FreeAndNil(FISIDS);
  FreeAndNil(FSCIDS);
  FreeAndNil(FMGIDS);
  FreeAndNil(FThemes);
  inherited Destroy;
end;

procedure TWebOudersConfig.LoadAfwColors(ADB: TWisaWebDatabase);

Var
  S : String;
  FACID, FACKLeur : TField;
  L : TStringList;
  Q : TWisaWebQuery;

begin
  If Assigned(LogWeb) then
    LogWeb('Loading afwezig kleur lijst from database',[]);
  S:='SELECT AC_ID,AC_KLEUR FROM AFWEZIGCODES';
//  S:=Format('SELECT AC_ID,AC_KLEUR FROM AFWEZIGCODES WHERE (AC_INSTELLING_FK IN (%s))',[ISIDS.AsString]);
  Q:=DatabaseManager.Getquery(Self,S,ADB);
  try
    Q.Open;
    FACID:=Q.FieldByName('AC_ID');
    FACKLeur:=Q.FieldByName('AC_KLEUR');
    L:=TStringList.Create;
    try
      While not Q.EOF do
        begin
        S:='.afwezigkleur'+FACID.AsString+' { background-color: #'+GetKleur(FACkleur.asInteger)+'; }';
        L.Add(S);
        Q.Next;
        end;
      FAfwezigCodesCSS:=L.Text;
    finally
      L.Free;
    end;
  finally
    Q.Free;
  end;
end;

procedure TWebOudersConfig.InitDatabase(ADB: TWisaWebDatabase);
begin
  LoadISCodes(ADB);
  LoadMainGroups(ADB);
  LoadRollen(ADB);
  LoadRapportData(ADB);
  LoadAfwColors(ADB)
end;

procedure TWebOudersConfig.LoadISCodes(ADB : TWisaWebDatabase);

Var
  CL,S : String;
  I  : Integer;
  Q : TWisaWebQuery;
  FISID, FSCID : TField;

begin
  S:='';
  CL:=Trim(InstellingsCodes);
  For I:=1 to WordCount(CL,[' ']) do
    begin
    If (S<>'') then
      S:=S+', ';
    S:=S+''''+Trim(ExtractWord(I,CL,[' ']))+'''';
    end;
  If Assigned(LogWeb) then
    LogWeb('Loading school list from database: %s',[S]);
  If (S='') then
    exit;
  ISIDS.Clear;
  SCIDS.Clear;
  Q:=DatabaseManager.Getquery(Self,Format(SSelectSchoolData,[S]),ADB);
  try
    Q.Open;
    try
      FISID:=Q.FieldByName('IS_ID');
      FSCID:=Q.FieldByName('SC_ID');
      While not Q.EOF do
        begin
        ISIDS.Add(FISID.AsInteger);
        SCIDS.Add(FSCID.AsInteger);
        Q.Next;
        end;
      SCIDS.Add(3204);
      SCIDS.Add(2355);
    finally
      Q.Close;
    end;
  finally
    Q.Free;
  end;
end;

procedure TWebOudersConfig.LoadMainGroups(ADB : TWisaWebDatabase);
Var
  CL,S : String;
  I  : Integer;
  Q : TWisaWebQuery;
  FMGID : TField;

begin
  CL:=Trim(HoofdGroepen);
  S:='';
  For I:=1 to WordCount(CL,[' ']) do
    begin
    If (S<>'') then
      S:=S+', ';
    S:=S+''''+Trim(ExtractWord(I,CL,[' ']))+'''';
    end;
  MGIDS.Clear;
  If Assigned(LogWeb) then
    Logweb('Loading hoofdgroepen list from database: %s',[S]);
  if (S='') then
    Exit;
  Q:=DatabaseManager.Getquery(Self,Format(SSelectMainGroupData,[S]),ADB);
  try
    Q.Open;
    try
      FMGID:=Q.FieldByName('MG_ID');
      While not Q.EOF do
        begin
        MGIDS.Add(FMGID.AsInteger);
        Q.Next;
        end;
    finally
      Q.Close;
    end;
  finally
    Q.Free;
  end;
  MGIDS.Add(42);
  If MGIDS.COunt=0 then
    MGIDS.Add(-1);
end;

procedure TWebOudersConfig.FillIDS(TypeRecord : Integer;RolFlag : Integer; IDS : TIDList; ADB : TWisaWebDatabase);

Var
  Q : TWisaWebQuery;
  F : TField;

begin
  IDS.Clear;
  IDS.Sorted:=False;
  Q:=DatabaseManager.GetQuery(Self,SSelectLVSTypes,ADB);
  try
    Q.Name:='LVSTypes';
    Q.Params.ParamByName('Type').AsInteger:=TypeRecord;
    Q.Params.ParamByName('Rol').AsInteger:=RolFlag;
    Q.Open;
    F:=Q.FieldByname('LVT_ID');
    While not Q.EOF do
      begin
      IDS.Add(F.AsInteger);
      Q.Next;
      end;
  finally
    Q.Free;
  end;
end;

procedure TWebOudersConfig.LoadRapportData(ADB : TWisaWebDatabase);

Var
  Q : TWisaWebQuery;

begin
  If ToonRapporten then
    begin
    Q:=DatabaseManager.Getquery(Self,SSelectRapportKastData,ADB);
    try
      Q.Name:='RapportData';
      Q.Open;
      try
        If (Q.EOF AND Q.BOF) then
          begin
          If Assigned(LogWeb) then
            LogWeb('Geen leerling rapporten kast gevonden',[]);
          ToonRapporten:=False;
          end
        else
          begin
          RapKastID:=Q.FieldByName('K_ID').AsInteger;
          RapKastURI:=Q.FieldByName('K_URI').AsString;
          If Assigned(LogWeb) then
            LogWeb('Leerling rapport kast (%d) URI : %s',[RapKastID,RapKastURI]);
          end;
      finally
        Q.Close;
      end;
    finally
      Q.Free;
    end;
    end;
end;

procedure TWebOudersConfig.SetMotd(const AValue: TStrings);
begin
  if FMotd=AValue then exit;
  FMotd.Assign(AValue);
end;

procedure TWebOudersConfig.ApplyConfig(Config: TJSONObject);
begin
  Config.Booleans['ToonGezinData']:=ToonGezinData;
  Config.Booleans['ToonExtraAdressen']:=ToonExtraAdressen;
end;

class function TWebOudersConfig.AdminConfigOnly: Boolean;
begin
  Result:=True;
end;

procedure TWebOudersConfig.LoadRollen(ADB : TWisaWebDatabase);

Var
  Q : TWisaWebQuery;
  R : TLVSRollen;
  I,RF : Integer;

begin
  Q:=DatabaseManager.GetQuery(Self,SSelectLVSRollen,ADB);
  try
    Q.Open;
    try
      R:=LVSRollen;
      R.LoadFromDataset(Q);
      If (OuderRol<>'') then
        For I:=0 to R.Count-1 do
          If CompareText(R[i].Code,OuderRol)=0 then
            begin
            OuderRolID:=R[i].LVRID;
            OuderRolNiveau:=R[i].SecurityLevel;
            end;
    finally
      Q.Close;
    end;
  finally
    Q.Free;
  end;
  RF:=1 SHL OuderRolID;
  If Assigned(LogWeb) then
     LogWeb('OuderrolID : %d -> %d',[OuderRolID,Rf]);
  FillIDS(0,RF,BGTypes,ADB);
  FillIDS(1,RF,BGSubTypes,ADB);
  FillIDS(2,RF,LITypes,ADB);
end;

initialization
Finalization
  FreeAndNil(TheSettings);
end.

