unit mdlwebwisad;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, DB, fieldmap;

Const
  lzMedisch       = 1;
  lzLeerproblemen = 2;
  lzSociaal       = 3;
  lzEmotioneel    = 4;


Type
  TLeerlingRol = (lrLeerling,lrVader,lrMoeder,lrVoogd);
  TAdressen = Class;
  TLeerlingDatas = Class;
  TAfwezigheden = Class;
  TLeerlingZorgList = Class;
  TBegeleidingen = Class;
  TInterventies = Class;
  TKalenderItemList = Class;
  TRapportPeriodes = Class;
  TRapportBestanden = Class;

  TMedischeFiche = Class(TPersistent)
  Private
    FAndere : AnsiString;
    FVlaggen : Longint;
    FGehoor : AnsiString;
    FGehoorsinds : TDateTime;
    FID : Longint;
    FMotoriek1 : AnsiString;
    FMotoriek2 : AnsiString;
    FMotoriek3 : AnsiString;
    FMotoriek4 : AnsiString;
    FSISkaart : AnsiString;
    FSpraak : AnsiString;
    FSpraakSinds : TDateTime;
    FVrijeParam1 : AnsiString;
    FVrijeParam2 : AnsiString;
    FVrijveld1 : AnsiString;
    FVrijVeld2 : AnsiString;
    FVrijveld3 : AnsiString;
    FZicht : AnsiString;
    FZichtSinds : TDateTime;
    FZiekenfondsnummer : AnsiString;
  Private
  Protected
  Public
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property Andere : AnsiString Read FAndere Write FAndere;
    Property Vlaggen : Longint Read FVlaggen Write FVlaggen;
    Property Gehoor : AnsiString Read FGehoor Write FGehoor;
    Property Gehoorsinds : TDateTime Read FGehoorsinds Write FGehoorsinds;
    Property ID : Longint Read FID Write FID;
    Property Motoriek1 : AnsiString Read FMotoriek1 Write FMotoriek1;
    Property Motoriek2 : AnsiString Read FMotoriek2 Write FMotoriek2;
    Property Motoriek3 : AnsiString Read FMotoriek3 Write FMotoriek3;
    Property Motoriek4 : AnsiString Read FMotoriek4 Write FMotoriek4;
    Property SISkaart : AnsiString Read FSISkaart Write FSISkaart;
    Property Spraak : AnsiString Read FSpraak Write FSpraak;
    Property SpraakSinds : TDateTime Read FSpraakSinds Write FSpraakSinds;
    Property VrijeParam1 : AnsiString Read FVrijeParam1 Write FVrijeParam1;
    Property VrijeParam2 : AnsiString Read FVrijeParam2 Write FVrijeParam2;
    Property Vrijveld1 : AnsiString Read FVrijveld1 Write FVrijveld1;
    Property VrijVeld2 : AnsiString Read FVrijVeld2 Write FVrijVeld2;
    Property Vrijveld3 : AnsiString Read FVrijveld3 Write FVrijveld3;
    Property Zicht : AnsiString Read FZicht Write FZicht;
    Property ZichtSinds : TDateTime Read FZichtSinds Write FZichtSinds;
    Property Ziekenfondsnummer : AnsiString Read FZiekenfondsnummer Write FZiekenfondsnummer;
  end;


  { TMedischeFicheMap }

  TMedischeFicheMap = Class(TFieldMap)
  Private
    FMF_ANDERE : TField;
    FMF_FLAGS : TField;
    FMF_GEHOOR : TField;
    FMF_GEHOORSINDS : TField;
    FMF_ID : TField;
    FMF_MOTORIEK1 : TField;
    FMF_MOTORIEK2 : TField;
    FMF_MOTORIEK3 : TField;
    FMF_MOTORIEK4 : TField;
    FMF_SISKAART : TField;
    FMF_SPRAAK : TField;
    FMF_SPRAAKSINDS : TField;
    FMF_VRIJPARAM1 : TField;
    FMF_VRIJPARAM2 : TField;
    FMF_VRIJVELD1 : TField;
    FMF_VRIJVELD2 : TField;
    FMF_VRIJVELD3 : TField;
    FMF_ZICHT : TField;
    FMF_ZICHTSINDS : TField;
    FMF_ZIEKENFONDSNUMMER : TField;
    Procedure DoLoad(AObject : TMedischeFiche);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TKlasgenoot }

  TKlasgenoot = Class(TCollectionItem)
  Private
    FKlasNR : SmallInt;
    FLeerlingID : Longint;
    FNaam : AnsiString;
    FVoornaam : AnsiString;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property KlasNR : SmallInt Read FKlasNR Write FKlasNR;
    Property LeerlingID : Longint Read FLeerlingID Write FLeerlingID;
    Property Naam : AnsiString Read FNaam Write FNaam;
    Property Voornaam : AnsiString Read FVoornaam Write FVoornaam;
  end;


  { TBegeleider }

  TBegeleider = Class(TCollectionItem)
  Private
    FInuitID : Longint;
    FLoopbaanID : Longint;
    FLeerlingID : Longint;
    FPersoneelID : Longint;
    FPersoneelNaam : AnsiString;
    FPersoneelVoornaam : AnsiString;
    FRolMask : Longint;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property InuitID : Longint Read FInuitID Write FInuitID;
    Property LoopbaanID : Longint Read FLoopbaanID Write FLoopbaanID;
    Property LeerlingID : Longint Read FLeerlingID Write FLeerlingID;
    Property PersoneelID : Longint Read FPersoneelID Write FPersoneelID;
    Property PersoneelNaam : AnsiString Read FPersoneelNaam Write FPersoneelNaam;
    Property PersoneelVoornaam : AnsiString Read FPersoneelVoornaam Write FPersoneelVoornaam;
    Property RolMask : Longint Read FRolMask Write FRolMask;
  end;


  { TBegeleiderMap }

  TBegeleiderMap = Class(TFieldMap)
  Private
    FIU_ID : TField;
    FLB_ID : TField;
    FLL_ID : TField;
    FPERSONEELID : TField;
    FPERSONEELNAAM : TField;
    FPERSONEELVOORNAAM : TField;
    FROLMASK : TField;
    Procedure DoLoad(AObject : TBegeleider);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TBegeleiderList }

  TBegeleiderList = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TBegeleider;
    Procedure SetObj(Index : Integer; AValue : TBegeleider);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property Begeleiders[Index : Integer] : TBegeleider Read GetObj Write SetObj; Default;
  end;

  { TKlasgenootMap }

  TKlasgenootMap = Class(TFieldMap)
  Private
    FLB_KLASNUMMER : TField;
    FLL_ID : TField;
    FLL_NAAM : TField;
    FLL_VOORNAAM : TField;
    Procedure DoLoad(AObject : TKlasgenoot);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TKlasgenoten }

  TKlasgenoten = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TKlasgenoot;
    Procedure SetObj(Index : Integer; AValue : TKlasgenoot);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property Klasgenoten[Index : Integer] : TKlasgenoot Read GetObj Write SetObj; Default;
  end;

  TVak = Class(TCollectionItem)
  Private
    FLeerkrachtID : Longint;
    FLeerkrachtNaam : AnsiString;
    FLeerkrachtVoornaam : AnsiString;
    FVakNaam : AnsiString;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property LeerkrachtID : Longint Read FLeerkrachtID Write FLeerkrachtID;
    Property LeerkrachtNaam : AnsiString Read FLeerkrachtNaam Write FLeerkrachtNaam;
    Property LeerkrachtVoornaam : AnsiString Read FLeerkrachtVoornaam Write FLeerkrachtVoornaam;
    Property VakNaam : AnsiString Read FVakNaam Write FVakNaam;
  end;


  { TVakMap }

  TVakMap = Class(TFieldMap)
  Private
    FLEERKRACHTID : TField;
    FLEERKRACHTNAAM : TField;
    FLEERKRACHTVOORNAAM : TField;
    FVAKNAAM : TField;
    Procedure DoLoad(AObject : TVak);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TVakken }

  TVakken = Class(TCollection)
  private
    function GetV(AIndex : integer): TVak;
    procedure SetV(AIndex : integer; const AValue: TVak);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property vakken[AIndex : integer] : TVak Read GetV Write SetV;default;
  end;

  { TLoopbaanData }

  TLoopbaanData = Class(TCollectionItem)
  Private
    FKalender: TKalenderItemList;
    FRapportBestanden: TRapportBestanden;
    FRapportPeriodes: TRapportPeriodes;
    FStudierichting : AnsiString;
    FDeelgemeente : AnsiString;
    FPostcode : AnsiString;
    FInstellingID : Longint;
    FInstelling : AnsiString;
    FDatumInschrijving : TDateTime;
    FDatumUitschrijving : TDateTime;
    FKlascode : AnsiString;
    FKlas : AnsiString;
    FKlasNr : AnsiString;
    FTot : TDateTime;
    FVan : TDateTime;
    FSchoolID : Longint;
    FSchoolnaam : AnsiString;
    FGraad: Ansistring;
    FInuitID: Integer;
    FKlasID: Integer;
    Fleerjaar: Ansistring;
    FLoopbaanID: Integer;
    FVakkenPakketID : Integer;
    FKlasGenoten : TKlasGenoten;
    FVakken : TVakken;
  Protected

  Public
    Destructor Destroy; override;
    Procedure Assign(ASource : TPersistent); override;
    Procedure LoadKlasgenoten(ADataset : TDataset);
    Procedure LoadVakken(ADataset : TDataset);
    Procedure LoadKalender(ADataset : TDataset);
    Procedure LoadRapportPeriodes(ADataset : TDataset);
    Procedure LoadRapportBestanden(ADataset : TDataset);
    Property Klasgenoten : TKlasGenoten Read FKlasGenoten;
    Property Vakken : TVakken Read FVakken;
    Property Kalender : TKalenderItemList Read FKalender;
    Property RapportPeriodes : TRapportPeriodes Read FRapportPeriodes;
    Property RapportBestanden : TRapportBestanden Read FRapportBestanden;
  Published
    Property LoopbaanID : Integer Read FLoopbaanID Write FLoopbaanID;
    Property InuitID : Integer Read FInuitID Write FInuitID;
    Property Studierichting : AnsiString Read FStudierichting Write FStudierichting;
    Property Deelgemeente : AnsiString Read FDeelgemeente Write FDeelgemeente;
    Property Postcode : AnsiString Read FPostcode Write FPostcode;
    Property InstellingID : Longint Read FInstellingID Write FInstellingID;
    Property Instelling : AnsiString Read FInstelling Write FInstelling;
    Property DatumInschrijving : TDateTime Read FDatumInschrijving Write FDatumInschrijving;
    Property DatumUitschrijving : TDateTime Read FDatumUitschrijving Write FDatumUitschrijving;
    Property KlasID : Integer Read FKlasID Write FKlasID;
    Property Klascode : AnsiString Read FKlascode Write FKlascode;
    Property Klas : AnsiString Read FKlas Write FKlas;
    Property KlasNr : AnsiString Read FKlasNr Write FKlasNr;
    Property Tot : TDateTime Read FTot Write FTot;
    Property Van : TDateTime Read FVan Write FVan;
    Property SchoolID : Longint Read FSchoolID Write FSchoolID;
    Property Schoolnaam : AnsiString Read FSchoolnaam Write FSchoolnaam;
    Property Leerjaar : Ansistring Read Fleerjaar Write FLeerjaar;
    Property Graad : Ansistring Read FGraad Write FGraad;
    Property VakkenPakketID : Integer Read FVakkenPakketID Write FVakkenPakketID;
  end;

  { TloopbaanDatas }

  TloopbaanDatas = Class(TCollection)
  private
    FCurrentLoopbaan: TloopbaanData;
    FlastLoopbaan: TLoopbaanData;
    function GetLB(AIndex : Integer): TLoopbaanData;
    procedure SetLB(AIndex : Integer; const AValue: TLoopbaanData);
  Public
    Procedure CalcLastCurrent;
    Property Lastloopbaan : TLoopbaanData Read FlastLoopbaan;
    Property CurrentLoopbaan : TloopbaanData Read FCurrentLoopbaan;
    Function AddLoopbaan(ALBID : Integer) : TLoopbaanData;
    Function IndexOfLoopbaan(Const ALBID : Integer) : Integer;
    Function LoopbaanByID(Const ALBID : Integer) : TLoopbaanData;
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property Loopbanen[AIndex : Integer] : TLoopbaanData Read GetLB Write SetLB; default;
  end;

  { TLoopbaanDataMap }

  TLoopbaanDataMap = Class(TFieldMap)
  Private
    FAG_OMSCHRIJVING : TField;
    FGM_DEELGEMEENTE : TField;
    FGM_POSTCODE : TField;
    FIS_ID : TField;
    FLB_ID : TField;
    FIU_ID : TField;
    FKL_ID : TField;
    FIS_NAAMGEBRUIKER : TField;
    FIU_DATUMINSCHRIJVING : TField;
    FIU_DATUMUITSCHRIJVING : TField;
    FKL_CODE : TField;
    FKL_OMSCHRIJVING : TField;
    FLB_KLASNUMMER : TField;
    FLB_TOT : TField;
    FLB_VAN : TField;
    FSC_ID : TField;
    FSC_NAAM : TField;
    FLB_GRAAD : TField;
    FLB_LEERJAAR : TField;
    FLB_VAKKENPAKKET_FK : TField;
    Procedure DoLoad(AObject : TLoopbaanData);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TArtikel }

  TArtikel = Class(TCollectionItem)
  Private
    FArtikelCode : AnsiString;
    FArtikelOmschrijving : AnsiString;
    FBedrag : Currency;
    FDatum : TDateTime;
    FDetailType : AnsiString;
    FDetailTypeCode : AnsiString;
    FDetailID : Longint;
    FEenheidsPrijs : Currency;
    FAantal : Currency;
    FKorting : Currency;
    FOmschrijving : AnsiString;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property ArtikelCode : AnsiString Read FArtikelCode Write FArtikelCode;
    Property ArtikelOmschrijving : AnsiString Read FArtikelOmschrijving Write FArtikelOmschrijving;
    Property Bedrag : Currency Read FBedrag Write FBedrag;
    Property Datum : TDateTime Read FDatum Write FDatum;
    Property DetailType : AnsiString Read FDetailType Write FDetailType;
    Property DetailTypeCode : AnsiString Read FDetailTypeCode Write FDetailTypeCode;
    Property DetailID : Longint Read FDetailID Write FDetailID;
    Property EenheidsPrijs : Currency Read FEenheidsPrijs Write FEenheidsPrijs;
    Property Aantal : Currency Read FAantal Write FAantal;
    Property Korting : Currency Read FKorting Write FKorting;
    Property Omschrijving : AnsiString Read FOmschrijving Write FOmschrijving;
  end;


  { TArtikelMap }

  TArtikelMap = Class(TFieldMap)
  Private
    FAR_CODE : TField;
    FAR_DESCRIPTION : TField;
    FCDD_AMOUNT : TField;
    FCDD_DATEDELIVERY : TField;
    FCDD_DETAILTYPE : TField;
    FCDD_DETAILTYPECODE : TField;
    FCDD_ID : TField;
    FCDD_PRICE : TField;
    FCDD_QTY : TField;
    FCDD_REDUCTION : TField;
    FCDT_DESCRIPTION : TField;
    Procedure DoLoad(AObject : TArtikel);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TArtikelList }

  TArtikelList = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TArtikel;
    Procedure SetObj(Index : Integer; AValue : TArtikel);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property Artikels[Index : Integer] : TArtikel Read GetObj Write SetObj; Default;
  end;

  { TFactuur }

  TFactuur = Class(TCollectionItem)
  Private
    FArtikels: TArtikelList;
    FBetaald : Currency;
    FFactuurDatum : TDateTime;
    FDocumentNr : Longint;
    FDomiciliering : AnsiString;
    FTebetalenTegen : TDateTime;
    FCDID : Longint;
    FMemo : String;
    FNo : Longint;
    FOGM : AnsiString;
    FTotaal : Currency;
    FDagboekCode : AnsiString;
    FGedeeldeBetaling : AnsiString;
    FIsBetaald : Longint;
    FHoofdGroepID : Longint;
    FTypeDetailBoek : AnsiString;
  Public
    Destructor Destroy; override;
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure LoadArtikels(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
    Property Artikels : TArtikelList Read FArtikels;
  Published
    Property Betaald : Currency Read FBetaald Write FBetaald;
    Property FactuurDatum : TDateTime Read FFactuurDatum Write FFactuurDatum;
    Property DocumentNr : Longint Read FDocumentNr Write FDocumentNr;
    Property Domiciliering : AnsiString Read FDomiciliering Write FDomiciliering;
    Property TebetalenTegen : TDateTime Read FTebetalenTegen Write FTebetalenTegen;
    Property CDID : Longint Read FCDID Write FCDID;
    Property Memo : String Read FMemo Write FMemo;
    Property No : Longint Read FNo Write FNo;
    Property OGM : AnsiString Read FOGM Write FOGM;
    Property Totaal : Currency Read FTotaal Write FTotaal;
    Property DagboekCode : AnsiString Read FDagboekCode Write FDagboekCode;
    Property GedeeldeBetaling : AnsiString Read FGedeeldeBetaling Write FGedeeldeBetaling;
    Property IsBetaald : Longint Read FIsBetaald Write FIsBetaald;
    Property HoofdGroepID : Longint Read FHoofdGroepID Write FHoofdGroepID;
    Property TypeDetailBoek : AnsiString Read FTypeDetailBoek Write FTypeDetailBoek;
  end;


  { TFactuurMap }

  TFactuurMap = Class(TFieldMap)
  Private
    FBETAALD : TField;
    FCD_DATE : TField;
    FCD_DOCUMENT : TField;
    FCD_DOMICIL : TField;
    FCD_DUEDATE : TField;
    FCD_ID : TField;
    FCD_MEMO : TField;
    FCD_NO : TField;
    FCD_OGM : TField;
    FCD_SUMTOTAL : TField;
    FDB_CODE : TField;
    FDB_SHAREPAYMENT : TField;
    FISBETAALD : TField;
    FMG_ID : TField;
    FTYPEDETAILBOEK : TField;
    Procedure DoLoad(AObject : TFactuur);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TFactuurList }

  TFactuurList = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TFactuur;
    Procedure SetObj(Index : Integer; AValue : TFactuur);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property Facturen[Index : Integer] : TFactuur Read GetObj Write SetObj; Default;
  end;

  { TLeerlingData }

  TLeerlingData = Class(TCollectionItem)
  private
    FAdressen: TAdressen;
    FAfwezigheden: TAfwezigheden;
    FBankrekening: AnsiString;
    FBankrekeningbic: AnsiString;
    FBegeleidingen: TBegeleidingen;
    FBisnummer: AnsiString;
    FEmail: AnsiString;
    FFacturen: TFactuurlist;
    FFoto: AnsiString;
    FGeboortedatum: TDateTime;
    FGeboortedatumOnbekend: Boolean;
    FGeboortegemeente: AnsiString;
    FGeboortegemeentepostnummer: AnsiString;
    FGeslacht: AnsiString;
    FGOK: Longint;
    FGSM: AnsiString;
    FIdentiteitskaartNr: AnsiString;
    FInterventies: TInterventies;
    FLLID: Integer;
    FLL_OVERLIJDENSDATUM: TDateTime;
    FLL_ROEPNAAM: AnsiString;
    FMoederBurgerlijkeStaat: AnsiString;
    FMoederEmail: String;
    FMoederGSM: String;
    FMoederNaam: AnsiString;
    FMoedertaal: AnsiString;
    FMoederVoornaam: AnsiString;
    FNaam: String;
    FNationaliteit: AnsiString;
    FNoodtelefoon: AnsiString;
    FNOODTELEFOONBIJ: AnsiString;
    FPDL: Boolean;
    FRijksregisternr: AnsiString;
    FThuistaal: AnsiString;
    FVaderBurgerlijkeStaat: AnsiString;
    FVaderEmail: String;
    FVaderGSM: String;
    FVaderNaam: AnsiString;
    FVaderVoornaam: AnsiString;
    FVeranderdop: TDateTime;
    FVlaggen: Longint;
    FVoogdNaam: AnsiString;
    FVoogdVoornaam: AnsiString;
    FVoorNaam: String;
    FVoornaam2: AnsiString;
    FBroerZus : TList;
    FLoopbaanData : TloopbaanDatas;
    FBegeleiders : TBegeleiderList;
    FZorgList: TLeerlingZorgList;
    FMedischeFiche : TMedischeFiche;
    function GetBroerZusCount: Integer;
    function GetBZ(AIndex : Integer): TLeerlingData;
    function GetLeerlingen: TLeerlingDatas;
  Public
    Constructor Create (ACollection : TCollection); override;
    Destructor Destroy; override;
    Function AddBroerZus(LL : TLeerlingData) : Integer;
    Procedure LoadLoopbanen(Dataset : TDataset);
    Procedure LoadBegeleiders(Dataset : TDataset);
    Procedure LoadAfwezigheden(Dataset : TDataset);
    Procedure LoadZorgList(Dataset : TDataset);
    Procedure LoadMedischeFiche(Dataset : TDataset);
    Procedure LoadBegeleidingen(Dataset : TDataset);
    Procedure LoadInterventies(Dataset : TDataset);
    Procedure LoadFacturen(Dataset : TDataset);
    Function GetLaatstCurrentLoopbaan : TLoopbaanData;
    Property Leerlingen : TLeerlingDatas Read GetLeerlingen;
    Property Adressen : TAdressen Read FAdressen;
    Property Facturen : TFactuurlist Read FFacturen;
    Property BroerZus [AIndex : Integer] : TLeerlingData Read GetBZ;
    Property BroerZusCount : Integer read GetBroerZusCount;
    Property Loopbanen : TloopbaanDatas Read FLoopbaanData;
    Property PedagogischTeam : TBegeleiderList Read FBegeleiders;
    Property PersoonsDataLoaded : Boolean Read FPDL Write FPDL;
    Property Afwezigheden : TAfwezigheden Read FAfwezigheden;
    Property ZorgList : TLeerlingZorgList Read FZorgList;
    Property MedischeFiche : TMedischeFiche Read FMedischeFiche;
    Property Begeleidingen : TBegeleidingen Read FBegeleidingen;
    Property Interventies : TInterventies Read FInterventies;
  Published
    Property LL_ID : Integer Read FLLID Write FLLID;
    Property Naam : String Read FNaam Write FNaam;
    Property Voornaam : String Read FVoorNaam Write FVoornaam;
    // Persoonsdata
    Property Bankrekening : AnsiString Read FBankrekening Write FBankrekening;
    Property Bankrekeningbic : AnsiString Read FBankrekeningbic Write FBankrekeningbic;
    Property Bisnummer : AnsiString Read FBisnummer Write FBisnummer;
    Property Email : AnsiString Read FEmail Write FEmail;
    Property Vlaggen : Longint Read FVlaggen Write FVlaggen;
    Property Foto : AnsiString Read FFoto Write FFoto;
    Property Geboortedatum : TDateTime Read FGeboortedatum Write FGeboortedatum;
    Property GeboortedatumOnbekend : Boolean Read FGeboortedatumOnbekend Write FGeboortedatumOnbekend;
    Property Geboortegemeente : AnsiString Read FGeboortegemeente Write FGeboortegemeente;
    Property Geboortegemeentepostnummer : AnsiString Read FGeboortegemeentepostnummer Write FGeboortegemeentepostnummer;
    Property Geslacht : AnsiString Read FGeslacht Write FGeslacht;
    Property GOK : Longint Read FGOK Write FGOK;
    Property GSM : AnsiString Read FGSM Write FGSM;
    Property IdentiteitskaartNr : AnsiString Read FIdentiteitskaartNr Write FIdentiteitskaartNr;
    Property Moedertaal : AnsiString Read FMoedertaal Write FMoedertaal;
    Property Nationaliteit : AnsiString Read FNationaliteit Write FNationaliteit;
    Property Noodtelefoon : AnsiString Read FNoodtelefoon Write FNoodtelefoon;
    Property NOODTELEFOONBIJ : AnsiString Read FNOODTELEFOONBIJ Write FNOODTELEFOONBIJ;
    Property Overlijdensdatum: TDateTime Read FLL_OVERLIJDENSDATUM Write FLL_OVERLIJDENSDATUM;
    Property Rijksregisternr : AnsiString Read FRijksregisternr Write FRijksregisternr;
    Property Roepnaam: AnsiString Read FLL_ROEPNAAM Write FLL_ROEPNAAM;
    Property Thuistaal : AnsiString Read FThuistaal Write FThuistaal;
    Property Veranderdop : TDateTime Read FVeranderdop Write FVeranderdop;
    Property Voornaam2 : AnsiString Read FVoornaam2 Write FVoornaam2;
    Property MoederNaam : AnsiString Read FMoederNaam Write FMoederNaam;
    Property VaderNaam : AnsiString Read FVaderNaam Write FVaderNaam;
    Property VoogdNaam : AnsiString Read FVoogdNaam Write FVoogdNaam;
    Property MoederVoornaam : AnsiString Read FMoederVoornaam Write FMoederVoornaam;
    Property VaderVoornaam : AnsiString Read FVaderVoornaam Write FVaderVoornaam;
    Property VoogdVoornaam : AnsiString Read FVoogdVoornaam Write FVoogdVoornaam;
    Property MoederGSM : String Read FMoederGSM Write FMoederGSM;
    Property VaderGSM : String Read FVaderGSM Write FVaderGSM;
    Property VaderEmail: String Read FVaderEmail Write FVaderEmail;
    Property MoederEmail: String Read FMoederEmail Write FMoederEmail;
    Property MoederBurgerlijkeStaat : AnsiString Read FMoederBurgerlijkeStaat Write FMoederBurgerlijkeStaat;
    Property VaderBurgerlijkeStaat : AnsiString Read FVaderBurgerlijkeStaat Write FVaderBurgerlijkeStaat;
   end;

  { TAdres }

  TAdres = Class(TCollectionItem)
  Private
    FGemeente : AnsiString;
    FPostnummer : AnsiString;
    FAanspreking : AnsiString;
    FNaam : AnsiString;
    FStraat : AnsiString;
    FNr : AnsiString;
    FTelefoon : AnsiString;
    FTypeAdres : AnsiString;
    FTypeAdresCode : AnsiString;
    FLand : AnsiString;
    FBus : AnsiString;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property Gemeente : AnsiString Read FGemeente Write FGemeente;
    Property Postnummer : AnsiString Read FPostnummer Write FPostnummer;
    Property Aanspreking : AnsiString Read FAanspreking Write FAanspreking;
    Property Naam : AnsiString Read FNaam Write FNaam;
    Property Straat : AnsiString Read FStraat Write FStraat;
    Property Nr : AnsiString Read FNr Write FNr;
    Property Bus : AnsiString Read FBus Write FBus;
    Property Telefoon : AnsiString Read FTelefoon Write FTelefoon;
    Property TypeAdres : AnsiString Read FTypeAdres Write FTypeAdres;
    Property TypeAdresCode : AnsiString Read FTypeAdresCode Write FTypeAdresCode;
    Property Land : AnsiString Read FLand Write FLand;
  end;

  { TAdresMap }

  TAdresMap = Class(TFieldMap)
  Private
    FGemeente : TField;
    FPostnummer : TField;
    FAanspreking : TField;
    FNaam : TField;
    FStraat : TField;
    FNr : TField;
    FBus : TField;
    FTelefoon : TField;
    FTypeAdres : TField;
    FTypeAdresCode : TField;
    FLand : TField;
    Procedure DoLoad(AObject : TAdres);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TAdressen }

  TAdressen = Class(TCollection)
  private
    function GetA(AIndex : Integer): TAdres;
    procedure SetA(AIndex : Integer; const AValue: TAdres);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property Adres[AIndex : Integer] : TAdres Read GetA Write SetA;  default;
  end;

  { TLeerlingDatas }

  TLeerlingDatas = Class(TCollection)
  private
    function GetL(Aindex : Integer): TLeerlingData;
    procedure SetL(Aindex : Integer; const AValue: TLeerlingData);
  Public
    Function IndexOfLLID(ALLID : Integer) : Integer;
    Function FindLeerling(ALLID : Integer) : TLeerlingData;
    Function AddLeerling(ALLID : Integer) : TLeerlingData;
    Property Leerling[Aindex : Integer] : TLeerlingData Read GetL Write SetL; default;
  end;

  { TUserData }

  TUserData = Class(TPersistent)
  private
    FMijnLeerlingen: TLeerlingDatas;
    FNaam: String;
    FPassword: String;
    FRol: TleerlingRol;
    FU_ID: Integer;
    FVoorNaam: String;
  Public
    Constructor Create(AID : Integer);
    Destructor Destroy; override;
    Property U_ID : Integer Read FU_ID;
    Property Naam : String Read FNaam Write FNaam;
    Property Voornaam : String Read FVoorNaam Write FVoornaam;
    Property MijnLeerlingen : TLeerlingDatas Read FMijnLeerlingen;
    Property Rol : TleerlingRol Read FRol Write FRol;
    Property password : String Read FPassword Write FPassword;
  end;

  { TLVSRolFieldMap }

  TLVSRolFieldMap = Class(TFieldMap)
  private
    FIDField: TField;
    FDescrField: TField;
    FCodeField: TField;
    FSecField: TField;
  Public
    Procedure InitFields; override;
    Procedure LoadObject(AObject : TObject); Override;
    Property IDField : TField Read FIDField;
    Property CodeField : TField Read FCodeField;
    Property DescrField : TField Read FDescrField;
    Property SecurityLevelField : TField Read FSecField;
  end;

  TLVSRol = Class(TCollectionItem)
  private
    FSecurityLevel: Integer;
    FLVRID: Integer;
    FDescription: String;
    FCode: String;
  Protected
    Procedure LoadFromMap(Map : TLVSRolFieldMap);
  Published
    Property Code : String Read FCode Write FCode;
    Property Description : String Read FDescription Write FDescription;
    Property SecurityLevel : Integer Read FSecurityLevel Write FSecurityLevel;
    Property LVRID : Integer Read FLVRID Write FLVRID;
  end;

  TLVSRollen = Class(TCollection)
  private
    function GetR(Index: Integer): TLVSRol;
    function GetRI(Index: Integer): TLVSRol;
    procedure SetR(Index: Integer; const Value: TLVSRol);
    procedure SetRI(Index: Integer; const Value: TLVSRol);
  Public
    Function AddRol : TLVSRol;
    Function GetMaxSecurity(ARollen : Integer) : Integer;
    Procedure LoadFromDataset(Dataset : TDataset);
    Procedure LoadFromMap(Map : TLVSRolFieldMap);
    Property Rollen[Index : Integer] : TLVSRol Read GetR Write SetR; default;
    Property RolByID[Index : Integer] : TLVSRol  Read GetRI Write SetRI;
  end;

  { TAfwezigheid }

  TAfwezigheid = Class(TCollectionItem)
  Private
    FCode : AnsiString;
    FKleur : Longint;
    FOmschrijving : AnsiString;
    FBriefjevan : AnsiString;
    FCommentaar : AnsiString;
    FAFID : Longint;
    FReferentie : AnsiString;
    FTot: TDateTime;
    FTotUur : Smallint;
    FVan : TDateTime;
    FVanuur : SmallInt;
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property Code : AnsiString Read FCode Write FCode;
    Property Kleur : Longint Read FKleur Write FKleur;
    Property Omschrijving : AnsiString Read FOmschrijving Write FOmschrijving;
    Property Briefjevan : AnsiString Read FBriefjevan Write FBriefjevan;
    Property Commentaar : AnsiString Read FCommentaar Write FCommentaar;
    Property AFID : Longint Read FAFID Write FAFID;
    Property Referentie : AnsiString Read FReferentie Write FReferentie;
    Property Tot : TDateTime Read FTot Write FTot;
    Property TotUur : Smallint Read FTotUur Write FTotUur;
    Property Van : TDateTime Read FVan Write FVan;
    Property Vanuur : SmallInt Read FVanuur Write FVanuur;
  end;


  { TAfwezigheidMap }

  TAfwezigheidMap = Class(TFieldMap)
  Private
    FAC_CODE : TField;
    FAC_KLEUR : TField;
    FAC_OMSCHRIJVING : TField;
    FAF_BRIEFJEVAN : TField;
    FAF_COMMENTAAR : TField;
    FAF_ID : TField;
    FAF_REFERENTIE : TField;
    FAF_TOT : TField;
    FAF_TOTUUR : TField;
    FAF_VAN : TField;
    FAF_VANUUR : TField;
    Procedure DoLoad(AObject : TAfwezigheid);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TAfwezigheden }

  TAfwezigheden = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TAfwezigheid;
    Procedure SetObj(Index : Integer; AValue : TAfwezigheid);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property Afwezigheden[Index : Integer] : TAfwezigheid Read GetObj Write SetObj; Default;
  end;

  { TLeerlingZorg }

  TLeerlingZorg = Class(TCollectionItem)
  Private
    FDatum : TDateTime;
    FGebied : SmallInt;
    FLZID : Longint;
    FOpmerking : AnsiString;
    FType : AnsiString;
    FTypeCode : AnsiString;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property Datum : TDateTime Read FDatum Write FDatum;
    Property Gebied : SmallInt Read FGebied Write FGebied;
    Property LZID : Longint Read FLZID Write FLZID;
    Property Opmerking : AnsiString Read FOpmerking Write FOpmerking;
    Property ZorgType : AnsiString Read FType Write FType;
    Property TypeCode : AnsiString Read FTypeCode Write FTypeCode;
  end;


  { TLeerlingZorgMap }

  TLeerlingZorgMap = Class(TFieldMap)
  Private
    FLZ_DATUM : TField;
    FLZ_GEBIED : TField;
    FLZ_ID : TField;
    FLZ_OPMERKING : TField;
    FLZ_TYPE : TField;
    FLZ_TYPECODE : TField;
    Procedure DoLoad(AObject : TLeerlingZorg);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TLeerlingZorgList }

  TLeerlingZorgList = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TLeerlingZorg;
    Procedure SetObj(Index : Integer; AValue : TLeerlingZorg);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property LeerlingZorgs[Index : Integer] : TLeerlingZorg Read GetObj Write SetObj; Default;
  end;
  { TBegeleiding }

  TBegeleiding = Class(TCollectionItem)
  Private
    FAchterliggendProbleem : AnsiString;
    FBetrokkenPersoon1 : AnsiString;
    FBetrokkenPersoon2 : AnsiString;
    FBetrokkenPersoon3 : AnsiString;
    FBetrokkenPersoonType1 : SmallInt;
    FBetrokkenPersoonType2 : SmallInt;
    FBetrokkenPersoonType3 : SmallInt;
    FDatum : TDateTime;
    FExterneIdentificatie1 : AnsiString;
    FExterneIdentificatie2 : AnsiString;
    FFinancieleTussenKomst : Currency;
    FVlaggen : Longint;
    FBGID : Longint;
    FInterventieMotief : AnsiString;
    FOmschrijving : AnsiString;
    FOpmerking : AnsiString;
    FStatus : AnsiString;
    FSubtype : AnsiString;
    FSubtypeCode : AnsiString;
    FTijd : TDateTime;
    FBegeleidingType : AnsiString;
    FBegeleidingTypeCode : AnsiString;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property AchterliggendProbleem : AnsiString Read FAchterliggendProbleem Write FAchterliggendProbleem;
    Property BetrokkenPersoon1 : AnsiString Read FBetrokkenPersoon1 Write FBetrokkenPersoon1;
    Property BetrokkenPersoon2 : AnsiString Read FBetrokkenPersoon2 Write FBetrokkenPersoon2;
    Property BetrokkenPersoon3 : AnsiString Read FBetrokkenPersoon3 Write FBetrokkenPersoon3;
    Property BetrokkenPersoonType1 : SmallInt Read FBetrokkenPersoonType1 Write FBetrokkenPersoonType1;
    Property BetrokkenPersoonType2 : SmallInt Read FBetrokkenPersoonType2 Write FBetrokkenPersoonType2;
    Property BetrokkenPersoonType3 : SmallInt Read FBetrokkenPersoonType3 Write FBetrokkenPersoonType3;
    Property Datum : TDateTime Read FDatum Write FDatum;
    Property ExterneIdentificatie1 : AnsiString Read FExterneIdentificatie1 Write FExterneIdentificatie1;
    Property ExterneIdentificatie2 : AnsiString Read FExterneIdentificatie2 Write FExterneIdentificatie2;
    Property FinancieleTussenKomst : Currency Read FFinancieleTussenKomst Write FFinancieleTussenKomst;
    Property Vlaggen : Longint Read FVlaggen Write FVlaggen;
    Property BGID : Longint Read FBGID Write FBGID;
    Property InterventieMotief : AnsiString Read FInterventieMotief Write FInterventieMotief;
    Property Omschrijving : AnsiString Read FOmschrijving Write FOmschrijving;
    Property Opmerking : AnsiString Read FOpmerking Write FOpmerking;
    Property Status : AnsiString Read FStatus Write FStatus;
    Property Subtype : AnsiString Read FSubtype Write FSubtype;
    Property SubtypeCode : AnsiString Read FSubtypeCode Write FSubtypeCode;
    Property Tijd : TDateTime Read FTijd Write FTijd;
    Property BegeleidingType : AnsiString Read FBegeleidingType Write FBegeleidingType;
    Property BegeleidingTypeCode : AnsiString Read FBegeleidingTypeCode Write FBegeleidingTypeCode;
  end;


  { TBegeleidingMap }

  TBegeleidingMap = Class(TFieldMap)
  Private
    FBG_ACHTERLIGGENDPROBLEEM : TField;
    FBG_BETROKKENPERSOON1 : TField;
    FBG_BETROKKENPERSOON2 : TField;
    FBG_BETROKKENPERSOON3 : TField;
    FBG_BETROKKENPERSOONTYPE1 : TField;
    FBG_BETROKKENPERSOONTYPE2 : TField;
    FBG_BETROKKENPERSOONTYPE3 : TField;
    FBG_DATUM : TField;
    FBG_EXTERNEIDENTIFICATIE1 : TField;
    FBG_EXTERNEIDENTIFICATIE2 : TField;
    FBG_FINANCIELETUSSENKOMST : TField;
    FBG_FLAGS : TField;
    FBG_ID : TField;
    FBG_INTERVENTIEMOTIEF : TField;
    FBG_OMSCHRIJVING : TField;
    FBG_OPMERKING : TField;
    FBG_STATUS : TField;
    FBG_SUBTYPE : TField;
    FBG_SUBTYPECODE : TField;
    FBG_TIJD : TField;
    FBG_TYPE : TField;
    FBG_TYPECODE : TField;
    Procedure DoLoad(AObject : TBegeleiding);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TBegeleidingen }

  TBegeleidingen = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TBegeleiding;
    Procedure SetObj(Index : Integer; AValue : TBegeleiding);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property Begeleidingen[Index : Integer] : TBegeleiding Read GetObj Write SetObj; Default;
  end;
  { TInterventie }

  TInterventie = Class(TCollectionItem)
  Private
    FBetrokkenpersoon : AnsiString;
    FBetrokkenPersoonType : SmallInt;
    FDatum : TDateTime;
    FDerden : AnsiString;
    FVlaggen : Longint;
    FLIID : Longint;
    FMotief : AnsiString;
    FMotiefCode : AnsiString;
    FNiveau : SmallInt;
    FOmschrijving : AnsiString;
    FOpmerking : AnsiString;
    FPlaats : AnsiString;
    FTijd : TDateTime;
    FInterventieType : AnsiString;
    FTypeCode : AnsiString;
  Private
    FBegeleidingID: Integer;
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property Betrokkenpersoon : AnsiString Read FBetrokkenpersoon Write FBetrokkenpersoon;
    Property BetrokkenPersoonType : SmallInt Read FBetrokkenPersoonType Write FBetrokkenPersoonType;
    Property Datum : TDateTime Read FDatum Write FDatum;
    Property Derden : AnsiString Read FDerden Write FDerden;
    Property Vlaggen : Longint Read FVlaggen Write FVlaggen;
    Property LIID : Longint Read FLIID Write FLIID;
    Property Motief : AnsiString Read FMotief Write FMotief;
    Property MotiefCode : AnsiString Read FMotiefCode Write FMotiefCode;
    Property Niveau : SmallInt Read FNiveau Write FNiveau;
    Property Omschrijving : AnsiString Read FOmschrijving Write FOmschrijving;
    Property Opmerking : AnsiString Read FOpmerking Write FOpmerking;
    Property Plaats : AnsiString Read FPlaats Write FPlaats;
    Property Tijd : TDateTime Read FTijd Write FTijd;
    Property InterventieType : AnsiString Read FInterventieType Write FInterventieType;
    Property TypeCode : AnsiString Read FTypeCode Write FTypeCode;
    Property BegeleidingID : Integer Read FBegeleidingID Write FBegeleidingID;
  end;


  { TInterventieMap }

  TInterventieMap = Class(TFieldMap)
  Private
    FLI_BETROKKENPERSOON : TField;
    FLI_BETROKKENPERSOONTYPE : TField;
    FLI_DATUM : TField;
    FLI_DERDEN : TField;
    FLI_FLAGS : TField;
    FLI_ID : TField;
    FLI_MOTIEF : TField;
    FLI_MOTIEFCODE : TField;
    FLI_NIVEAU : TField;
    FLI_OMSCHRIJVING : TField;
    FLI_OPMERKING : TField;
    FLI_PLAATS : TField;
    FLI_TIJD : TField;
    FLI_TYPE : TField;
    FLI_TYPECODE : TField;
    FLI_BEGELEIDINGID : TField;
    Procedure DoLoad(AObject : TInterventie);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TInterventies }

  TInterventies = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TInterventie;
    Procedure SetObj(Index : Integer; AValue : TInterventie);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property Interventies[Index : Integer] : TInterventie Read GetObj Write SetObj; Default;
  end;
  { TKalenderItem }

  TKalenderItem = Class(TCollectionItem)
  Private
    FCode : AnsiString;
    FCodeID : Longint;
    FOmschrijving : AnsiString;
    FKDID : Longint;
    FTot : TDateTime;
    FTotUur : SmallInt;
    FVan : TDateTime;
    FVanuur : SmallInt;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property Code : AnsiString Read FCode Write FCode;
    Property CodeID : Longint Read FCodeID Write FCodeID;
    Property Omschrijving : AnsiString Read FOmschrijving Write FOmschrijving;
    Property KDID : Longint Read FKDID Write FKDID;
    Property Tot : TDateTime Read FTot Write FTot;
    Property TotUur : SmallInt Read FTotUur Write FTotUur;
    Property Van : TDateTime Read FVan Write FVan;
    Property Vanuur : SmallInt Read FVanuur Write FVanuur;
  end;


  { TKalenderItemMap }

  TKalenderItemMap = Class(TFieldMap)
  Private
    FAC_CODE : TField;
    FAC_ID : TField;
    FAC_OMSCHRIJVING : TField;
    FKD_ID : TField;
    FKD_TOT : TField;
    FKD_TOTUUR : TField;
    FKD_VAN : TField;
    FKD_VANUUR : TField;
    Procedure DoLoad(AObject : TKalenderItem);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TKalenderItemList }

  TKalenderItemList = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TKalenderItem;
    Procedure SetObj(Index : Integer; AValue : TKalenderItem);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property KalenderItems[Index : Integer] : TKalenderItem Read GetObj Write SetObj; Default;
  end;

  TRapportPeriode = Class(TCollectionItem)
  Private
    FCode : AnsiString;
    FDatum : TDateTime;
    FPEID : Longint;
    FOmschrijving : AnsiString;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property Code : AnsiString Read FCode Write FCode;
    Property Datum : TDateTime Read FDatum Write FDatum;
    Property PEID : Longint Read FPEID Write FPEID;
    Property Omschrijving : AnsiString Read FOmschrijving Write FOmschrijving;
  end;


  { TRapportPeriodeMap }

  TRapportPeriodeMap = Class(TFieldMap)
  Private
    FPE_CODE : TField;
    FPED_DATUM : TField;
    FPE_ID : TField;
    FPE_OMSCHRIJVING : TField;
    Procedure DoLoad(AObject : TRapportPeriode);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TRapportPeriodes }

  TRapportPeriodes = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TRapportPeriode;
    Procedure SetObj(Index : Integer; AValue : TRapportPeriode);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property RapportPeriodes[Index : Integer] : TRapportPeriode Read GetObj Write SetObj; Default;
  end;

  { TRapportBestand }

  TRapportBestand = Class(TCollectionItem)
  Private
    FDocumentID : Longint;
    FLaatsteWijziging : TDateTime;
    FBestandsNaam : AnsiString;
    FOmschrijving : AnsiString;
    FOrigineleNaam : AnsiString;
    FRef1 : AnsiString;
    FRef2 : AnsiString;
  Private
  Protected
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
  Published
    Property DocumentID : Longint Read FDocumentID Write FDocumentID;
    Property LaatsteWijziging : TDateTime Read FLaatsteWijziging Write FLaatsteWijziging;
    Property BestandsNaam : AnsiString Read FBestandsNaam Write FBestandsNaam;
    Property Omschrijving : AnsiString Read FOmschrijving Write FOmschrijving;
    Property OrigineleNaam : AnsiString Read FOrigineleNaam Write FOrigineleNaam;
    Property Ref1 : AnsiString Read FRef1 Write FRef1;
    Property Ref2 : AnsiString Read FRef2 Write FRef2;
  end;


  { TRapportBestandMap }

  TRapportBestandMap = Class(TFieldMap)
  Private
    FD_ID : TField;
    FD_LAATSTEWIJZIGING : TField;
    FD_NAAM : TField;
    FD_OMSCHRIJVING : TField;
    FD_ORIGINELENAAM : TField;
    FD_REFERENTIE1 : TField;
    FD_REFERENTIE2 : TField;
    Procedure DoLoad(AObject : TRapportBestand);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TRapportBestanden }

  TRapportBestanden = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TRapportBestand;
    Procedure SetObj(Index : Integer; AValue : TRapportBestand);
  Public
    Function IndexOfBestand(DocID : Integer) : Integer;
    Function FindBestand(DocID : Integer) : TRapportBestand;
    Function FindBestandForPeriode(APeriodeID : integer) : TRapportBestand; overload;
    Function FindBestandForPeriode(APeriodeCode : String) : TRapportBestand; overload;
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Property RapportBestand[Index : Integer] : TRapportBestand Read GetObj Write SetObj; Default;
  end;

  { TLoginData }

  TLoginData = Class(TCollectionItem)
  Private
    FLeerlingID : Longint;
    FMoederEmail : AnsiString;
    FMoedernaam : AnsiString;
    FMoederUID : Longint;
    FMoederVoornaam : AnsiString;
    FNaam : AnsiString;
    FRijksregisterNummer : AnsiString;
    FVaderEmail : AnsiString;
    FVadernaam : AnsiString;
    FVaderUID : Longint;
    FVaderVoornaam : AnsiString;
    FVoogdEmail : AnsiString;
    FVoogdnaam : AnsiString;
    FVoogdUID : Longint;
    FVoogdVoornaam : AnsiString;
    FVoornaam : AnsiString;
  Private
  Protected
  Public
    Function EmailMatch(AEmail : String; Var ARol : TLeerlingRol) : Boolean;
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(ADataset : TDataset);
    Procedure Assign(ASource : TPersistent); override;
    Function GetRolNaam(ARol : TLeerlingRol) : string;
    Function GetRolVoornaam(ARol : TLeerlingRol) : string;
    Function GetRolUID(ARoL :TLeerlingRol) : integer;
    Function GetRolEmail(ARol : TLeerlingRol) : String;
  Published
    Property LeerlingID : Longint Read FLeerlingID Write FLeerlingID;
    Property MoederEmail : AnsiString Read FMoederEmail Write FMoederEmail;
    Property Moedernaam : AnsiString Read FMoedernaam Write FMoedernaam;
    Property MoederUID : Longint Read FMoederUID Write FMoederUID;
    Property MoederVoornaam : AnsiString Read FMoederVoornaam Write FMoederVoornaam;
    Property Naam : AnsiString Read FNaam Write FNaam;
    Property RijksregisterNummer : AnsiString Read FRijksregisterNummer Write FRijksregisterNummer;
    Property VaderEmail : AnsiString Read FVaderEmail Write FVaderEmail;
    Property Vadernaam : AnsiString Read FVadernaam Write FVadernaam;
    Property VaderUID : Longint Read FVaderUID Write FVaderUID;
    Property VaderVoornaam : AnsiString Read FVaderVoornaam Write FVaderVoornaam;
    Property VoogdEmail : AnsiString Read FVoogdEmail Write FVoogdEmail;
    Property Voogdnaam : AnsiString Read FVoogdnaam Write FVoogdnaam;
    Property VoogdUID : Longint Read FVoogdUID Write FVoogdUID;
    Property VoogdVoornaam : AnsiString Read FVoogdVoornaam Write FVoogdVoornaam;
    Property Voornaam : AnsiString Read FVoornaam Write FVoornaam;
  end;


  { TLoginDataMap }

  TLoginDataMap = Class(TFieldMap)
  Private
    FLL_ID : TField;
    FLL_MOEDEREMAIL : TField;
    FLL_MOEDERNAAM : TField;
    FLL_MOEDERUSER_FK : TField;
    FLL_MOEDERVOORNAAM : TField;
    FLL_NAAM : TField;
    FLL_RIJKSREGISTERNR : TField;
    FLL_VADEREMAIL : TField;
    FLL_VADERNAAM : TField;
    FLL_VADERUSER_FK : TField;
    FLL_VADERVOORNAAM : TField;
    FLL_VOOGDEMAIL : TField;
    FLL_VOOGDNAAM : TField;
    FLL_VOOGDUSER_FK : TField;
    FLL_VOOGDVOORNAAM : TField;
    FLL_VOORNAAM : TField;
    Procedure DoLoad(AObject : TLoginData);
  Public
    Procedure InitFields; Override;
    Procedure LoadObject(AObject : TObject); Override;
  end;

  { TLoginDataList }

  TLoginDataList = Class(TCollection)
  Private
    Function GetObj(Index : Integer) : TLoginData;
    Procedure SetObj(Index : Integer; AValue : TLoginData);
  Public
    Procedure LoadFromMap(Map : TFieldMap);
    Procedure LoadFromDataset(Dataset : TDataset);
    Function GetRRMatch(Const APassword : String) : TLoginData;
    Function GetMatch(Const AUserNaam : String; Out ARol : TleerlingRol) : TLoginData;
    Property LoginDatas[Index : Integer] : TLoginData Read GetObj Write SetObj; Default;
  end;

Function LVSRollen : TLVSRollen;
Procedure InitRollen(Dataset : TDataset);
Function GetRolNamen(RolMask : Integer) : String;
Function Uurnaam (U : Integer; Long : Boolean = False) : String;
Function DatumUur(D : TDateTime; U : SmallInt) : String;
Function GetKleur(AKleur : Integer) : String;
Function DatumString(D : TDateTime) : String;
function TijdString(T: TDateTime): String;

implementation

Function Uurnaam (U : Integer; Long : Boolean = False) : String;

begin
  If Long then
    Case U of
      1 : Result:='VM';
      2 : Result:='NM';
    else
      Result:='?'
    end
  else
    Case U of
      1 : Result:='V';
      2 : Result:='N';
    else
      Result:='?'
    end;
end;

Function DatumUur(D : TDateTime; U : SmallInt) : String;

begin
 Result:=DatumString(D)+' '+UurNaam(U,True);
end;

Function GetKleur(AKleur : Integer) : String;

Var
  R,G,B : Byte;

begin
  If (AKleur=-1) or (AKleur=0) then
    Result:='FFFFFF'
  else
    begin
    B:=Akleur and $FF;
    G:=(AKleur SHR 8) and $FF;
    R:=(AKleur SHR 16) and $FF;
    Result:=HexStr(R,2)+HexStr(G,2)+HexStr(B,2);
    end;
end;

function DatumString(D: TDateTime): String;
begin
  If (Round(D)=0) then
    Result:=''
  else
    Result:=FormatDateTime('yyyy"-"mm"-"dd',D);
end;

function TijdString(T: TDateTime): String;

begin
  Result:=FormatDatetime('hh:nn',T);
end;
{ TLeerlingDatas }

function TLeerlingDatas.GetL(Aindex : Integer): TLeerlingData;
begin
  Result:=TLeerlingData(Items[AIndex])
end;

procedure TLeerlingDatas.SetL(Aindex : Integer; const AValue: TLeerlingData);
begin
  Items[AIndex]:=AValue;
end;

function TLeerlingDatas.IndexOfLLID(ALLID: Integer): Integer;
begin
  Result:=Count-1;
  While (Result>=0) and (GetL(Result).LL_ID<>ALLID) do
    Dec(Result);
end;

function TLeerlingDatas.FindLeerling(ALLID: Integer): TLeerlingData;

Var
  I : Integer;

begin
  I:=IndexOfLLID(ALLID);
  If (I=-1) then
    Result:=Nil
  else
    Result:=GetL(I);
end;

function TLeerlingDatas.AddLeerling(ALLID: Integer): TLeerlingData;
begin
  Result:=Add as TLeerlingData;
  Result.LL_ID:=ALLID;
end;

{ TUserData }

constructor TUserData.Create(AID: Integer);

begin
  FU_ID:=AID;
  FMijnLeerlingen:=TLeerlingDatas.Create(TLeerlingData);
end;

destructor TUserData.Destroy;
begin
  FreeAndNil(FMijnLeerlingen);
  inherited Destroy;
end;

 { TAdres }

Procedure TAdres.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TAdres.LoadFromDataset(ADataset : TDataset);

begin
  With TAdresMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TAdres.Assign(ASource : TPersistent);

var
  O : TAdres ;
begin
  If (ASource is TAdres) then
    begin
    O:=(ASource as TAdres);
    FGemeente:=O.FGemeente;
    FPostnummer:=O.FPostnummer;
    FAanspreking:=O.FAanspreking;
    FNaam:=O.FNaam;
    FStraat:=O.FStraat;
    FNr:=O.FNr;
    FTelefoon:=O.FTelefoon;
    FTypeAdres:=O.FTypeAdres;
    FTypeAdresCode:=O.FTypeAdresCode;
    FLand:=O.FLand;
    end
  else
    Inherited;
end;


 { TAdresMap }

Procedure TAdresMap.DoLoad(AObject : TAdres);

begin
  With AObject do
    begin
    Gemeente:=GetFromField(Self.FGemeente,Gemeente);
    Postnummer:=GetFromField(Self.FPostnummer,Postnummer);
    Aanspreking:=GetFromField(Self.FAanspreking,Aanspreking);
    Naam:=GetFromField(Self.FNaam,Naam);
    Straat:=GetFromField(Self.FStraat,Straat);
    Nr:=GetFromField(Self.FNr,Nr);
    Bus:=GetFromField(Self.FBus,Nr);
    Telefoon:=GetFromField(Self.FTelefoon,Telefoon);
    TypeAdres:=GetFromField(Self.FTypeAdres,TypeAdres);
    TypeAdresCode:=GetFromField(Self.FTypeAdresCode,TypeAdresCode);
    Land:=GetFromField(Self.FLand,Land);
    end;
end;


Procedure TAdresMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TAdres);
end;


Procedure TAdresMap.InitFields;

begin
  FGemeente:=FindField('GM_DEELGEMEENTE');
  FPostnummer:=FindField('GM_POSTNUMMER');
  FAanspreking:=FindField('LA_AANSPREKING');
  FNaam:=FindField('LA_NAAM');
  FStraat:=FindField('LA_STRAAT');
  FNr:=FindField('LA_STRAATNR');
  FNr:=FindField('LA_STRAATNR');
  FBUS:=FindField('LA_STRAATBUS');
  FTelefoon:=FindField('LA_Telefoon');
  FTypeAdres:=FindField('LA_TypeAdres');
  FTypeAdresCode:=FindField('LA_TypeAdresCode');
  FLand:=FindField('LN_ISOCODE');
end;


{ TAdressen }

function TAdressen.GetA(AIndex : Integer): TAdres;
begin
  Result:=TAdres(Items[Aindex])
end;

procedure TAdressen.SetA(AIndex : Integer; const AValue: TAdres);
begin
  Items[AIndex]:=AValue;
end;

Procedure TAdressen.LoadFromMap(Map : TFieldMap);


Var
  Obj : TAdres;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TAdres;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;



Procedure TAdressen.LoadFromDataset(Dataset : TDataset);


Var
  Map : TAdresMap;

begin
  Map:=TAdresMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

{ TLeerlingData }

function TLeerlingData.GetLeerlingen: TLeerlingDatas;
begin
  Result:=Collection as TLeerlingDatas;
end;

function TLeerlingData.GetBZ(AIndex : Integer): TLeerlingData;
begin
  If Assigned(FBroerZus) then
    Result:=TLeerlingData(FBroerZus[AIndex])
  else
    Result:=Nil;
end;

function TLeerlingData.GetBroerZusCount: Integer;
begin
  If Assigned(FBroerZus) then
    Result:=FBroerZus.Count
  else
    Result:=0;
end;


constructor TLeerlingData.Create(ACollection: TCollection);
begin
  inherited Create(ACollection);
  FAdressen:=TAdressen.Create(TAdres);
end;

destructor TLeerlingData.Destroy;
begin
  FreeAndNil(FLoopbaanData);
  FreeAndNil(FAdressen);
  FreeAndNil(FBroerZus);
  FreeAndNil(FBegeleiders);
  FreeAndNil(FAfwezigheden);
  FreeAndNil(FZorgList);
  FreeAndNil(FMedischeFiche);
  FreeAndNil(FInterventies);
  FreeAndNil(FBegeleidingen);
  FreeAndNil(FFacturen);
  inherited Destroy;
end;

function TLeerlingData.AddBroerZus(LL: TLeerlingData): Integer;
begin
  If not Assigned(FBroerZus) then
    FBroerZus:=TList.Create;
  Result:=FBroerZus.Add(LL);
end;

procedure TLeerlingData.LoadLoopbanen(Dataset: TDataset);
begin
  If not Assigned(FLoopbaanData) then
    FLoopbaanData:=TLoopbaanDatas.Create(TLoopbaanData)
  else
    FLoopBaanData.Clear;
  FLoopbaanData.LoadFromDataset(Dataset);
end;

procedure TLeerlingData.LoadBegeleiders(Dataset: TDataset);
begin
  If not Assigned(FBegeleiders) then
    FBegeleiders:=TBegeleiderList.Create(TBegeleider)
  else
    FBegeleiders.Clear;
  FBegeleiders.LoadFromDataset(Dataset);
end;

procedure TLeerlingData.LoadAfwezigheden(Dataset: TDataset);
begin
  If Not Assigned(FAfwezigheden) then
    FAfwezigheden:=TAfwezigheden.Create(TAfwezigheid)
  else
    FAfwezigheden.Clear;
  FAfwezigheden.LoadFromDataset(Dataset);
end;

procedure TLeerlingData.LoadZorgList(Dataset: TDataset);
begin
  If not Assigned(FZorgList) then
    FZorgList:=TLeerlingZorgList.Create(TLeerlingZorg)
  else
    FZorgList.Clear;
  FZorgList.LoadFromDataset(Dataset);
end;

procedure TLeerlingData.LoadMedischeFiche(Dataset: TDataset);

Var
  M : TMedischeFicheMap;

begin
  If (Dataset.EOF and Dataset.Bof) then
    begin
    If Assigned(FMedischeFiche) then
      FreeAndNil(FMedischeFiche)
    end
  else
    begin
    If Not Assigned(FMedischeFiche) then
      FMedischeFiche:=TMedischeFiche.Create;
    M:=TMedischeFicheMap.Create(Dataset);
    try
      M.LoadObject(FMedischeFiche);
    finally
      M.Free;
    end;
    end;
end;

procedure TLeerlingData.LoadBegeleidingen(Dataset: TDataset);
begin
  If not Assigned(FBegeleidingen) then
    FBegeleidingen:=TBegeleidingen.Create(TBegeleiding)
  else
    FBegeleidingen.Clear;
  FBegeleidingen.LoadFromDataset(Dataset);
end;

procedure TLeerlingData.LoadInterventies(Dataset: TDataset);
begin
  If not Assigned(FInterventies) then
    FInterventies:=TInterventies.Create(TInterventie)
  else
    FInterventies.Clear;
  Finterventies.LoadFromDataset(Dataset);
end;

procedure TLeerlingData.LoadFacturen(Dataset: TDataset);
begin
  If Assigned(FFacturen) then
    FFacturen.Clear
  else
    FFacturen:=TFactuurList.Create(TFactuur);
  FFacturen.LoadFromDataset(Dataset);
end;

function TLeerlingData.GetLaatstCurrentLoopbaan: TLoopbaanData;
begin
  Result:=Nil;
  If Assigned(FLoopbaanData) then
    If Loopbanen.Count=0 then
      begin
      Result:=Loopbanen.CurrentLoopbaan;
      If Result=Nil then
        Result:=Loopbanen.Lastloopbaan;
      end;
end;

 { TLoopbaanData }

destructor TLoopbaanData.Destroy;
begin
  FreeAndNil(FVakken);
  FreeAndNil(FKlasgenoten);
  FreeAndNil(FKalender);
  FreeAndNil(FRapportPeriodes);
  FreeAndNil(FRapportBestanden);
  inherited Destroy;
end;

Procedure TLoopbaanData.Assign(ASource : TPersistent);

var
  O : TLoopbaanData ;
begin
  If (ASource is TLoopbaanData) then
    begin
    O:=(ASource as TLoopbaanData);
    FStudierichting:=O.FStudierichting;
    FDeelgemeente:=O.FDeelgemeente;
    FPostcode:=O.FPostcode;
    FInstellingID:=O.FInstellingID;
    FInstelling:=O.FInstelling;
    FDatumInschrijving:=O.FDatumInschrijving;
    FDatumUitschrijving:=O.FDatumUitschrijving;
    FKlascode:=O.FKlascode;
    FKlas:=O.FKlas;
    FKlasNr:=O.FKlasNr;
    FTot:=O.FTot;
    FVan:=O.FVan;
    FSchoolID:=O.FSchoolID;
    FSchoolnaam:=O.FSchoolnaam;
    end
  else
    Inherited;
end;

procedure TLoopbaanData.LoadKlasgenoten(ADataset: TDataset);
begin
  If Assigned(FKLasGenoten) then
    FKlasgenoten.Clear
  else
    FKlasgenoten:=TKlasGenoten.Create(TKlasGenoot);
  FKlasGenoten.LoadFromDataset(ADataset);
end;

procedure TLoopbaanData.LoadVakken(ADataset: TDataset);
begin
  If Assigned(FVakken) then
    FVakken.Clear
  else
    FVakken:=TVakken.Create(TVak);
  FVakken.LoadFromDataset(ADataset);
end;

procedure TLoopbaanData.LoadKalender(ADataset: TDataset);
begin
  If not Assigned(FKalender) then
    FKalender:=TKalenderItemList.Create(TKalenderItem)
  else
    FKalender.Clear;
  FKalender.LoadFromDataset(ADataset);
end;

procedure TLoopbaanData.LoadRapportPeriodes(ADataset: TDataset);
begin
  If not Assigned(FRapportPeriodes) then
    FRapportPeriodes:=TRapportPeriodes.Create(TRapportPeriode)
  else
    FRapportPeriodes.Clear;
  FrapportPeriodes.LoadFromDataset(ADataset);
end;

procedure TLoopbaanData.LoadRapportBestanden(ADataset: TDataset);
begin
  If Not Assigned(FRapportBestanden) then
    FRapportBestanden:=TRapportBestanden.Create(TRapportBestand)
  else
    FRapportBestanden.Clear;
  FRapportBestanden.LoadFromDataset(ADataset);
end;


 { TLoopbaanDataMap }

Procedure TLoopbaanDataMap.DoLoad(AObject : TLoopbaanData);

begin
  With AObject do
    begin
    Studierichting:=GetFromField(Self.FAG_OMSCHRIJVING,Studierichting);
    Deelgemeente:=GetFromField(Self.FGM_DEELGEMEENTE,Deelgemeente);
    Postcode:=GetFromField(Self.FGM_POSTCODE,Postcode);
    InstellingID:=GetFromField(Self.FIS_ID,InstellingID);
    Instelling:=GetFromField(Self.FIS_NAAMGEBRUIKER,Instelling);
    DatumInschrijving:=GetFromField(Self.FIU_DATUMINSCHRIJVING,DatumInschrijving);
    DatumUitschrijving:=GetFromField(Self.FIU_DATUMUITSCHRIJVING,DatumUitschrijving);
    Klascode:=GetFromField(Self.FKL_CODE,Klascode);
    Klas:=GetFromField(Self.FKL_OMSCHRIJVING,Klas);
    KlasNr:=GetFromField(Self.FLB_KLASNUMMER,KlasNr);
    Tot:=GetFromField(Self.FLB_TOT,Tot);
    Van:=GetFromField(Self.FLB_VAN,Van);
    SchoolID:=GetFromField(Self.FSC_ID,SchoolID);
    Schoolnaam:=GetFromField(Self.FSC_NAAM,Schoolnaam);
    LoopbaanID:=GetFromField(Self.FLB_ID,LoopbaanID);
    InuitID:=GetFromField(Self.FIU_ID,InuitID);
    KlasID:=GetFromField(Self.FKL_ID,KlasID);
    Graad:=GetFromField(Self.FLB_GRAAD,Graad);
    Leerjaar:=GetFromField(Self.FLB_LEERJAAR,Leerjaar);
    VakkenPakketID:=GetFromField(Self.FLB_VAKKENPAKKET_FK,VakkenPakketID);
    Writeln('Loaded vakkenpakket',VakkenPakketID);
    end;
end;


Procedure TLoopbaanDataMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TLoopbaanData);
end;


Procedure TLoopbaanDataMap.InitFields;

begin
  FAG_OMSCHRIJVING:=FindField('AG_OMSCHRIJVING');
  FGM_DEELGEMEENTE:=FindField('GM_DEELGEMEENTE');
  FGM_POSTCODE:=FindField('GM_POSTCODE');
  FIS_ID:=FindField('IS_ID');
  FLB_ID:=FindField('LB_ID');
  FIU_ID:=FindField('IU_ID');
  FKL_ID:=FindField('KL_ID');
  FIS_NAAMGEBRUIKER:=FindField('IS_NAAMGEBRUIKER');
  FIU_DATUMINSCHRIJVING:=FindField('IU_DATUMINSCHRIJVING');
  FIU_DATUMUITSCHRIJVING:=FindField('IU_DATUMUITSCHRIJVING');
  FKL_CODE:=FindField('KL_CODE');
  FKL_OMSCHRIJVING:=FindField('KL_OMSCHRIJVING');
  FLB_KLASNUMMER:=FindField('LB_KLASNUMMER');
  FLB_TOT:=FindField('LB_TOT');
  FLB_VAN:=FindField('LB_VAN');
  FSC_ID:=FindField('SC_ID');
  FSC_NAAM:=FindField('SC_NAAM');
  FLB_GRAAD:=FindField('LB_GRAAD');
  FLB_LEERJAAR:=FindField('LB_LEERJAAR');
  FLB_VAKKENPAKKET_FK:=FindField('LB_VAKKENPAKKET_FK');
end;

{ TloopbaanDatas }

function TloopbaanDatas.GetLB(AIndex : Integer): TLoopbaanData;
begin
  Result:=TLoopbaanData(Items[AIndex]);
end;

procedure TloopbaanDatas.SetLB(AIndex : Integer; const AValue: TLoopbaanData);
begin
  Items[AIndex]:=AValue;
end;

procedure TloopbaanDatas.CalcLastCurrent;

Var
  I,CID,LID : Integer;
  D : TDateTime;
  LB : TLoopbaanData;

begin
  FLastLoopbaan:=Nil;
  FCurrentLoopbaan:=Nil;
  D:=Date;
  CID:=-1;
  LID:=-1;
  For I:=0 to Count-1 do
    begin
    LB:=Loopbanen[i];
    If (LB.Van<=D) and (D<=LB.Tot) then
      CID:=I;
    If (LB.KlasID>0) then
      LID:=I;
    end;
  If (CID<>-1) then
    FCurrentLoopbaan:=Loopbanen[CID];
  If (LID<>-1) then
    FLastLoopbaan:=Loopbanen[LID];
end;

function TloopbaanDatas.AddLoopbaan(ALBID: Integer): TLoopbaanData;
begin
  Result:=Add as TLoopbaanData;
  Result.LoopbaanID:=ALBID;
end;

function TloopbaanDatas.IndexOfLoopbaan(const ALBID: Integer): Integer;
begin
  Result:=Count-1;
  While (Result>=0) and (GetLB(Result).LoopbaanID<>ALBID) Do
    Dec(Result);
end;

function TloopbaanDatas.LoopbaanByID(const ALBID: Integer): TLoopbaanData;

Var
  I: Integer;

begin
  I:=IndexOfLoopbaan(ALBID);
  If (I=-1) then
    Result:=Nil
  else
    Result:=GetLB(I);
end;

Procedure TloopbaanDatas.LoadFromMap(Map : TFieldMap);


Var
  Obj : TLoopbaanData;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TLoopbaanData;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;

Procedure TloopbaanDatas.LoadFromDataset(Dataset : TDataset);

Var
  Map : TLoopbaanDataMap;

begin
  Map:=TLoopbaanDataMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

 { TKlasgenoot }

Procedure TKlasgenoot.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TKlasgenoot.LoadFromDataset(ADataset : TDataset);

begin
  With TKlasgenootMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TKlasgenoot.Assign(ASource : TPersistent);

var
  O : TKlasgenoot ;
begin
  If (ASource is TKlasgenoot) then
    begin
    O:=(ASource as TKlasgenoot);
    FKlasNR:=O.FKlasNR;
    FLeerlingID:=O.FLeerlingID;
    FNaam:=O.FNaam;
    FVoornaam:=O.FVoornaam;
    end
  else
    Inherited;
end;


 { TKlasgenootMap }

Procedure TKlasgenootMap.DoLoad(AObject : TKlasgenoot);

begin
  With AObject do
    begin
    KlasNR:=GetFromField(Self.FLB_KLASNUMMER,KlasNR);
    LeerlingID:=GetFromField(Self.FLL_ID,LeerlingID);
    Naam:=GetFromField(Self.FLL_NAAM,Naam);
    Voornaam:=GetFromField(Self.FLL_VOORNAAM,Voornaam);
    end;
end;


Procedure TKlasgenootMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TKlasgenoot);
end;


Procedure TKlasgenootMap.InitFields;

begin
  FLB_KLASNUMMER:=FindField('LB_KLASNUMMER');
  FLL_ID:=FindField('LL_ID');
  FLL_NAAM:=FindField('LL_NAAM');
  FLL_VOORNAAM:=FindField('LL_VOORNAAM');
end;


{ TKlasgenoten }
Function TKlasgenoten.GetObj(Index : Integer) : TKlasgenoot;

begin
  Result:=TKlasgenoot(Items[Index]);
end;


Procedure TKlasgenoten.SetObj(Index : Integer; AValue : TKlasgenoot);

begin
  Items[Index]:=AValue;
end;



Procedure TKlasgenoten.LoadFromMap(Map : TFieldMap);


Var
  Obj : TKlasgenoot;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TKlasgenoot;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;



Procedure TKlasgenoten.LoadFromDataset(Dataset : TDataset);


Var
  Map : TKlasgenootMap;

begin
  Map:=TKlasgenootMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

 { TVak }

Procedure TVak.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TVak.LoadFromDataset(ADataset : TDataset);

begin
  With TVakMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TVak.Assign(ASource : TPersistent);

var
  O : TVak ;
begin
  If (ASource is TVak) then
    begin
    O:=(ASource as TVak);
    FLeerkrachtID:=O.FLeerkrachtID;
    FLeerkrachtNaam:=O.FLeerkrachtNaam;
    FLeerkrachtVoornaam:=O.FLeerkrachtVoornaam;
    FVakNaam:=O.FVakNaam;
    end
  else
    Inherited;
end;


 { TVakMap }

Procedure TVakMap.DoLoad(AObject : TVak);

begin
  With AObject do
    begin
    LeerkrachtID:=GetFromField(Self.FLEERKRACHTID,LeerkrachtID);
    LeerkrachtNaam:=GetFromField(Self.FLEERKRACHTNAAM,LeerkrachtNaam);
    LeerkrachtVoornaam:=GetFromField(Self.FLEERKRACHTVOORNAAM,LeerkrachtVoornaam);
    VakNaam:=GetFromField(Self.FVAKNAAM,VakNaam);
    end;
end;


Procedure TVakMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TVak);
end;


Procedure TVakMap.InitFields;

begin
  FLEERKRACHTID:=FindField('LEERKRACHTID');
  FLEERKRACHTNAAM:=FindField('LEERKRACHTNAAM');
  FLEERKRACHTVOORNAAM:=FindField('LEERKRACHTVOORNAAM');
  FVAKNAAM:=FindField('VAKNAAM');
end;


{ TVakken }

function TVakken.GetV(AIndex : integer): TVak;
begin
  Result:=TVak(Items[AIndex]);
end;

procedure TVakken.SetV(AIndex : integer; const AValue: TVak);
begin
  Items[AIndex]:=AValue;
end;

Procedure TVakken.LoadFromMap(Map : TFieldMap);

Var
  Obj : TVak;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TVak;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;

Procedure TVakken.LoadFromDataset(Dataset : TDataset);

Var
  Map : TVakMap;

begin
  Map:=TVakMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

 { TBegeleider }

Procedure TBegeleider.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TBegeleider.LoadFromDataset(ADataset : TDataset);

begin
  With TBegeleiderMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TBegeleider.Assign(ASource : TPersistent);

var
  O : TBegeleider ;
begin
  If (ASource is TBegeleider) then
    begin
    O:=(ASource as TBegeleider);
    FInuitID:=O.FInuitID;
    FLoopbaanID:=O.FLoopbaanID;
    FLeerlingID:=O.FLeerlingID;
    FPersoneelID:=O.FPersoneelID;
    FPersoneelNaam:=O.FPersoneelNaam;
    FPersoneelVoornaam:=O.FPersoneelVoornaam;
    FRolMask:=O.FRolMask;
    end
  else
    Inherited;
end;


 { TBegeleiderMap }

Procedure TBegeleiderMap.DoLoad(AObject : TBegeleider);

begin
  With AObject do
    begin
    InuitID:=GetFromField(Self.FIU_ID,InuitID);
    LoopbaanID:=GetFromField(Self.FLB_ID,LoopbaanID);
    LeerlingID:=GetFromField(Self.FLL_ID,LeerlingID);
    PersoneelID:=GetFromField(Self.FPERSONEELID,PersoneelID);
    PersoneelNaam:=GetFromField(Self.FPERSONEELNAAM,PersoneelNaam);
    PersoneelVoornaam:=GetFromField(Self.FPERSONEELVOORNAAM,PersoneelVoornaam);
    RolMask:=GetFromField(Self.FROLMASK,RolMask);
    end;
end;


Procedure TBegeleiderMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TBegeleider);
end;


Procedure TBegeleiderMap.InitFields;

begin
  FIU_ID:=FindField('IU_ID');
  FLB_ID:=FindField('LB_ID');
  FLL_ID:=FindField('LL_ID');
  FPERSONEELID:=FindField('PERSONEELID');
  FPERSONEELNAAM:=FindField('PERSONEELNAAM');
  FPERSONEELVOORNAAM:=FindField('PERSONEELVOORNAAM');
  FROLMASK:=FindField('ROLMASK');
end;

{ TBegeleiderList }

Function TBegeleiderList.GetObj(Index : Integer) : TBegeleider;

begin
  Result:=TBegeleider(Items[Index]);
end;

Procedure TBegeleiderList.SetObj(Index : Integer; AValue : TBegeleider);

begin
  Items[Index]:=AValue;
end;

Procedure TBegeleiderList.LoadFromMap(Map : TFieldMap);


Var
  Obj : TBegeleider;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TBegeleider;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;

Procedure TBegeleiderList.LoadFromDataset(Dataset : TDataset);

Var
  Map : TBegeleiderMap;

begin
  Map:=TBegeleiderMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;
{ TLVSRollen }

function TLVSRollen.AddRol: TLVSRol;
begin
  Result:=Add as TLVSRol;
end;

function TLVSRollen.GetMaxSecurity(ARollen: Integer): Integer;

Var
  I,S : Integer;

begin
  Result:=0;
  For I:=1 to 16 do
    If ((ARollen and (1 shl I))<>0) then
      begin
      S:=Rollen[i-1].SecurityLevel;
      if (S>Result) then
        Result:=S
      end;
end;

function TLVSRollen.GetR(Index: Integer): TLVSRol;
begin
  Result:=TLVSRol(Items[Index]);
end;

function TLVSRollen.GetRI(Index: Integer): TLVSRol;
begin
  Result:=TLVSRol(Items[Index-1]);
end;

procedure TLVSRollen.LoadFromDataset(Dataset: TDataset);

Var
  Map: TLVSRolFieldMap;

begin
  Map:=TLVSRolFieldMap.Create(Dataset);
  try
    LoadFromMap(Map);
  finally
    Map.Free;
  end;
end;

procedure TLVSRollen.LoadFromMap(Map: TLVSRolFieldMap);

begin
  While not Map.Dataset.EOF do
    begin
    AddRol.LoadFromMap(Map);
    Map.Dataset.Next;
    end;
end;

procedure TLVSRollen.SetR(Index: Integer; const Value: TLVSRol);
begin
  Items[Index]:=Value;
end;

procedure TLVSRollen.SetRI(Index: Integer; const Value: TLVSRol);
begin
  Items[Index-1]:=Value;
end;

{ TLVSRolFieldMap }

procedure TLVSRolFieldMap.InitFields;
begin
  FIDField:=FieldByName('LVR_ID');
  FCodeField:=FieldByName('LVR_CODE');
  FDescrField:=FieldByName('LVR_OMSCHRIJVING');
  FSecField:=FieldByname('LVR_USERNIVEAU')
end;

procedure TLVSRolFieldMap.LoadObject(AObject: TObject);
begin
  // inherited LoadObject(AObject);
end;

{ TLVSRol }

procedure TLVSRol.LoadFromMap(Map: TLVSRolFieldMap);
begin
  With Map do
    begin
    FLVRID:=IDField.asInteger;
    FCode:=CodeField.AsString;
    FDescription:=DescrField.AsString;
    FSecurityLevel:=SecurityLevelField.AsInteger;
    end;
end;

Var
  Rollen : TLVSRollen;

Function LVSRollen : TLVSRollen;

begin
  If not Assigned(Rollen) then
    Rollen:=TLVSRollen.Create(TLVSROl);
  Result:=Rollen;
end;

procedure InitRollen(Dataset: TDataset);
begin
  If not Assigned(Rollen) then
    Rollen:=TLVSRollen.Create(TLVSROl)
  else
    Rollen.Clear;
  Rollen.LoadFromDataset(Dataset);
end;

Function GetRolNamen(RolMask : Integer) : String;

Var
  I : Integer;
  R : TLVSRol;

begin
  Result:='';
  For I:=0 to LVSrollen.Count-1 do
    begin
    R:=LVSrollen[i];
    if (RolMask and (1 shl R.LVRID))<>0 then
      begin
      If (Result<>'') then
        Result:=Result+', ';
      Result:=Result+R.Description;
      end;
    end;
end;

 { TAfwezigheid }

Procedure TAfwezigheid.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TAfwezigheid.LoadFromDataset(ADataset : TDataset);

begin
  With TAfwezigheidMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TAfwezigheid.Assign(ASource : TPersistent);

var
  O : TAfwezigheid ;
begin
  If (ASource is TAfwezigheid) then
    begin
    O:=(ASource as TAfwezigheid);
    FCode:=O.FCode;
    FKleur:=O.FKleur;
    FOmschrijving:=O.FOmschrijving;
    FBriefjevan:=O.FBriefjevan;
    FCommentaar:=O.FCommentaar;
    FAFID:=O.FAFID;
    FReferentie:=O.FReferentie;
    FTotUur:=O.FTotUur;
    FVan:=O.FVan;
    FVanuur:=O.FVanuur;
    end
  else
    Inherited;
end;


 { TAfwezigheidMap }

Procedure TAfwezigheidMap.DoLoad(AObject : TAfwezigheid);

begin
  With AObject do
    begin
    Code:=GetFromField(Self.FAC_CODE,Code);
    Kleur:=GetFromField(Self.FAC_KLEUR,Kleur);
    Omschrijving:=GetFromField(Self.FAC_OMSCHRIJVING,Omschrijving);
    Briefjevan:=GetFromField(Self.FAF_BRIEFJEVAN,Briefjevan);
    Commentaar:=GetFromField(Self.FAF_COMMENTAAR,Commentaar);
    AFID:=GetFromField(Self.FAF_ID,AFID);
    Referentie:=GetFromField(Self.FAF_REFERENTIE,Referentie);
    Tot:=GetFromField(Self.FAF_TOT,Tot);
    TotUur:=GetFromField(Self.FAF_TOTUUR,TotUur);
    Van:=GetFromField(Self.FAF_VAN,Van);
    Vanuur:=GetFromField(Self.FAF_VANUUR,Vanuur);
    end;
end;


Procedure TAfwezigheidMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TAfwezigheid);
end;


Procedure TAfwezigheidMap.InitFields;

begin
  FAC_CODE:=FindField('AC_CODE');
  FAC_KLEUR:=FindField('AC_KLEUR');
  FAC_OMSCHRIJVING:=FindField('AC_OMSCHRIJVING');
  FAF_BRIEFJEVAN:=FindField('AF_BRIEFJEVAN');
  FAF_COMMENTAAR:=FindField('AF_COMMENTAAR');
  FAF_ID:=FindField('AF_ID');
  FAF_REFERENTIE:=FindField('AF_REFERENTIE');
  FAF_TOT:=FindField('AF_TOT');
  FAF_TOTUUR:=FindField('AF_TOTUUR');
  FAF_VAN:=FindField('AF_VAN');
  FAF_VANUUR:=FindField('AF_VANUUR');
end;


{ TAfwezigheden }
Function TAfwezigheden.GetObj(Index : Integer) : TAfwezigheid;

begin
  Result:=TAfwezigheid(Items[Index]);
end;


Procedure TAfwezigheden.SetObj(Index : Integer; AValue : TAfwezigheid);

begin
  Items[Index]:=AValue;
end;



Procedure TAfwezigheden.LoadFromMap(Map : TFieldMap);


Var
  Obj : TAfwezigheid;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TAfwezigheid;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;



Procedure TAfwezigheden.LoadFromDataset(Dataset : TDataset);


Var
  Map : TAfwezigheidMap;

begin
  Map:=TAfwezigheidMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

 { TLeerlingZorg }

Procedure TLeerlingZorg.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TLeerlingZorg.LoadFromDataset(ADataset : TDataset);

begin
  With TLeerlingZorgMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TLeerlingZorg.Assign(ASource : TPersistent);

var
  O : TLeerlingZorg ;
begin
  If (ASource is TLeerlingZorg) then
    begin
    O:=(ASource as TLeerlingZorg);
    FDatum:=O.FDatum;
    FGebied:=O.FGebied;
    FLZID:=O.FLZID;
    FOpmerking:=O.FOpmerking;
    FType:=O.FType;
    FTypeCode:=O.FTypeCode;
    end
  else
    Inherited;
end;


 { TLeerlingZorgMap }

Procedure TLeerlingZorgMap.DoLoad(AObject : TLeerlingZorg);

begin
  With AObject do
    begin
    Datum:=GetFromField(Self.FLZ_DATUM,Datum);
    Gebied:=GetFromField(Self.FLZ_GEBIED,Gebied);
    LZID:=GetFromField(Self.FLZ_ID,LZID);
    Opmerking:=GetFromField(Self.FLZ_OPMERKING,Opmerking);
    ZorgType:=GetFromField(Self.FLZ_TYPE,ZorgType);
    TypeCode:=GetFromField(Self.FLZ_TYPECODE,TypeCode);
    end;
end;


Procedure TLeerlingZorgMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TLeerlingZorg);
end;


Procedure TLeerlingZorgMap.InitFields;

begin
  FLZ_DATUM:=FindField('LZ_DATUM');
  FLZ_GEBIED:=FindField('LZ_GEBIED');
  FLZ_ID:=FindField('LZ_ID');
  FLZ_OPMERKING:=FindField('LZ_OPMERKING');
  FLZ_TYPE:=FindField('LZ_TYPE');
  FLZ_TYPECODE:=FindField('LZ_TYPECODE');
end;

{ TLeerlingZorgList }

Function TLeerlingZorgList.GetObj(Index : Integer) : TLeerlingZorg;

begin
  Result:=TLeerlingZorg(Items[Index]);
end;


Procedure TLeerlingZorgList.SetObj(Index : Integer; AValue : TLeerlingZorg);

begin
  Items[Index]:=AValue;
end;



Procedure TLeerlingZorgList.LoadFromMap(Map : TFieldMap);


Var
  Obj : TLeerlingZorg;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TLeerlingZorg;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;



Procedure TLeerlingZorgList.LoadFromDataset(Dataset : TDataset);

Var
  Map : TLeerlingZorgMap;

begin
  Map:=TLeerlingZorgMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

 { TMedischeFiche }

Procedure TMedischeFiche.Assign(ASource : TPersistent);

var
  O : TMedischeFiche ;
begin
  If (ASource is TMedischeFiche) then
    begin
    O:=(ASource as TMedischeFiche);
    FAndere:=O.FAndere;
    FVlaggen:=O.FVlaggen;
    FGehoor:=O.FGehoor;
    FGehoorsinds:=O.FGehoorsinds;
    FID:=O.FID;
    FMotoriek1:=O.FMotoriek1;
    FMotoriek2:=O.FMotoriek2;
    FMotoriek3:=O.FMotoriek3;
    FMotoriek4:=O.FMotoriek4;
    FSISkaart:=O.FSISkaart;
    FSpraak:=O.FSpraak;
    FSpraakSinds:=O.FSpraakSinds;
    FVrijeParam1:=O.FVrijeParam1;
    FVrijeParam2:=O.FVrijeParam2;
    FVrijveld1:=O.FVrijveld1;
    FVrijVeld2:=O.FVrijVeld2;
    FVrijveld3:=O.FVrijveld3;
    FZicht:=O.FZicht;
    FZichtSinds:=O.FZichtSinds;
    FZiekenfondsnummer:=O.FZiekenfondsnummer;
    end
  else
    Inherited;
end;


 { TMedischeFicheMap }

Procedure TMedischeFicheMap.DoLoad(AObject : TMedischeFiche);

begin
  With AObject do
    begin
    Andere:=GetFromField(Self.FMF_ANDERE,Andere);
    Vlaggen:=GetFromField(Self.FMF_FLAGS,Vlaggen);
    Gehoor:=GetFromField(Self.FMF_GEHOOR,Gehoor);
    Gehoorsinds:=GetFromField(Self.FMF_GEHOORSINDS,Gehoorsinds);
    ID:=GetFromField(Self.FMF_ID,ID);
    Motoriek1:=GetFromField(Self.FMF_MOTORIEK1,Motoriek1);
    Motoriek2:=GetFromField(Self.FMF_MOTORIEK2,Motoriek2);
    Motoriek3:=GetFromField(Self.FMF_MOTORIEK3,Motoriek3);
    Motoriek4:=GetFromField(Self.FMF_MOTORIEK4,Motoriek4);
    SISkaart:=GetFromField(Self.FMF_SISKAART,SISkaart);
    Spraak:=GetFromField(Self.FMF_SPRAAK,Spraak);
    SpraakSinds:=GetFromField(Self.FMF_SPRAAKSINDS,SpraakSinds);
    VrijeParam1:=GetFromField(Self.FMF_VRIJPARAM1,VrijeParam1);
    VrijeParam2:=GetFromField(Self.FMF_VRIJPARAM2,VrijeParam2);
    Vrijveld1:=GetFromField(Self.FMF_VRIJVELD1,Vrijveld1);
    VrijVeld2:=GetFromField(Self.FMF_VRIJVELD2,VrijVeld2);
    Vrijveld3:=GetFromField(Self.FMF_VRIJVELD3,Vrijveld3);
    Zicht:=GetFromField(Self.FMF_ZICHT,Zicht);
    ZichtSinds:=GetFromField(Self.FMF_ZICHTSINDS,ZichtSinds);
    Ziekenfondsnummer:=GetFromField(Self.FMF_ZIEKENFONDSNUMMER,Ziekenfondsnummer);
    end;
end;


Procedure TMedischeFicheMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TMedischeFiche);
end;


Procedure TMedischeFicheMap.InitFields;

begin
  FMF_ANDERE:=FindField('MF_ANDERE');
  FMF_FLAGS:=FindField('MF_FLAGS');
  FMF_GEHOOR:=FindField('MF_GEHOOR');
  FMF_GEHOORSINDS:=FindField('MF_GEHOORSINDS');
  FMF_ID:=FindField('MF_ID');
  FMF_MOTORIEK1:=FindField('MF_MOTORIEK1');
  FMF_MOTORIEK2:=FindField('MF_MOTORIEK2');
  FMF_MOTORIEK3:=FindField('MF_MOTORIEK3');
  FMF_MOTORIEK4:=FindField('MF_MOTORIEK4');
  FMF_SISKAART:=FindField('MF_SISKAART');
  FMF_SPRAAK:=FindField('MF_SPRAAK');
  FMF_SPRAAKSINDS:=FindField('MF_SPRAAKSINDS');
  FMF_VRIJPARAM1:=FindField('MF_VRIJPARAM1');
  FMF_VRIJPARAM2:=FindField('MF_VRIJPARAM2');
  FMF_VRIJVELD1:=FindField('MF_VRIJVELD1');
  FMF_VRIJVELD2:=FindField('MF_VRIJVELD2');
  FMF_VRIJVELD3:=FindField('MF_VRIJVELD3');
  FMF_ZICHT:=FindField('MF_ZICHT');
  FMF_ZICHTSINDS:=FindField('MF_ZICHTSINDS');
  FMF_ZIEKENFONDSNUMMER:=FindField('MF_ZIEKENFONDSNUMMER');
end;


 { TBegeleiding }

Procedure TBegeleiding.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TBegeleiding.LoadFromDataset(ADataset : TDataset);

begin
  With TBegeleidingMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TBegeleiding.Assign(ASource : TPersistent);

var
  O : TBegeleiding ;
begin
  If (ASource is TBegeleiding) then
    begin
    O:=(ASource as TBegeleiding);
    FAchterliggendProbleem:=O.FAchterliggendProbleem;
    FBetrokkenPersoon1:=O.FBetrokkenPersoon1;
    FBetrokkenPersoon2:=O.FBetrokkenPersoon2;
    FBetrokkenPersoon3:=O.FBetrokkenPersoon3;
    FBetrokkenPersoonType1:=O.FBetrokkenPersoonType1;
    FBetrokkenPersoonType2:=O.FBetrokkenPersoonType2;
    FBetrokkenPersoonType3:=O.FBetrokkenPersoonType3;
    FDatum:=O.FDatum;
    FExterneIdentificatie1:=O.FExterneIdentificatie1;
    FExterneIdentificatie2:=O.FExterneIdentificatie2;
    FFinancieleTussenKomst:=O.FFinancieleTussenKomst;
    FVlaggen:=O.FVlaggen;
    FBGID:=O.FBGID;
    FInterventieMotief:=O.FInterventieMotief;
    FOmschrijving:=O.FOmschrijving;
    FOpmerking:=O.FOpmerking;
    FStatus:=O.FStatus;
    FSubtype:=O.FSubtype;
    FSubtypeCode:=O.FSubtypeCode;
    FTijd:=O.FTijd;
    FBegeleidingType:=O.FBegeleidingType;
    FBegeleidingTypeCode:=O.FBegeleidingTypeCode;
    end
  else
    Inherited;
end;


 { TBegeleidingMap }

Procedure TBegeleidingMap.DoLoad(AObject : TBegeleiding);

begin
  With AObject do
    begin
    AchterliggendProbleem:=GetFromField(Self.FBG_ACHTERLIGGENDPROBLEEM,AchterliggendProbleem);
    BetrokkenPersoon1:=GetFromField(Self.FBG_BETROKKENPERSOON1,BetrokkenPersoon1);
    BetrokkenPersoon2:=GetFromField(Self.FBG_BETROKKENPERSOON2,BetrokkenPersoon2);
    BetrokkenPersoon3:=GetFromField(Self.FBG_BETROKKENPERSOON3,BetrokkenPersoon3);
    BetrokkenPersoonType1:=GetFromField(Self.FBG_BETROKKENPERSOONTYPE1,BetrokkenPersoonType1);
    BetrokkenPersoonType2:=GetFromField(Self.FBG_BETROKKENPERSOONTYPE2,BetrokkenPersoonType2);
    BetrokkenPersoonType3:=GetFromField(Self.FBG_BETROKKENPERSOONTYPE3,BetrokkenPersoonType3);
    Datum:=GetFromField(Self.FBG_DATUM,Datum);
    ExterneIdentificatie1:=GetFromField(Self.FBG_EXTERNEIDENTIFICATIE1,ExterneIdentificatie1);
    ExterneIdentificatie2:=GetFromField(Self.FBG_EXTERNEIDENTIFICATIE2,ExterneIdentificatie2);
  //  FinancieleTussenKomst:=GetFromField(Self.FBG_FINANCIELETUSSENKOMST,FinancieleTussenKomst);
    Vlaggen:=GetFromField(Self.FBG_FLAGS,Vlaggen);
    BGID:=GetFromField(Self.FBG_ID,BGID);
    InterventieMotief:=GetFromField(Self.FBG_INTERVENTIEMOTIEF,InterventieMotief);
    Omschrijving:=GetFromField(Self.FBG_OMSCHRIJVING,Omschrijving);
    Opmerking:=GetFromField(Self.FBG_OPMERKING,Opmerking);
    Status:=GetFromField(Self.FBG_STATUS,Status);
    Subtype:=GetFromField(Self.FBG_SUBTYPE,Subtype);
    SubtypeCode:=GetFromField(Self.FBG_SUBTYPECODE,SubtypeCode);
    Tijd:=GetFromField(Self.FBG_TIJD,Tijd);
    BegeleidingType:=GetFromField(Self.FBG_TYPE,BegeleidingType);
    BegeleidingTypeCode:=GetFromField(Self.FBG_TYPECODE,BegeleidingTypeCode);
    end;
end;


Procedure TBegeleidingMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TBegeleiding);
end;


Procedure TBegeleidingMap.InitFields;

begin
  FBG_ACHTERLIGGENDPROBLEEM:=FindField('BG_ACHTERLIGGENDPROBLEEM');
  FBG_BETROKKENPERSOON1:=FindField('BG_BETROKKENPERSOON1');
  FBG_BETROKKENPERSOON2:=FindField('BG_BETROKKENPERSOON2');
  FBG_BETROKKENPERSOON3:=FindField('BG_BETROKKENPERSOON3');
  FBG_BETROKKENPERSOONTYPE1:=FindField('BG_BETROKKENPERSOONTYPE1');
  FBG_BETROKKENPERSOONTYPE2:=FindField('BG_BETROKKENPERSOONTYPE2');
  FBG_BETROKKENPERSOONTYPE3:=FindField('BG_BETROKKENPERSOONTYPE3');
  FBG_DATUM:=FindField('BG_DATUM');
  FBG_EXTERNEIDENTIFICATIE1:=FindField('BG_EXTERNEIDENTIFICATIE1');
  FBG_EXTERNEIDENTIFICATIE2:=FindField('BG_EXTERNEIDENTIFICATIE2');
  FBG_FINANCIELETUSSENKOMST:=FindField('BG_FINANCIELETUSSENKOMST');
  FBG_FLAGS:=FindField('BG_FLAGS');
  FBG_ID:=FindField('BG_ID');
  FBG_INTERVENTIEMOTIEF:=FindField('BG_INTERVENTIEMOTIEF');
  FBG_OMSCHRIJVING:=FindField('BG_OMSCHRIJVING');
  FBG_OPMERKING:=FindField('BG_OPMERKING');
  FBG_STATUS:=FindField('BG_STATUS');
  FBG_SUBTYPE:=FindField('BG_SUBTYPE');
  FBG_SUBTYPECODE:=FindField('BG_SUBTYPECODE');
  FBG_TIJD:=FindField('BG_TIJD');
  FBG_TYPE:=FindField('BG_TYPE');
  FBG_TYPECODE:=FindField('BG_TYPECODE');
end;


{ TBegeleidingen }
Function TBegeleidingen.GetObj(Index : Integer) : TBegeleiding;

begin
  Result:=TBegeleiding(Items[Index]);
end;


Procedure TBegeleidingen.SetObj(Index : Integer; AValue : TBegeleiding);

begin
  Items[Index]:=AValue;
end;



Procedure TBegeleidingen.LoadFromMap(Map : TFieldMap);


Var
  Obj : TBegeleiding;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TBegeleiding;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;

Procedure TBegeleidingen.LoadFromDataset(Dataset : TDataset);


Var
  Map : TBegeleidingMap;

begin
  Map:=TBegeleidingMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;


 { TInterventie }

Procedure TInterventie.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TInterventie.LoadFromDataset(ADataset : TDataset);

begin
  With TInterventieMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TInterventie.Assign(ASource : TPersistent);

var
  O : TInterventie ;
begin
  If (ASource is TInterventie) then
    begin
    O:=(ASource as TInterventie);
    FBetrokkenpersoon:=O.FBetrokkenpersoon;
    FBetrokkenPersoonType:=O.FBetrokkenPersoonType;
    FDatum:=O.FDatum;
    FDerden:=O.FDerden;
    FVlaggen:=O.FVlaggen;
    FLIID:=O.FLIID;
    FMotief:=O.FMotief;
    FMotiefCode:=O.FMotiefCode;
    FNiveau:=O.FNiveau;
    FOmschrijving:=O.FOmschrijving;
    FOpmerking:=O.FOpmerking;
    FPlaats:=O.FPlaats;
    FTijd:=O.FTijd;
    FInterventieType:=O.FInterventieType;
    FTypeCode:=O.FTypeCode;
    end
  else
    Inherited;
end;


 { TInterventieMap }

Procedure TInterventieMap.DoLoad(AObject : TInterventie);

begin
  With AObject do
    begin
    Betrokkenpersoon:=GetFromField(Self.FLI_BETROKKENPERSOON,Betrokkenpersoon);
    BetrokkenPersoonType:=GetFromField(Self.FLI_BETROKKENPERSOONTYPE,BetrokkenPersoonType);
    Datum:=GetFromField(Self.FLI_DATUM,Datum);
    Derden:=GetFromField(Self.FLI_DERDEN,Derden);
    Vlaggen:=GetFromField(Self.FLI_FLAGS,Vlaggen);
    LIID:=GetFromField(Self.FLI_ID,LIID);
    Motief:=GetFromField(Self.FLI_MOTIEF,Motief);
    MotiefCode:=GetFromField(Self.FLI_MOTIEFCODE,MotiefCode);
    Niveau:=GetFromField(Self.FLI_NIVEAU,Niveau);
    Omschrijving:=GetFromField(Self.FLI_OMSCHRIJVING,Omschrijving);
    Opmerking:=GetFromField(Self.FLI_OPMERKING,Opmerking);
    Plaats:=GetFromField(Self.FLI_PLAATS,Plaats);
    Tijd:=GetFromField(Self.FLI_TIJD,Tijd);
    InterventieType:=GetFromField(Self.FLI_TYPE,InterventieType);
    TypeCode:=GetFromField(Self.FLI_TYPECODE,TypeCode);
    BegeleidingID:=GetFromField(FLI_BEGELEIDINGID,-1);
    end;
end;


Procedure TInterventieMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TInterventie);
end;


Procedure TInterventieMap.InitFields;

begin
  FLI_BETROKKENPERSOON:=FindField('LI_BETROKKENPERSOON');
  FLI_BETROKKENPERSOONTYPE:=FindField('LI_BETROKKENPERSOONTYPE');
  FLI_DATUM:=FindField('LI_DATUM');
  FLI_DERDEN:=FindField('LI_DERDEN');
  FLI_FLAGS:=FindField('LI_FLAGS');
  FLI_ID:=FindField('LI_ID');
  FLI_MOTIEF:=FindField('LI_MOTIEF');
  FLI_MOTIEFCODE:=FindField('LI_MOTIEFCODE');
  FLI_NIVEAU:=FindField('LI_NIVEAU');
  FLI_OMSCHRIJVING:=FindField('LI_OMSCHRIJVING');
  FLI_OPMERKING:=FindField('LI_OPMERKING');
  FLI_PLAATS:=FindField('LI_PLAATS');
  FLI_TIJD:=FindField('LI_TIJD');
  FLI_TYPE:=FindField('LI_TYPE');
  FLI_TYPECODE:=FindField('LI_TYPECODE');
  FLI_BEGELEIDINGID:=FindField('LI_BEGELEIDING_FK');
end;

{ TInterventies }

Function TInterventies.GetObj(Index : Integer) : TInterventie;

begin
  Result:=TInterventie(Items[Index]);
end;

Procedure TInterventies.SetObj(Index : Integer; AValue : TInterventie);

begin
  Items[Index]:=AValue;
end;

Procedure TInterventies.LoadFromMap(Map : TFieldMap);


Var
  Obj : TInterventie;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TInterventie;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;

Procedure TInterventies.LoadFromDataset(Dataset : TDataset);

Var
  Map : TInterventieMap;

begin
  Map:=TInterventieMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

 { TKalenderItem }

Procedure TKalenderItem.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TKalenderItem.LoadFromDataset(ADataset : TDataset);

begin
  With TKalenderItemMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TKalenderItem.Assign(ASource : TPersistent);

var
  O : TKalenderItem ;
begin
  If (ASource is TKalenderItem) then
    begin
    O:=(ASource as TKalenderItem);
    FCode:=O.FCode;
    FCodeID:=O.FCodeID;
    FOmschrijving:=O.FOmschrijving;
    FKDID:=O.FKDID;
    FTot:=O.FTot;
    FTotUur:=O.FTotUur;
    FVan:=O.FVan;
    FVanuur:=O.FVanuur;
    end
  else
    Inherited;
end;


 { TKalenderItemMap }

Procedure TKalenderItemMap.DoLoad(AObject : TKalenderItem);

begin
  With AObject do
    begin
    Code:=GetFromField(Self.FAC_CODE,Code);
    CodeID:=GetFromField(Self.FAC_ID,CodeID);
    Omschrijving:=GetFromField(Self.FAC_OMSCHRIJVING,Omschrijving);
    KDID:=GetFromField(Self.FKD_ID,KDID);
    Tot:=GetFromField(Self.FKD_TOT,Tot);
    TotUur:=GetFromField(Self.FKD_TOTUUR,TotUur);
    Van:=GetFromField(Self.FKD_VAN,Van);
    Vanuur:=GetFromField(Self.FKD_VANUUR,Vanuur);
    end;
end;


Procedure TKalenderItemMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TKalenderItem);
end;


Procedure TKalenderItemMap.InitFields;

begin
  FAC_CODE:=FindField('AC_CODE');
  FAC_ID:=FindField('AC_ID');
  FAC_OMSCHRIJVING:=FindField('AC_OMSCHRIJVING');
  FKD_ID:=FindField('KD_ID');
  FKD_TOT:=FindField('KD_TOT');
  FKD_TOTUUR:=FindField('KD_TOTUUR');
  FKD_VAN:=FindField('KD_VAN');
  FKD_VANUUR:=FindField('KD_VANUUR');
end;


{ TKalenderItemList }
Function TKalenderItemList.GetObj(Index : Integer) : TKalenderItem;

begin
  Result:=TKalenderItem(Items[Index]);
end;


Procedure TKalenderItemList.SetObj(Index : Integer; AValue : TKalenderItem);

begin
  Items[Index]:=AValue;
end;

Procedure TKalenderItemList.LoadFromMap(Map : TFieldMap);


Var
  Obj : TKalenderItem;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TKalenderItem;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;

Procedure TKalenderItemList.LoadFromDataset(Dataset : TDataset);

Var
  Map : TKalenderItemMap;

begin
  Map:=TKalenderItemMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

 { TFactuur }

destructor TFactuur.Destroy;
begin
  FreeandNil(Fartikels)
end;

Procedure TFactuur.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TFactuur.LoadFromDataset(ADataset : TDataset);

begin
  With TFactuurMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;

procedure TFactuur.LoadArtikels(ADataset: TDataset);
begin
  If Assigned(FArtikels) then
    FArtikels.Clear
  else
    FArtikels:=TArtikelList.Create(TArtikel);
  Fartikels.LoadFromDataset(ADataset);
end;


Procedure TFactuur.Assign(ASource : TPersistent);

var
  O : TFactuur ;
begin
  If (ASource is TFactuur) then
    begin
    O:=(ASource as TFactuur);
    FBetaald:=O.FBetaald;
    FFactuurDatum:=O.FFactuurDatum;
    FDocumentNr:=O.FDocumentNr;
    FDomiciliering:=O.FDomiciliering;
    FTebetalenTegen:=O.FTebetalenTegen;
    FCDID:=O.FCDID;
    FMemo:=O.FMemo;
    FNo:=O.FNo;
    FOGM:=O.FOGM;
    FTotaal:=O.FTotaal;
    FDagboekCode:=O.FDagboekCode;
    FGedeeldeBetaling:=O.FGedeeldeBetaling;
    FIsBetaald:=O.FIsBetaald;
    FHoofdGroepID:=O.FHoofdGroepID;
    FTypeDetailBoek:=O.FTypeDetailBoek;
    end
  else
    Inherited;
end;


 { TFactuurMap }

Procedure TFactuurMap.DoLoad(AObject : TFactuur);

begin
  With AObject do
    begin
    Betaald:=GetFromField(Self.FBETAALD,Betaald);
    FactuurDatum:=GetFromField(Self.FCD_DATE,FactuurDatum);
    DocumentNr:=GetFromField(Self.FCD_DOCUMENT,DocumentNr);
    Domiciliering:=GetFromField(Self.FCD_DOMICIL,Domiciliering);
    TebetalenTegen:=GetFromField(Self.FCD_DUEDATE,TebetalenTegen);
    CDID:=GetFromField(Self.FCD_ID,CDID);
    Memo:=FCD_MEMO.AsString;
    No:=GetFromField(Self.FCD_NO,No);
    OGM:=GetFromField(Self.FCD_OGM,OGM);
    Totaal:=GetFromField(Self.FCD_SUMTOTAL,Totaal);
    DagboekCode:=GetFromField(Self.FDB_CODE,DagboekCode);
    GedeeldeBetaling:=GetFromField(Self.FDB_SHAREPAYMENT,GedeeldeBetaling);
    IsBetaald:=GetFromField(Self.FISBETAALD,IsBetaald);
    HoofdGroepID:=GetFromField(Self.FMG_ID,HoofdGroepID);
    TypeDetailBoek:=GetFromField(Self.FTYPEDETAILBOEK,TypeDetailBoek);
    end;
end;


Procedure TFactuurMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TFactuur);
end;


Procedure TFactuurMap.InitFields;

begin
  FBETAALD:=FindField('BETAALD');
  FCD_DATE:=FindField('CD_DATE');
  FCD_DOCUMENT:=FindField('CD_DOCUMENT');
  FCD_DOMICIL:=FindField('CD_DOMICIL');
  FCD_DUEDATE:=FindField('CD_DUEDATE');
  FCD_ID:=FindField('CD_ID');
  FCD_MEMO:=FindField('CD_MEMO');
  FCD_NO:=FindField('CD_NO');
  FCD_OGM:=FindField('CD_OGM');
  FCD_SUMTOTAL:=FindField('CD_SUMTOTAL');
  FDB_CODE:=FindField('DB_CODE');
  FDB_SHAREPAYMENT:=FindField('DB_SHAREPAYMENT');
  FISBETAALD:=FindField('ISBETAALD');
  FMG_ID:=FindField('MG_ID');
  FTYPEDETAILBOEK:=FindField('TYPEDETAILBOEK');
end;


{ TFactuurList }
Function TFactuurList.GetObj(Index : Integer) : TFactuur;

begin
  Result:=TFactuur(Items[Index]);
end;


Procedure TFactuurList.SetObj(Index : Integer; AValue : TFactuur);

begin
  Items[Index]:=AValue;
end;



Procedure TFactuurList.LoadFromMap(Map : TFieldMap);


Var
  Obj : TFactuur;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TFactuur;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;



Procedure TFactuurList.LoadFromDataset(Dataset : TDataset);


Var
  Map : TFactuurMap;

begin
  Map:=TFactuurMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

 { TArtikel }

Procedure TArtikel.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TArtikel.LoadFromDataset(ADataset : TDataset);

begin
  With TArtikelMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TArtikel.Assign(ASource : TPersistent);

var
  O : TArtikel ;
begin
  If (ASource is TArtikel) then
    begin
    O:=(ASource as TArtikel);
    FArtikelCode:=O.FArtikelCode;
    FArtikelOmschrijving:=O.FArtikelOmschrijving;
    FBedrag:=O.FBedrag;
    FDatum:=O.FDatum;
    FDetailType:=O.FDetailType;
    FDetailTypeCode:=O.FDetailTypeCode;
    FDetailID:=O.FDetailID;
    FEenheidsPrijs:=O.FEenheidsPrijs;
    FAantal:=O.FAantal;
    FKorting:=O.FKorting;
    FOmschrijving:=O.FOmschrijving;
    end
  else
    Inherited;
end;


 { TArtikelMap }

Procedure TArtikelMap.DoLoad(AObject : TArtikel);

begin
  With AObject do
    begin
    ArtikelCode:=GetFromField(Self.FAR_CODE,ArtikelCode);
    ArtikelOmschrijving:=GetFromField(Self.FAR_DESCRIPTION,ArtikelOmschrijving);
    Bedrag:=GetFromField(Self.FCDD_AMOUNT,Bedrag);
    Datum:=GetFromField(Self.FCDD_DATEDELIVERY,Datum);
    DetailType:=GetFromField(Self.FCDD_DETAILTYPE,DetailType);
    DetailTypeCode:=GetFromField(Self.FCDD_DETAILTYPECODE,DetailTypeCode);
    DetailID:=GetFromField(Self.FCDD_ID,DetailID);
    EenheidsPrijs:=GetFromField(Self.FCDD_PRICE,EenheidsPrijs);
    Aantal:=GetFromField(Self.FCDD_QTY,Aantal);
    Korting:=GetFromField(Self.FCDD_REDUCTION,Korting);
    Omschrijving:=GetFromField(Self.FCDT_DESCRIPTION,Omschrijving);
    end;
end;


Procedure TArtikelMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TArtikel);
end;


Procedure TArtikelMap.InitFields;

begin
  FAR_CODE:=FindField('AR_CODE');
  FAR_DESCRIPTION:=FindField('AR_DESCRIPTION');
  FCDD_AMOUNT:=FindField('CDD_AMOUNT');
  FCDD_DATEDELIVERY:=FindField('CDD_DATEDELIVERY');
  FCDD_DETAILTYPE:=FindField('CDD_DETAILTYPE');
  FCDD_DETAILTYPECODE:=FindField('CDD_DETAILTYPECODE');
  FCDD_ID:=FindField('CDD_ID');
  FCDD_PRICE:=FindField('CDD_PRICE');
  FCDD_QTY:=FindField('CDD_QTY');
  FCDD_REDUCTION:=FindField('CDD_REDUCTION');
  FCDT_DESCRIPTION:=FindField('CDT_DESCRIPTION');
end;


{ TArtikelList }

Function TArtikelList.GetObj(Index : Integer) : TArtikel;

begin
  Result:=TArtikel(Items[Index]);
end;


Procedure TArtikelList.SetObj(Index : Integer; AValue : TArtikel);

begin
  Items[Index]:=AValue;
end;

Procedure TArtikelList.LoadFromMap(Map : TFieldMap);

Var
  Obj : TArtikel;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TArtikel;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;


Procedure TArtikelList.LoadFromDataset(Dataset : TDataset);

Var
  Map : TArtikelMap;

begin
  Map:=TArtikelMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;


 { TRapportPeriode }

Procedure TRapportPeriode.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TRapportPeriode.LoadFromDataset(ADataset : TDataset);

begin
  With TRapportPeriodeMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TRapportPeriode.Assign(ASource : TPersistent);

var
  O : TRapportPeriode ;
begin
  If (ASource is TRapportPeriode) then
    begin
    O:=(ASource as TRapportPeriode);
    FCode:=O.FCode;
    FDatum:=O.FDatum;
    FPEID:=O.FPEID;
    FOmschrijving:=O.FOmschrijving;
    end
  else
    Inherited;
end;


 { TRapportPeriodeMap }

Procedure TRapportPeriodeMap.DoLoad(AObject : TRapportPeriode);

begin
  With AObject do
    begin
    Code:=GetFromField(Self.FPE_CODE,Code);
    Datum:=GetFromField(Self.FPED_DATUM,Datum);
    PEID:=GetFromField(Self.FPE_ID,PEID);
    Omschrijving:=GetFromField(Self.FPE_OMSCHRIJVING,Omschrijving);
    end;
end;


Procedure TRapportPeriodeMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TRapportPeriode);
end;


Procedure TRapportPeriodeMap.InitFields;

begin
  FPE_CODE:=FindField('PE_CODE');
  FPED_DATUM:=FindField('PED_DATUM');
  FPE_ID:=FindField('PE_ID');
  FPE_OMSCHRIJVING:=FindField('PE_OMSCHRIJVING');
end;


{ TRapportPeriodes }
Function TRapportPeriodes.GetObj(Index : Integer) : TRapportPeriode;

begin
  Result:=TRapportPeriode(Items[Index]);
end;


Procedure TRapportPeriodes.SetObj(Index : Integer; AValue : TRapportPeriode);

begin
  Items[Index]:=AValue;
end;



Procedure TRapportPeriodes.LoadFromMap(Map : TFieldMap);


Var
  Obj : TRapportPeriode;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TRapportPeriode;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;

Procedure TRapportPeriodes.LoadFromDataset(Dataset : TDataset);

Var
  Map : TRapportPeriodeMap;

begin
  Map:=TRapportPeriodeMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

 { TRapportBestand }

Procedure TRapportBestand.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TRapportBestand.LoadFromDataset(ADataset : TDataset);

begin
  With TRapportBestandMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TRapportBestand.Assign(ASource : TPersistent);

var
  O : TRapportBestand ;
begin
  If (ASource is TRapportBestand) then
    begin
    O:=(ASource as TRapportBestand);
    FDocumentID:=O.FDocumentID;
    FLaatsteWijziging:=O.FLaatsteWijziging;
    FBestandsNaam:=O.FBestandsNaam;
    FOmschrijving:=O.FOmschrijving;
    FOrigineleNaam:=O.FOrigineleNaam;
    FRef1:=O.FRef1;
    FRef2:=O.FRef2;
    end
  else
    Inherited;
end;


 { TRapportBestandMap }

Procedure TRapportBestandMap.DoLoad(AObject : TRapportBestand);

begin
  With AObject do
    begin
    DocumentID:=GetFromField(Self.FD_ID,DocumentID);
    LaatsteWijziging:=GetFromField(Self.FD_LAATSTEWIJZIGING,LaatsteWijziging);
    BestandsNaam:=GetFromField(Self.FD_NAAM,BestandsNaam);
    Omschrijving:=GetFromField(Self.FD_OMSCHRIJVING,Omschrijving);
    OrigineleNaam:=GetFromField(Self.FD_ORIGINELENAAM,OrigineleNaam);
    Ref1:=GetFromField(Self.FD_REFERENTIE1,Ref1);
    Ref2:=GetFromField(Self.FD_REFERENTIE2,Ref2);
    end;
end;


Procedure TRapportBestandMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TRapportBestand);
end;


Procedure TRapportBestandMap.InitFields;

begin
  FD_ID:=FindField('D_ID');
  FD_LAATSTEWIJZIGING:=FindField('D_LAATSTEWIJZIGING');
  FD_NAAM:=FindField('D_NAAM');
  FD_OMSCHRIJVING:=FindField('D_OMSCHRIJVING');
  FD_ORIGINELENAAM:=FindField('D_ORIGINELENAAM');
  FD_REFERENTIE1:=FindField('D_REFERENTIE1');
  FD_REFERENTIE2:=FindField('D_REFERENTIE2');
end;

{ TRapportBestanden }

Function TRapportBestanden.GetObj(Index : Integer) : TRapportBestand;

begin
  Result:=TRapportBestand(Items[Index]);
end;


Procedure TRapportBestanden.SetObj(Index : Integer; AValue : TRapportBestand);

begin
  Items[Index]:=AValue;
end;

function TRapportBestanden.IndexOfBestand(DocID: Integer): Integer;
begin
  Result:=Count-1;
  While (Result>=0) and (GetObj(Result).DocumentID<>DocID) do
    Dec(Result);
end;

function TRapportBestanden.FindBestand(DocID: Integer): TRapportBestand;

Var
  I : Integer;

begin
  I:=IndexOfBestand(DocID);
  If (I=-1) then
    Result:=Nil
  else
    Result:=GetObj(I);
end;

function TRapportBestanden.FindBestandForPeriode(APeriodeID: integer
  ): TRapportBestand;

Var
  I : Integer;

begin
  I:=Count-1;
  While (I>=0) and (StrToIntDef(GetObj(i).Ref2,-1)<>APeriodeID) do
    Dec(I);
  If (I>=0) then
    Result:=GetObj(i)
  else
    Result:=Nil;
end;

function TRapportBestanden.FindBestandForPeriode(APeriodeCode: String
  ): TRapportBestand;

Var
  I : Integer;

begin
  I:=Count-1;
  While (I>=0) and (CompareText(GetObj(i).Ref1,APeriodeCode)<>0) do
    Dec(I);
  If (I>=0) then
    Result:=GetObj(i)
  else
    Result:=Nil;
end;

Procedure TRapportBestanden.LoadFromMap(Map : TFieldMap);


Var
  Obj : TRapportBestand;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TRapportBestand;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;


Procedure TRapportBestanden.LoadFromDataset(Dataset : TDataset);

Var
  Map : TRapportBestandMap;

begin
  Map:=TRapportBestandMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;



 { TLoginData }

function TLoginData.EmailMatch(AEmail: String; var ARol: TLeerlingRol
  ): Boolean;
begin
  Result:=CompareText(AEmail,VaderEmail)=0;
  If Result then
    ARol:=lrVader
  else
    begin
    Result:=CompareText(AEmail,MoederEmail)=0;
    If Result then
      ARol:=lrMoeder
    else
      begin
      Result:=CompareText(AEmail,VoogdEmail)=0;
      If Result then
        ARol:=lrVoogd;
      end;
    end;
end;

Procedure TLoginData.LoadFromMap(Map : TFieldMap);

begin
  Map.LoadObject(Self);
end;


Procedure TLoginData.LoadFromDataset(ADataset : TDataset);

begin
  With TLoginDataMap.Create(ADataset) do
    try
      LoadObject(Self);
    Finally
      Free;
    end;
end;


Procedure TLoginData.Assign(ASource : TPersistent);

var
  O : TLoginData ;
begin
  If (ASource is TLoginData) then
    begin
    O:=(ASource as TLoginData);
    FLeerlingID:=O.FLeerlingID;
    FMoederEmail:=O.FMoederEmail;
    FMoedernaam:=O.FMoedernaam;
    FMoederUID:=O.FMoederUID;
    FMoederVoornaam:=O.FMoederVoornaam;
    FNaam:=O.FNaam;
    FRijksregisterNummer:=O.FRijksregisterNummer;
    FVaderEmail:=O.FVaderEmail;
    FVadernaam:=O.FVadernaam;
    FVaderUID:=O.FVaderUID;
    FVaderVoornaam:=O.FVaderVoornaam;
    FVoogdEmail:=O.FVoogdEmail;
    FVoogdnaam:=O.FVoogdnaam;
    FVoogdUID:=O.FVoogdUID;
    FVoogdVoornaam:=O.FVoogdVoornaam;
    FVoornaam:=O.FVoornaam;
    end
  else
    Inherited;
end;

function TLoginData.GetRolNaam(ARol: TLeerlingRol): string;
begin
  Case ARol of
   lrLeerling : Result:=Naam;
   lrVader    : Result:=VaderNaam;
   lrMoeder   : Result:=MoederNaam;
   lrVoogd    : Result:=VoogdNaam;
  else
    Raise Exception.CreateFmt('GetRolnaam: Onbekende rol %d',[Ord(ARol)]);
  end;
end;

function TLoginData.GetRolVoornaam(ARol: TLeerlingRol): string;
begin
  Case ARol of
   lrLeerling : Result:=VoorNaam;
   lrVader    : Result:=VaderVoorNaam;
   lrMoeder   : Result:=MoederVoorNaam;
   lrVoogd    : Result:=VoogdVoorNaam;
  else
    Raise Exception.CreateFmt('GetRolvoornaam: Onbekende rol %d',[Ord(ARol)]);
  end;
end;

function TLoginData.GetRolUID(ARoL: TLeerlingRol): integer;
begin
  Case ARol of
   lrLeerling : Result:=-1;
   lrVader    : Result:=VaderUID;
   lrMoeder   : Result:=MoederUID;
   lrVoogd    : Result:=VoogdUID;
  else
    Raise Exception.CreateFmt('GetRolUID: Onbekende rol %d',[Ord(ARol)]);
  end;
end;

function TLoginData.GetRolEmail(ARol: TLeerlingRol): String;
begin
  Case ARol of
   lrLeerling : Result:='';
   lrVader    : Result:=VaderEmail;
   lrMoeder   : Result:=MoederEmail;
   lrVoogd    : Result:=VoogdEmail;
  else
    Raise Exception.CreateFmt('GetRolEmail: Onbekende rol %d',[Ord(ARol)]);
  end;
end;


 { TLoginDataMap }

Procedure TLoginDataMap.DoLoad(AObject : TLoginData);

begin
  With AObject do
    begin
    LeerlingID:=GetFromField(Self.FLL_ID,LeerlingID);
    MoederEmail:=GetFromField(Self.FLL_MOEDEREMAIL,MoederEmail);
    Moedernaam:=GetFromField(Self.FLL_MOEDERNAAM,Moedernaam);
    MoederUID:=GetFromField(Self.FLL_MOEDERUSER_FK,MoederUID);
    MoederVoornaam:=GetFromField(Self.FLL_MOEDERVOORNAAM,MoederVoornaam);
    Naam:=GetFromField(Self.FLL_NAAM,Naam);
    RijksregisterNummer:=GetFromField(Self.FLL_RIJKSREGISTERNR,RijksregisterNummer);
    VaderEmail:=GetFromField(Self.FLL_VADEREMAIL,VaderEmail);
    Vadernaam:=GetFromField(Self.FLL_VADERNAAM,Vadernaam);
    VaderUID:=GetFromField(Self.FLL_VADERUSER_FK,VaderUID);
    VaderVoornaam:=GetFromField(Self.FLL_VADERVOORNAAM,VaderVoornaam);
    VoogdEmail:=GetFromField(Self.FLL_VOOGDEMAIL,VoogdEmail);
    Voogdnaam:=GetFromField(Self.FLL_VOOGDNAAM,Voogdnaam);
    VoogdUID:=GetFromField(Self.FLL_VOOGDUSER_FK,VoogdUID);
    VoogdVoornaam:=GetFromField(Self.FLL_VOOGDVOORNAAM,VoogdVoornaam);
    Voornaam:=GetFromField(Self.FLL_VOORNAAM,Voornaam);
    end;
end;


Procedure TLoginDataMap.LoadObject(AObject : TObject);

begin
  DoLoad(AObject as TLoginData);
end;


Procedure TLoginDataMap.InitFields;

begin
  FLL_ID:=FindField('LL_ID');
  FLL_MOEDEREMAIL:=FindField('LL_MOEDEREMAIL');
  FLL_MOEDERNAAM:=FindField('LL_MOEDERNAAM');
  FLL_MOEDERUSER_FK:=FindField('LL_MOEDERUSER_FK');
  FLL_MOEDERVOORNAAM:=FindField('LL_MOEDERVOORNAAM');
  FLL_NAAM:=FindField('LL_NAAM');
  FLL_RIJKSREGISTERNR:=FindField('LL_RIJKSREGISTERNR');
  FLL_VADEREMAIL:=FindField('LL_VADEREMAIL');
  FLL_VADERNAAM:=FindField('LL_VADERNAAM');
  FLL_VADERUSER_FK:=FindField('LL_VADERUSER_FK');
  FLL_VADERVOORNAAM:=FindField('LL_VADERVOORNAAM');
  FLL_VOOGDEMAIL:=FindField('LL_VOOGDEMAIL');
  FLL_VOOGDNAAM:=FindField('LL_VOOGDNAAM');
  FLL_VOOGDUSER_FK:=FindField('LL_VOOGDUSER_FK');
  FLL_VOOGDVOORNAAM:=FindField('LL_VOOGDVOORNAAM');
  FLL_VOORNAAM:=FindField('LL_VOORNAAM');
end;


{ TLoginDataList }
Function TLoginDataList.GetObj(Index : Integer) : TLoginData;

begin
  Result:=TLoginData(Items[Index]);
end;


Procedure TLoginDataList.SetObj(Index : Integer; AValue : TLoginData);

begin
  Items[Index]:=AValue;
end;



Procedure TLoginDataList.LoadFromMap(Map : TFieldMap);


Var
  Obj : TLoginData;

begin
  With Map do
    While not Dataset.EOF do
      begin
      Obj:=Self.Add as TLoginData;
      Try
        LoadObject(Obj);
      Except
        FreeAndNil(Obj);
        Raise;
      end;
      Dataset.Next;
      end;
end;

Procedure TLoginDataList.LoadFromDataset(Dataset : TDataset);

Var
  Map : TLoginDataMap;

begin
  Map:=TLoginDataMap.Create(Dataset);
  Try
    LoadFromMap(Map);
  Finally
    FreeAndNil(Map);
  end;
end;

function TLoginDataList.GetRRMatch(const APassword: String): TLoginData;

Var
  I : Integer;
  L : TLoginData;
  R  : String;
begin
  Result:=Nil;
  I:=Count-1;
  While (Result=Nil) and (I>=0) do
    begin
    L:=LoginDatas[i];
    R:=L.RijksregisterNummer;
    If (R<>'') then
      begin
      R:=Copy(R,Length(R)-4,5);
      If (L.RijksRegisterNummer=APassword) or (R=APassword) then
        Result:=L;
      end;
    Dec(I);
    end;
end;

function TLoginDataList.GetMatch(const AUserNaam: String; Out ARol: TleerlingRol
  ): TLoginData;

Var
  I : Integer;
  L : TLoginData;
begin
  Result:=Nil;
  // First find match with UID.
  I:=Count-1;
  While (Result=Nil) and (I>=0) do
    begin
    L:=LoginDatas[i];
    If L.EmailMatch(AUserNaam,ARol) then
      If L.GetRolUID(ARol)>=0 then
        Result:=L;
    Dec(I);
    end;
  // Then find match without.
  I:=Count-1;
  While (Result=Nil) and (I>=0) do
    begin
    L:=LoginDatas[i];
    If L.EmailMatch(AUserNaam,ARol) then
      If L.GetRolUID(ARol)>=0 then
        Result:=L;
    Dec(I);
    end;
end;


finalization
  If Assigned(Rollen) then
    FreeAndNil(Rollen);
end.

